@@ -39,13 +39,18 @@ def process_html(fn, lines):
 
 def process_tex(lines):
     """
-    Remove unnecessary section titles from the LaTeX file,
-    and convert UTF-8 non-breaking spaces to Latex nbsps.
-
+    Remove unnecessary section titles from the LaTeX file.
     """
     new_lines = []
     for line in lines:
-        if re.match(r'^\\(section|subsection|subsubsection|paragraph|subparagraph){(numpy|scipy)\.', line):
+        line = re.sub(r'^\s*\\strong{See Also:}\s*$', r'\paragraph{See Also}', line)
+
+        if (line.startswith(r'\section{scipy.')
+            or line.startswith(r'\subsection{scipy.')
+            or line.startswith(r'\subsubsection{scipy.')
+            or line.startswith(r'\paragraph{scipy.')
+            or line.startswith(r'\subparagraph{scipy.')
+            ):
             pass # skip!
         else:
             new_lines.append(line)
