@@ -2,7 +2,7 @@ import numpy as np
 import matplotlib.pyplot as plt
 from scipy import stats
 
-x1 = np.array([-7, -5, 1, 4, 5], dtype=np.float)
+x1 = np.array([-7, -5, 1, 4, 5], dtype=float)
 x_eval = np.linspace(-10, 10, num=200)
 kde1 = stats.gaussian_kde(x1)
 kde2 = stats.gaussian_kde(x1, bw_method='silverman')
