@@ -1,13 +1,17 @@
-from distutils.core import setup
-import setuptools
-import sys, os
+from __future__ import division, print_function
 
-version = "0.3.dev"
+import sys
+import setuptools
+from distutils.core import setup
+
+if sys.version_info[:2] < (2, 6) or (3, 0) <= sys.version_info[0:2] < (3, 3):
+    raise RuntimeError("Python version 2.6, 2.7 or >= 3.3 required.")
+
+version = "0.6.dev"
 
 setup(
     name="numpydoc",
     packages=["numpydoc"],
-    package_dir={"numpydoc": ""},
     version=version,
     description="Sphinx extension to support docstrings in Numpy format",
     # classifiers from http://pypi.python.org/pypi?%3Aaction=list_classifiers
@@ -18,14 +22,9 @@ setup(
     keywords="sphinx numpy",
     author="Pauli Virtanen and others",
     author_email="pav@iki.fi",
-    url="http://projects.scipy.org/numpy/browser/trunk/doc/sphinxext",
+    url="https://github.com/numpy/numpy/blob/master/doc/HOWTO_DOCUMENT.rst.txt",
     license="BSD",
-    zip_safe=False,
-    install_requires=["Sphinx >= 0.5"],
-    package_data={'numpydoc': 'tests', '': ''},
-    entry_points={
-        "console_scripts": [
-            "autosummary_generate = numpydoc.autosummary_generate:main",
-        ],
-    },
+    requires=["sphinx (>= 1.0.1)"],
+    package_data={'numpydoc': ['tests/test_*.py']},
+    test_suite = 'nose.collector',
 )
