@@ -3,7 +3,7 @@ SciPy: A scientific computing package for Python
 ================================================
 
 Documentation is available in the docstrings and
-online at http://docs.scipy.org.
+online at https://docs.scipy.org.
 
 Contents
 --------
@@ -22,9 +22,9 @@ Using any of these subpackages requires an explicit import.  For example,
  integrate                    --- Integration routines
  interpolate                  --- Interpolation Tools
  io                           --- Data input and output
- lib                          --- Python wrappers to external libraries
- lib.lapack                   --- Wrappers to LAPACK library
  linalg                       --- Linear algebra routines
+ linalg.blas                  --- Wrappers to BLAS library
+ linalg.lapack                --- Wrappers to LAPACK library
  misc                         --- Various utilities that don't have
                                   another home.
  ndimage                      --- n-dimensional image package
@@ -36,20 +36,11 @@ Using any of these subpackages requires an explicit import.  For example,
  sparse.linalg.dsolve         --- Linear Solvers
  sparse.linalg.dsolve.umfpack --- :Interface to the UMFPACK library:
                                   Conjugate Gradient Method (LOBPCG)
+ sparse.linalg.eigen          --- Sparse Eigenvalue Solvers
  sparse.linalg.eigen.lobpcg   --- Locally Optimal Block Preconditioned
-                                  Conjugate Gradient Method (LOBPCG) [*]
- special                      --- Airy Functions [*]
- lib.blas                     --- Wrappers to BLAS library [*]
- sparse.linalg.eigen          --- Sparse Eigenvalue Solvers [*]
- stats                        --- Statistical Functions [*]
- lib                          --- Python wrappers to external libraries
-                                  [*]
- lib.lapack                   --- Wrappers to LAPACK library [*]
- integrate                    --- Integration routines [*]
- ndimage                      --- n-dimensional image package [*]
- linalg                       --- Linear algebra routines [*]
+                                  Conjugate Gradient Method (LOBPCG)
  spatial                      --- Spatial data structures and algorithms
- special                      --- Airy Functions
+ special                      --- Special functions
  stats                        --- Statistical Functions
 
 Utility tools
@@ -74,22 +65,13 @@ from numpy import __version__ as __numpy_version__
 
 # Import numpy symbols to scipy name space
 import numpy as _num
-from numpy import oldnumeric
+linalg = None
 from numpy import *
 from numpy.random import rand, randn
 from numpy.fft import fft, ifft
 from numpy.lib.scimath import *
 
-# Emit a warning if numpy is too old
-majver, minver = [float(i) for i in _num.version.version.split('.')[:2]]
-if majver < 1 or (majver == 1 and minver < 5):
-    import warnings
-    warnings.warn("Numpy 1.5.0 or above is recommended for this version of "
-                  "scipy (detected version %s)" % _num.version.version,
-                  UserWarning)
-
-__all__ += ['oldnumeric']+_num.__all__
-
+__all__ += _num.__all__
 __all__ += ['randn', 'rand', 'fft', 'ifft']
 
 del _num
@@ -118,8 +100,33 @@ else:
         being in scipy source directory; please exit the scipy source
         tree first, and relaunch your python intepreter."""
         raise ImportError(msg)
+
     from scipy.version import version as __version__
+    from scipy._lib._version import NumpyVersion as _NumpyVersion
+    if _NumpyVersion(__numpy_version__) < '1.8.2':
+        import warnings
+        warnings.warn("Numpy 1.8.2 or above is recommended for this version of "
+                      "scipy (detected version %s)" % __numpy_version__,
+                      UserWarning)
+
+    del _NumpyVersion
+
+    from scipy._lib._ccallback import LowLevelCallable
 
     from numpy.testing import Tester
-    test = Tester().test
-    bench = Tester().bench
+
+    def test(*a, **kw):
+        # Nose never recurses into directories with underscores prefix, so we
+        # need to list those explicitly. Note that numpy.testing.Tester inserts
+        # the top-level package path determined from __file__ to argv unconditionally,
+        # so we only need to add the part that is not otherwise recursed into.
+        import os
+        underscore_modules = ['_lib', '_build_utils']
+        base_dir = os.path.abspath(os.path.dirname(__file__))
+        underscore_paths = [os.path.join(base_dir, name) for name in underscore_modules]
+        kw['extra_argv'] = list(kw.get('extra_argv', [])) + underscore_paths
+        return test._tester.test(*a, **kw)
+
+    test._tester = Tester()
+    test.__doc__ = test._tester.test.__doc__
+    test.__test__ = False  # Prevent nose from treating test() as a test
