@@ -55,10 +55,17 @@ def _is_safe_size(n):
     Composite numbers of 2, 3, and 5 are accepted, as FFTPACK has those
     """
     n = int(n)
-    for c in (2, 3, 5):
+
+    if n == 0:
+        return True
+
+    # Divide by 3 until you can't, then by 5 until you can't
+    for c in (3, 5):
         while n % c == 0:
-            n /= c
-    return (n <= 1)
+            n //= c
+
+    # Return True if the remainder is a power of 2
+    return not n & (n-1)
 
 
 def _fake_crfft(x, n, *a, **kw):
@@ -118,12 +125,20 @@ def _asfarray(x):
     already an array with a float dtype, and do not cast complex types to
     real."""
     if hasattr(x, "dtype") and x.dtype.char in numpy.typecodes["AllFloat"]:
-        return x
+        # 'dtype' attribute does not ensure that the
+        # object is an ndarray (e.g. Series class
+        # from the pandas library)
+        if x.dtype == numpy.half:
+            # no half-precision routines, so convert to single precision
+            return numpy.asarray(x, dtype=numpy.float32)
+        return numpy.asarray(x, dtype=x.dtype)
     else:
         # We cannot use asfarray directly because it converts sequences of
         # complex to sequence of real
         ret = numpy.asarray(x)
-        if not ret.dtype.char in numpy.typecodes["AllFloat"]:
+        if ret.dtype == numpy.half:
+            return numpy.asarray(ret, dtype=numpy.float32)
+        elif ret.dtype.char not in numpy.typecodes["AllFloat"]:
             return numpy.asfarray(x)
         return ret
 
@@ -152,6 +167,11 @@ def _raw_fft(x, n, axis, direction, overwrite_x, work_function):
     elif n != x.shape[axis]:
         x, copy_made = _fix_shape(x,n,axis)
         overwrite_x = overwrite_x or copy_made
+
+    if n < 1:
+        raise ValueError("Invalid number of FFT data points "
+                         "(%d) specified." % n)
+
     if axis == -1 or axis == len(x.shape)-1:
         r = work_function(x,n,direction,overwrite_x=overwrite_x)
     else:
@@ -161,7 +181,7 @@ def _raw_fft(x, n, axis, direction, overwrite_x, work_function):
     return r
 
 
-def fft(x, n=None, axis=-1, overwrite_x=0):
+def fft(x, n=None, axis=-1, overwrite_x=False):
     """
     Return discrete Fourier transform of real or complex sequence.
 
@@ -181,7 +201,7 @@ def fft(x, n=None, axis=-1, overwrite_x=0):
         Axis along which the fft's are computed; the default is over the
         last axis (i.e., ``axis=-1``).
     overwrite_x : bool, optional
-        If True the contents of `x` can be destroyed; the default is False.
+        If True, the contents of `x` can be destroyed; the default is False.
 
     Returns
     -------
@@ -216,7 +236,20 @@ def fft(x, n=None, axis=-1, overwrite_x=0):
     negative-frequency terms.  For `n` even and `x` real, ``A[n/2]`` will
     always be real.
 
-    This function is most efficient when `n` is a power of two.
+    Both single and double precision routines are implemented.  Half precision
+    inputs will be converted to single precision.  Non floating-point inputs
+    will be converted to double precision.  Long-double precision inputs are
+    not supported.
+
+    This function is most efficient when `n` is a power of two, and least
+    efficient when `n` is prime.
+
+    If the data type of `x` is real, a "real FFT" algorithm is automatically
+    used, which roughly halves the computation time.  To increase efficiency
+    a little further, use `rfft`, which does the same calculation, but only
+    outputs half of the symmetrical spectrum.  If the data is both real and
+    symmetrical, the `dct` can again double the efficiency, by generating
+    half of the spectrum from half of the signal.
 
     Examples
     --------
@@ -244,6 +277,10 @@ def fft(x, n=None, axis=-1, overwrite_x=0):
         tmp, copy_made = _fix_shape(tmp,n,axis)
         overwrite_x = overwrite_x or copy_made
 
+    if n < 1:
+        raise ValueError("Invalid number of FFT data points "
+                         "(%d) specified." % n)
+
     if axis == -1 or axis == len(tmp.shape) - 1:
         return work_function(tmp,n,1,0,overwrite_x)
 
@@ -252,7 +289,7 @@ def fft(x, n=None, axis=-1, overwrite_x=0):
     return swapaxes(tmp, axis, -1)
 
 
-def ifft(x, n=None, axis=-1, overwrite_x=0):
+def ifft(x, n=None, axis=-1, overwrite_x=False):
     """
     Return discrete inverse Fourier transform of real or complex sequence.
 
@@ -272,7 +309,29 @@ def ifft(x, n=None, axis=-1, overwrite_x=0):
         Axis along which the ifft's are computed; the default is over the
         last axis (i.e., ``axis=-1``).
     overwrite_x : bool, optional
-        If True the contents of `x` can be destroyed; the default is False.
+        If True, the contents of `x` can be destroyed; the default is False.
+
+    Returns
+    -------
+    ifft : ndarray of floats
+        The inverse discrete Fourier transform.
+
+    See Also
+    --------
+    fft : Forward FFT
+
+    Notes
+    -----
+    Both single and double precision routines are implemented.  Half precision
+    inputs will be converted to single precision.  Non floating-point inputs
+    will be converted to double precision.  Long-double precision inputs are
+    not supported.
+
+    This function is most efficient when `n` is a power of two, and least
+    efficient when `n` is prime.
+
+    If the data type of `x` is real, a "real IFFT" algorithm is automatically
+    used, which roughly halves the computation time.
 
     """
     tmp = _asfarray(x)
@@ -293,6 +352,10 @@ def ifft(x, n=None, axis=-1, overwrite_x=0):
         tmp, copy_made = _fix_shape(tmp,n,axis)
         overwrite_x = overwrite_x or copy_made
 
+    if n < 1:
+        raise ValueError("Invalid number of FFT data points "
+                         "(%d) specified." % n)
+
     if axis == -1 or axis == len(tmp.shape) - 1:
         return work_function(tmp,n,-1,1,overwrite_x)
 
@@ -301,23 +364,10 @@ def ifft(x, n=None, axis=-1, overwrite_x=0):
     return swapaxes(tmp, axis, -1)
 
 
-def rfft(x, n=None, axis=-1, overwrite_x=0):
+def rfft(x, n=None, axis=-1, overwrite_x=False):
     """
     Discrete Fourier transform of a real sequence.
 
-    The returned real arrays contains::
-
-      [y(0),Re(y(1)),Im(y(1)),...,Re(y(n/2))]              if n is even
-      [y(0),Re(y(1)),Im(y(1)),...,Re(y(n/2)),Im(y(n/2))]   if n is odd
-
-    where
-    ::
-
-      y(j) = sum[k=0..n-1] x[k] * exp(-sqrt(-1)*j*k*2*pi/n)
-      j = 0..n-1
-
-    Note that ``y(-j) == y(n-j).conjugate()``.
-
     Parameters
     ----------
     x : array_like, real-valued
@@ -333,6 +383,21 @@ def rfft(x, n=None, axis=-1, overwrite_x=0):
         If set to true, the contents of `x` can be overwritten. Default is
         False.
 
+    Returns
+    -------
+    z : real ndarray
+        The returned real array contains::
+
+          [y(0),Re(y(1)),Im(y(1)),...,Re(y(n/2))]              if n is even
+          [y(0),Re(y(1)),Im(y(1)),...,Re(y(n/2)),Im(y(n/2))]   if n is odd
+
+        where::
+
+          y(j) = sum[k=0..n-1] x[k] * exp(-sqrt(-1)*j*k*2*pi/n)
+          j = 0..n-1
+
+        Note that ``y(-j) == y(n-j).conjugate()``.
+
     See Also
     --------
     fft, irfft, scipy.fftpack.basic
@@ -341,6 +406,20 @@ def rfft(x, n=None, axis=-1, overwrite_x=0):
     -----
     Within numerical accuracy, ``y == rfft(irfft(y))``.
 
+    Both single and double precision routines are implemented.  Half precision
+    inputs will be converted to single precision.  Non floating-point inputs
+    will be converted to double precision.  Long-double precision inputs are
+    not supported.
+
+    Examples
+    --------
+    >>> from scipy.fftpack import fft, rfft
+    >>> a = [9, -9, 1, 3]
+    >>> fft(a)
+    array([  4. +0.j,   8.+12.j,  16. +0.j,   8.-12.j])
+    >>> rfft(a)
+    array([  4.,   8.,  12.,  16.])
+
     """
     tmp = _asfarray(x)
 
@@ -357,11 +436,11 @@ def rfft(x, n=None, axis=-1, overwrite_x=0):
     return _raw_fft(tmp,n,axis,1,overwrite_x,work_function)
 
 
-def irfft(x, n=None, axis=-1, overwrite_x=0):
+def irfft(x, n=None, axis=-1, overwrite_x=False):
     """
     Return inverse discrete Fourier transform of real sequence x.
 
-    The contents of x is interpreted as the output of the ``rfft(..)``
+    The contents of `x` are interpreted as the output of the `rfft`
     function.
 
     Parameters
@@ -377,7 +456,7 @@ def irfft(x, n=None, axis=-1, overwrite_x=0):
         Axis along which the ifft's are computed; the default is over
         the last axis (i.e., axis=-1).
     overwrite_x : bool, optional
-        If True the contents of `x` can be destroyed; the default is False.
+        If True, the contents of `x` can be destroyed; the default is False.
 
     Returns
     -------
@@ -406,7 +485,7 @@ def irfft(x, n=None, axis=-1, overwrite_x=0):
                                      * exp(sqrt(-1)*j*k* 2*pi/n)
                     + c.c. + x[0])
 
-    c.c. denotes complex conjugate of preceeding expression.
+    c.c. denotes complex conjugate of preceding expression.
 
     For details on input parameters, see `rfft`.
 
@@ -445,7 +524,8 @@ def _raw_fftnd(x, s, axes, direction, overwrite_x, work_function):
 
     for dim in s:
         if dim < 1:
-            raise ValueError("Invalid number of FFT data points (%d) specified." % s)
+            raise ValueError("Invalid number of FFT data points "
+                             "(%s) specified." % (s,))
 
     # No need to swap axes, array is in C order
     if noaxes:
@@ -487,7 +567,7 @@ def _raw_fftnd(x, s, axes, direction, overwrite_x, work_function):
     return r
 
 
-def fftn(x, shape=None, axes=None, overwrite_x=0):
+def fftn(x, shape=None, axes=None, overwrite_x=False):
     """
     Return multidimensional discrete Fourier transform.
 
@@ -525,8 +605,16 @@ def fftn(x, shape=None, axes=None, overwrite_x=0):
     --------
     ifftn
 
+    Notes
+    -----
+    Both single and double precision routines are implemented.  Half precision
+    inputs will be converted to single precision.  Non floating-point inputs
+    will be converted to double precision.  Long-double precision inputs are
+    not supported.
+
     Examples
     --------
+    >>> from scipy.fftpack import fftn, ifftn
     >>> y = (-np.arange(16), 8 - np.arange(16), np.arange(16))
     >>> np.allclose(y, fftn(ifftn(y)))
     True
@@ -550,7 +638,7 @@ def _raw_fftn_dispatch(x, shape, axes, overwrite_x, direction):
     return _raw_fftnd(tmp,shape,axes,direction,overwrite_x,work_function)
 
 
-def ifftn(x, shape=None, axes=None, overwrite_x=0):
+def ifftn(x, shape=None, axes=None, overwrite_x=False):
     """
     Return inverse multi-dimensional discrete Fourier transform of
     arbitrary type sequence x.
@@ -572,7 +660,7 @@ def ifftn(x, shape=None, axes=None, overwrite_x=0):
     return _raw_fftn_dispatch(x, shape, axes, overwrite_x, -1)
 
 
-def fft2(x, shape=None, axes=(-2,-1), overwrite_x=0):
+def fft2(x, shape=None, axes=(-2,-1), overwrite_x=False):
     """
     2-D discrete Fourier transform.
 
@@ -587,7 +675,7 @@ def fft2(x, shape=None, axes=(-2,-1), overwrite_x=0):
     return fftn(x,shape,axes,overwrite_x)
 
 
-def ifft2(x, shape=None, axes=(-2,-1), overwrite_x=0):
+def ifft2(x, shape=None, axes=(-2,-1), overwrite_x=False):
     """
     2-D discrete inverse Fourier transform of real or complex sequence.
 
