@@ -27,44 +27,47 @@
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 # DEALINGS IN THE SOFTWARE.
 
+from __future__ import division, print_function, absolute_import
+
 import struct
 import numpy as np
-from numpy.compat import asbytes, asstr
+from numpy.compat import asstr
 import tempfile
 import zlib
 import warnings
 
 # Define the different data types that can be found in an IDL save file
-DTYPE_DICT = {}
-DTYPE_DICT[1] = '>u1'
-DTYPE_DICT[2] = '>i2'
-DTYPE_DICT[3] = '>i4'
-DTYPE_DICT[4] = '>f4'
-DTYPE_DICT[5] = '>f8'
-DTYPE_DICT[6] = '>c8'
-DTYPE_DICT[7] = '|O'
-DTYPE_DICT[8] = '|O'
-DTYPE_DICT[9] = '>c16'
-DTYPE_DICT[12] = '>u2'
-DTYPE_DICT[13] = '>u4'
-DTYPE_DICT[14] = '>i8'
-DTYPE_DICT[15] = '>u8'
+DTYPE_DICT = {1: '>u1',
+              2: '>i2',
+              3: '>i4',
+              4: '>f4',
+              5: '>f8',
+              6: '>c8',
+              7: '|O',
+              8: '|O',
+              9: '>c16',
+              10: '|O',
+              11: '|O',
+              12: '>u2',
+              13: '>u4',
+              14: '>i8',
+              15: '>u8'}
 
 # Define the different record types that can be found in an IDL save file
-RECTYPE_DICT = {}
-RECTYPE_DICT[0] = "START_MARKER"
-RECTYPE_DICT[1] = "COMMON_VARIABLE"
-RECTYPE_DICT[2] = "VARIABLE"
-RECTYPE_DICT[3] = "SYSTEM_VARIABLE"
-RECTYPE_DICT[6] = "END_MARKER"
-RECTYPE_DICT[10] = "TIMESTAMP"
-RECTYPE_DICT[12] = "COMPILED"
-RECTYPE_DICT[13] = "IDENTIFICATION"
-RECTYPE_DICT[14] = "VERSION"
-RECTYPE_DICT[15] = "HEAP_HEADER"
-RECTYPE_DICT[16] = "HEAP_DATA"
-RECTYPE_DICT[17] = "PROMOTE64"
-RECTYPE_DICT[19] = "NOTICE"
+RECTYPE_DICT = {0: "START_MARKER",
+                1: "COMMON_VARIABLE",
+                2: "VARIABLE",
+                3: "SYSTEM_VARIABLE",
+                6: "END_MARKER",
+                10: "TIMESTAMP",
+                12: "COMPILED",
+                13: "IDENTIFICATION",
+                14: "VERSION",
+                15: "HEAP_HEADER",
+                16: "HEAP_DATA",
+                17: "PROMOTE64",
+                19: "NOTICE",
+                20: "DESCRIPTION"}
 
 # Define a dictionary to contain structure definitions
 STRUCT_DICT = {}
@@ -74,7 +77,7 @@ def _align_32(f):
     '''Align to the next 32-bit position in a file'''
 
     pos = f.tell()
-    if pos % 4 <> 0:
+    if pos % 4 != 0:
         f.seek(pos + 4 - pos % 4)
     return
 
@@ -148,6 +151,11 @@ class Pointer(object):
         return
 
 
+class ObjectPointer(Pointer):
+    '''Class used to define object pointers'''
+    pass
+
+
 def _read_string(f):
     '''Read a string'''
     length = _read_long(f)
@@ -156,7 +164,7 @@ def _read_string(f):
         _align_32(f)
         chars = asstr(chars)
     else:
-        chars = None
+        chars = ''
     return chars
 
 
@@ -165,50 +173,50 @@ def _read_string_data(f):
     length = _read_long(f)
     if length > 0:
         length = _read_long(f)
-        string = _read_bytes(f, length)
+        string_data = _read_bytes(f, length)
         _align_32(f)
     else:
-        string = None
-    return string
+        string_data = ''
+    return string_data
 
 
 def _read_data(f, dtype):
     '''Read a variable with a specified data type'''
-    if dtype==1:
-        if _read_int32(f) <> 1:
-            raise Exception("Error occured while reading byte variable")
+    if dtype == 1:
+        if _read_int32(f) != 1:
+            raise Exception("Error occurred while reading byte variable")
         return _read_byte(f)
-    elif dtype==2:
+    elif dtype == 2:
         return _read_int16(f)
-    elif dtype==3:
+    elif dtype == 3:
         return _read_int32(f)
-    elif dtype==4:
+    elif dtype == 4:
         return _read_float32(f)
-    elif dtype==5:
+    elif dtype == 5:
         return _read_float64(f)
-    elif dtype==6:
+    elif dtype == 6:
         real = _read_float32(f)
         imag = _read_float32(f)
         return np.complex64(real + imag * 1j)
-    elif dtype==7:
+    elif dtype == 7:
         return _read_string_data(f)
-    elif dtype==8:
+    elif dtype == 8:
         raise Exception("Should not be here - please report this")
-    elif dtype==9:
+    elif dtype == 9:
         real = _read_float64(f)
         imag = _read_float64(f)
         return np.complex128(real + imag * 1j)
-    elif dtype==10:
+    elif dtype == 10:
         return Pointer(_read_int32(f))
-    elif dtype==11:
-        raise Exception("Object reference type not implemented")
-    elif dtype==12:
+    elif dtype == 11:
+        return ObjectPointer(_read_int32(f))
+    elif dtype == 12:
         return _read_uint16(f)
-    elif dtype==13:
+    elif dtype == 13:
         return _read_uint32(f)
-    elif dtype==14:
+    elif dtype == 14:
         return _read_int64(f)
-    elif dtype==15:
+    elif dtype == 15:
         return _read_uint64(f)
     else:
         raise Exception("Unknown IDL type: %i - please report this" % dtype)
@@ -221,7 +229,6 @@ def _read_structure(f, array_desc, struct_desc):
     '''
 
     nrows = array_desc['nelements']
-    ncols = struct_desc['ntags']
     columns = struct_desc['tagtable']
 
     dtype = []
@@ -242,15 +249,21 @@ def _read_structure(f, array_desc, struct_desc):
         for col in columns:
             dtype = col['typecode']
             if col['structure']:
-                structure[col['name']][i] = _read_structure(f, \
-                                      struct_desc['arrtable'][col['name']], \
+                structure[col['name']][i] = _read_structure(f,
+                                      struct_desc['arrtable'][col['name']],
                                       struct_desc['structtable'][col['name']])
             elif col['array']:
-                structure[col['name']][i] = _read_array(f, dtype, \
+                structure[col['name']][i] = _read_array(f, dtype,
                                       struct_desc['arrtable'][col['name']])
             else:
                 structure[col['name']][i] = _read_data(f, dtype)
 
+    # Reshape structure if needed
+    if array_desc['ndims'] > 1:
+        dims = array_desc['dims'][:int(array_desc['ndims'])]
+        dims.reverse()
+        structure = structure.reshape(dims)
+
     return structure
 
 
@@ -264,18 +277,18 @@ def _read_array(f, typecode, array_desc):
 
         if typecode == 1:
             nbytes = _read_int32(f)
-            if nbytes <> array_desc['nbytes']:
-                raise Exception("Error occured while reading byte array")
+            if nbytes != array_desc['nbytes']:
+                warnings.warn("Not able to verify number of bytes from header")
 
         # Read bytes as numpy array
-        array = np.fromstring(f.read(array_desc['nbytes']), \
+        array = np.fromstring(f.read(array_desc['nbytes']),
                                 dtype=DTYPE_DICT[typecode])
 
     elif typecode in [2, 12]:
 
         # These are 2 byte types, need to skip every two as they are not packed
 
-        array = np.fromstring(f.read(array_desc['nbytes']*2), \
+        array = np.fromstring(f.read(array_desc['nbytes']*2),
                                 dtype=DTYPE_DICT[typecode])[1::2]
 
     else:
@@ -304,10 +317,7 @@ def _read_array(f, typecode, array_desc):
 def _read_record(f):
     '''Function to read in a full record'''
 
-    record = {}
-
-    recpos = f.tell()
-    record['rectype'] = _read_long(f)
+    record = {'rectype': _read_long(f)}
 
     nextrec = _read_uint32(f)
     nextrec += _read_uint32(f) * 2**32
@@ -329,19 +339,28 @@ def _read_record(f):
 
         rectypedesc = _read_typedesc(f)
 
-        varstart = _read_long(f)
-        if varstart <> 7:
-            raise Exception("VARSTART is not 7")
+        if rectypedesc['typecode'] == 0:
+
+            if nextrec == f.tell():
+                record['data'] = None  # Indicates NULL value
+            else:
+                raise ValueError("Unexpected type code: 0")
 
-        if rectypedesc['structure']:
-            record['data'] = _read_structure(f, rectypedesc['array_desc'], \
-                                          rectypedesc['struct_desc'])
-        elif rectypedesc['array']:
-            record['data'] = _read_array(f, rectypedesc['typecode'], \
-                                      rectypedesc['array_desc'])
         else:
-            dtype = rectypedesc['typecode']
-            record['data'] = _read_data(f, dtype)
+
+            varstart = _read_long(f)
+            if varstart != 7:
+                raise Exception("VARSTART is not 7")
+
+            if rectypedesc['structure']:
+                record['data'] = _read_structure(f, rectypedesc['array_desc'],
+                                                    rectypedesc['struct_desc'])
+            elif rectypedesc['array']:
+                record['data'] = _read_array(f, rectypedesc['typecode'],
+                                                rectypedesc['array_desc'])
+            else:
+                dtype = rectypedesc['typecode']
+                record['data'] = _read_data(f, dtype)
 
     elif record['rectype'] == "TIMESTAMP":
 
@@ -367,6 +386,10 @@ def _read_record(f):
 
         record['notice'] = _read_string(f)
 
+    elif record['rectype'] == "DESCRIPTION":
+
+        record['description'] = _read_string_data(f)
+
     elif record['rectype'] == "HEAP_HEADER":
 
         record['nvalues'] = _read_long(f)
@@ -396,7 +419,7 @@ def _read_record(f):
 
     else:
 
-        raise Exception("record['rectype']=%s not implemented" % \
+        raise Exception("record['rectype']=%s not implemented" %
                                                             record['rectype'])
 
     f.seek(nextrec)
@@ -407,10 +430,7 @@ def _read_record(f):
 def _read_typedesc(f):
     '''Function to read in a type descriptor'''
 
-    typedesc = {}
-
-    typedesc['typecode'] = _read_long(f)
-    typedesc['varflags'] = _read_long(f)
+    typedesc = {'typecode': _read_long(f), 'varflags': _read_long(f)}
 
     if typedesc['varflags'] & 2 == 2:
         raise Exception("System variables not implemented")
@@ -430,9 +450,7 @@ def _read_typedesc(f):
 def _read_arraydesc(f):
     '''Function to read in an array descriptor'''
 
-    arraydesc = {}
-
-    arraydesc['arrstart'] = _read_long(f)
+    arraydesc = {'arrstart': _read_long(f)}
 
     if arraydesc['arrstart'] == 8:
 
@@ -467,7 +485,7 @@ def _read_arraydesc(f):
         arraydesc['dims'] = []
         for d in range(arraydesc['nmax']):
             v = _read_long(f)
-            if v <> 0:
+            if v != 0:
                 raise Exception("Expected a zero in ARRAY_DESC")
             arraydesc['dims'].append(_read_long(f))
 
@@ -484,15 +502,19 @@ def _read_structdesc(f):
     structdesc = {}
 
     structstart = _read_long(f)
-    if structstart <> 9:
+    if structstart != 9:
         raise Exception("STRUCTSTART should be 9")
 
     structdesc['name'] = _read_string(f)
-    structdesc['predef'] = _read_long(f)
+    predef = _read_long(f)
     structdesc['ntags'] = _read_long(f)
     structdesc['nbytes'] = _read_long(f)
 
-    if structdesc['predef'] & 1 == 0:
+    structdesc['predef'] = predef & 1
+    structdesc['inherits'] = predef & 2
+    structdesc['is_super'] = predef & 4
+
+    if not structdesc['predef']:
 
         structdesc['tagtable'] = []
         for t in range(structdesc['ntags']):
@@ -511,18 +533,24 @@ def _read_structdesc(f):
             if tag['structure']:
                 structdesc['structtable'][tag['name']] = _read_structdesc(f)
 
-        STRUCT_DICT[structdesc['name']] = (structdesc['tagtable'], \
-                                           structdesc['arrtable'], \
-                                           structdesc['structtable'])
+        if structdesc['inherits'] or structdesc['is_super']:
+            structdesc['classname'] = _read_string(f)
+            structdesc['nsupclasses'] = _read_long(f)
+            structdesc['supclassnames'] = []
+            for s in range(structdesc['nsupclasses']):
+                structdesc['supclassnames'].append(_read_string(f))
+            structdesc['supclasstable'] = []
+            for s in range(structdesc['nsupclasses']):
+                structdesc['supclasstable'].append(_read_structdesc(f))
+
+        STRUCT_DICT[structdesc['name']] = structdesc
 
     else:
 
         if not structdesc['name'] in STRUCT_DICT:
             raise Exception("PREDEF=1 but can't find definition")
 
-        structdesc['tagtable'], \
-        structdesc['arrtable'], \
-        structdesc['structtable'] = STRUCT_DICT[structdesc['name']]
+        structdesc = STRUCT_DICT[structdesc['name']]
 
     return structdesc
 
@@ -530,9 +558,7 @@ def _read_structdesc(f):
 def _read_tagdesc(f):
     '''Function to read in a tag descriptor'''
 
-    tagdesc = {}
-
-    tagdesc['offset'] = _read_long(f)
+    tagdesc = {'offset': _read_long(f)}
 
     if tagdesc['offset'] == -1:
         tagdesc['offset'] = _read_uint64(f)
@@ -548,6 +574,72 @@ def _read_tagdesc(f):
     return tagdesc
 
 
+def _replace_heap(variable, heap):
+
+    if isinstance(variable, Pointer):
+
+        while isinstance(variable, Pointer):
+
+            if variable.index == 0:
+                variable = None
+            else:
+                if variable.index in heap:
+                    variable = heap[variable.index]
+                else:
+                    warnings.warn("Variable referenced by pointer not found "
+                                  "in heap: variable will be set to None")
+                    variable = None
+
+        replace, new = _replace_heap(variable, heap)
+
+        if replace:
+            variable = new
+
+        return True, variable
+
+    elif isinstance(variable, np.core.records.recarray):
+
+        # Loop over records
+        for ir, record in enumerate(variable):
+
+            replace, new = _replace_heap(record, heap)
+
+            if replace:
+                variable[ir] = new
+
+        return False, variable
+
+    elif isinstance(variable, np.core.records.record):
+
+        # Loop over values
+        for iv, value in enumerate(variable):
+
+            replace, new = _replace_heap(value, heap)
+
+            if replace:
+                variable[iv] = new
+
+        return False, variable
+
+    elif isinstance(variable, np.ndarray):
+
+        # Loop over values if type is np.object_
+        if variable.dtype.type is np.object_:
+
+            for iv in range(variable.size):
+
+                replace, new = _replace_heap(variable.item(iv), heap)
+
+                if replace:
+                    variable.itemset(iv, new)
+
+        return False, variable
+
+    else:
+
+        return False, variable
+
+
 class AttrDict(dict):
     '''
     A case-insensitive dictionary with access via item, attribute, and call
@@ -581,16 +673,16 @@ class AttrDict(dict):
 
 def readsav(file_name, idict=None, python_dict=False,
             uncompressed_file_name=None, verbose=False):
-    '''
-    Read an IDL .sav file
+    """
+    Read an IDL .sav file.
 
     Parameters
     ----------
     file_name : str
         Name of the IDL save file.
     idict : dict, optional
-        Dictionary in which to insert .sav file variables
-    python_dict: bool, optional
+        Dictionary in which to insert .sav file variables.
+    python_dict : bool, optional
         By default, the object return is not a Python dictionary, but a
         case-insensitive dictionary with item, attribute, and call access
         to variables. To get a standard Python dictionary, set this option
@@ -607,7 +699,7 @@ def readsav(file_name, idict=None, python_dict=False,
         the records read, and available variables.
 
     Returns
-    ----------
+    -------
     idl_dict : AttrDict or dict
         If `python_dict` is set to False (default), this function returns a
         case-insensitive dictionary with item, attribute, and call access
@@ -615,7 +707,8 @@ def readsav(file_name, idict=None, python_dict=False,
         returns a Python dictionary with all variable names in lowercase.
         If `idict` was specified, then variables are written to the
         dictionary specified, and the updated dictionary is returned.
-    '''
+
+    """
 
     # Initialize record and variable holders
     records = []
@@ -629,20 +722,20 @@ def readsav(file_name, idict=None, python_dict=False,
 
     # Read the signature, which should be 'SR'
     signature = _read_bytes(f, 2)
-    if signature <> asbytes('SR'):
+    if signature != b'SR':
         raise Exception("Invalid SIGNATURE: %s" % signature)
 
     # Next, the record format, which is '\x00\x04' for normal .sav
     # files, and '\x00\x06' for compressed .sav files.
     recfmt = _read_bytes(f, 2)
 
-    if recfmt == asbytes('\x00\x04'):
+    if recfmt == b'\x00\x04':
         pass
 
-    elif recfmt == asbytes('\x00\x06'):
+    elif recfmt == b'\x00\x06':
 
         if verbose:
-            print "IDL Save file is compressed"
+            print("IDL Save file is compressed")
 
         if uncompressed_file_name:
             fout = open(uncompressed_file_name, 'w+b')
@@ -650,10 +743,10 @@ def readsav(file_name, idict=None, python_dict=False,
             fout = tempfile.NamedTemporaryFile(suffix='.sav')
 
         if verbose:
-            print " -> expanding to %s" % fout.name
+            print(" -> expanding to %s" % fout.name)
 
         # Write header
-        fout.write(asbytes('SR\x00\x04'))
+        fout.write(b'SR\x00\x04')
 
         # Cycle through records
         while True:
@@ -680,16 +773,16 @@ def readsav(file_name, idict=None, python_dict=False,
             pos = f.tell()
 
             # Decompress record
-            string = zlib.decompress(f.read(nextrec-pos))
+            rec_string = zlib.decompress(f.read(nextrec-pos))
 
             # Find new position of next record
-            nextrec = fout.tell() + len(string) + 12
+            nextrec = fout.tell() + len(rec_string) + 12
 
             # Write out record
             fout.write(struct.pack('>I', int(nextrec % 2**32)))
             fout.write(struct.pack('>I', int((nextrec - (nextrec % 2**32)) / 2**32)))
             fout.write(unknown)
-            fout.write(string)
+            fout.write(rec_string)
 
         # Close the original compressed file
         f.close()
@@ -721,8 +814,9 @@ def readsav(file_name, idict=None, python_dict=False,
     # Find all variables
     for r in records:
         if r['rectype'] == "VARIABLE":
-            while isinstance(r['data'], Pointer):
-                r['data'] = heap[r['data'].index]
+            replace, new = _replace_heap(r['data'], heap)
+            if replace:
+                r['data'] = new
             variables[r['varname'].lower()] = r['data']
 
     if verbose:
@@ -730,48 +824,55 @@ def readsav(file_name, idict=None, python_dict=False,
         # Print out timestamp info about the file
         for record in records:
             if record['rectype'] == "TIMESTAMP":
-                print "-"*50
-                print "Date: %s" % record['date']
-                print "User: %s" % record['user']
-                print "Host: %s" % record['host']
+                print("-"*50)
+                print("Date: %s" % record['date'])
+                print("User: %s" % record['user'])
+                print("Host: %s" % record['host'])
                 break
 
         # Print out version info about the file
         for record in records:
             if record['rectype'] == "VERSION":
-                print "-"*50
-                print "Format: %s" % record['format']
-                print "Architecture: %s" % record['arch']
-                print "Operating System: %s" % record['os']
-                print "IDL Version: %s" % record['release']
+                print("-"*50)
+                print("Format: %s" % record['format'])
+                print("Architecture: %s" % record['arch'])
+                print("Operating System: %s" % record['os'])
+                print("IDL Version: %s" % record['release'])
                 break
 
         # Print out identification info about the file
         for record in records:
             if record['rectype'] == "IDENTIFICATON":
-                print "-"*50
-                print "Author: %s" % record['author']
-                print "Title: %s" % record['title']
-                print "ID Code: %s" % record['idcode']
+                print("-"*50)
+                print("Author: %s" % record['author'])
+                print("Title: %s" % record['title'])
+                print("ID Code: %s" % record['idcode'])
                 break
 
-        print "-"*50
-        print "Successfully read %i records of which:" % \
-                                            (len(records))
+        # Print out descriptions saved with the file
+        for record in records:
+            if record['rectype'] == "DESCRIPTION":
+                print("-"*50)
+                print("Description: %s" % record['description'])
+                break
+
+        print("-"*50)
+        print("Successfully read %i records of which:" %
+                                            (len(records)))
 
         # Create convenience list of record types
         rectypes = [r['rectype'] for r in records]
 
         for rt in set(rectypes):
-            if rt <> 'END_MARKER':
-                print " - %i are of type %s" % (rectypes.count(rt), rt)
-        print "-"*50
+            if rt != 'END_MARKER':
+                print(" - %i are of type %s" % (rectypes.count(rt), rt))
+        print("-"*50)
 
         if 'VARIABLE' in rectypes:
-            print "Available variables:"
+            print("Available variables:")
             for var in variables:
-                print " - %s [%s]" % (var, type(variables[var]))
-            print "-"*50
+                print(" - %s [%s]" % (var, type(variables[var])))
+            print("-"*50)
 
     if idict:
         for var in variables:
