@@ -9,15 +9,7 @@ import numpy as np
 from numpy.testing import assert_array_equal, assert_array_almost_equal, \
      run_module_suite, assert_
 
-from scipy.io.matlab.mio_utils import cproduct, squeeze_element, \
-    chars_to_strings
-
-
-def test_cproduct():
-    assert_(cproduct(()) == 1)
-    assert_(cproduct((1,)) == 1)
-    assert_(cproduct((1,3)) == 3)
-    assert_(cproduct([1,3]) == 3)
+from scipy.io.matlab.mio_utils import squeeze_element, chars_to_strings
 
 
 def test_squeeze_element():
