@@ -6,13 +6,9 @@
 
 from __future__ import division, print_function, absolute_import
 
-__all__ = ['solve', 'solve_triangular', 'solveh_banded', 'solve_banded',
-           'solve_toeplitz', 'solve_circulant', 'inv', 'det', 'lstsq',
-           'pinv', 'pinv2', 'pinvh']
-
 import warnings
 import numpy as np
-
+from numpy import atleast_1d, atleast_2d
 from .flinalg import get_flinalg_funcs
 from .lapack import get_lapack_funcs, _compute_lwork
 from .misc import LinAlgError, _datacopied
@@ -20,24 +16,50 @@ from .decomp import _asarray_validated
 from . import decomp, decomp_svd
 from ._solve_toeplitz import levinson
 
+__all__ = ['solve', 'solve_triangular', 'solveh_banded', 'solve_banded',
+           'solve_toeplitz', 'solve_circulant', 'inv', 'det', 'lstsq',
+           'pinv', 'pinv2', 'pinvh', 'matrix_balance']
+
 
 # Linear equations
 def solve(a, b, sym_pos=False, lower=False, overwrite_a=False,
-          overwrite_b=False, debug=False, check_finite=True):
+          overwrite_b=False, debug=None, check_finite=True, assume_a='gen',
+          transposed=False):
     """
-    Solve the equation ``a x = b`` for ``x``.
+    Solves the linear equation set ``a * x = b`` for the unknown ``x``
+    for square ``a`` matrix.
+
+    If the data matrix is known to be a particular type then supplying the
+    corresponding string to ``assume_a`` key chooses the dedicated solver.
+    The available options are
+
+    ===================  ========
+     generic matrix       'gen'
+     symmetric            'sym'
+     hermitian            'her'
+     positive definite    'pos'
+    ===================  ========
+
+    If omitted, ``'gen'`` is the default structure.
+
+    The datatype of the arrays define which solver is called regardless
+    of the values. In other words, even when the complex array entries have
+    precisely zero imaginary parts, the complex solver will be called based
+    on the data type of the array.
 
     Parameters
     ----------
-    a : (M, M) array_like
-        A square matrix.
-    b : (M,) or (M, N) array_like
-        Right-hand side matrix in ``a x = b``.
+    a : (N, N) array_like
+        Square input data
+    b : (N, NRHS) array_like
+        Input data for the right hand side.
     sym_pos : bool, optional
-        Assume `a` is symmetric and positive definite.
+        Assume `a` is symmetric and positive definite. This key is deprecated
+        and assume_a = 'pos' keyword is recommended instead. The functionality
+        is the same. It will be removed in the future.
     lower : bool, optional
-        Use only data contained in the lower triangle of `a`, if `sym_pos` is
-        true.  Default is to use upper triangle.
+        If True, only the data contained in the lower triangle of `a`. Default
+        is to use upper triangle. (ignored for ``'gen'``)
     overwrite_a : bool, optional
         Allow overwriting data in `a` (may enhance performance).
         Default is False.
@@ -48,19 +70,25 @@ def solve(a, b, sym_pos=False, lower=False, overwrite_a=False,
         Whether to check that the input matrices contain only finite numbers.
         Disabling may give a performance gain, but may result in problems
         (crashes, non-termination) if the inputs do contain infinities or NaNs.
+    assume_a : str, optional
+        Valid entries are explained above.
+    transposed: bool, optional
+        If True, depending on the data type ``a^T x = b`` or ``a^H x = b`` is
+        solved (only taken into account for ``'gen'``).
 
     Returns
     -------
-    x : (M,) or (M, N) ndarray
-        Solution to the system ``a x = b``.  Shape of the return matches the
-        shape of `b`.
+    x : (N, NRHS) ndarray
+        The solution array.
 
     Raises
     ------
-    LinAlgError
-        If `a` is singular.
     ValueError
-        If `a` is not square
+        If size mismatches detected or input a is not square.
+    LinAlgError
+        If the matrix is singular.
+    RuntimeWarning
+        If an ill-conditioned input a is detected.
 
     Examples
     --------
@@ -75,38 +103,129 @@ def solve(a, b, sym_pos=False, lower=False, overwrite_a=False,
     >>> np.dot(a, x) == b
     array([ True,  True,  True], dtype=bool)
 
+    Notes
+    -----
+    If the input b matrix is a 1D array with N elements, when supplied
+    together with an NxN input a, it is assumed as a valid column vector
+    despite the apparent size mismatch. This is compatible with the
+    numpy.dot() behavior and the returned result is still 1D array.
+
+    The generic, symmetric, hermitian and positive definite solutions are
+    obtained via calling ?GESVX, ?SYSVX, ?HESVX, and ?POSVX routines of
+    LAPACK respectively.
     """
-    a1 = _asarray_validated(a, check_finite=check_finite)
-    b1 = _asarray_validated(b, check_finite=check_finite)
-    if len(a1.shape) != 2 or a1.shape[0] != a1.shape[1]:
-        raise ValueError('expected square matrix')
-    if a1.shape[0] != b1.shape[0]:
-        raise ValueError('incompatible dimensions')
+    # Flags for 1D or nD right hand side
+    b_is_1D = False
+    b_is_ND = False
+
+    a1 = atleast_2d(_asarray_validated(a, check_finite=check_finite))
+    b1 = atleast_1d(_asarray_validated(b, check_finite=check_finite))
+    n = a1.shape[0]
+
     overwrite_a = overwrite_a or _datacopied(a1, a)
     overwrite_b = overwrite_b or _datacopied(b1, b)
-    if debug:
-        print('solve:overwrite_a=', overwrite_a)
-        print('solve:overwrite_b=', overwrite_b)
-    if sym_pos:
-        posv, = get_lapack_funcs(('posv',), (a1, b1))
-        c, x, info = posv(a1, b1, lower=lower,
-                          overwrite_a=overwrite_a,
-                          overwrite_b=overwrite_b)
-    else:
-        gesv, = get_lapack_funcs(('gesv',), (a1, b1))
-        lu, piv, x, info = gesv(a1, b1, overwrite_a=overwrite_a,
-                                overwrite_b=overwrite_b)
 
-    if info == 0:
+    if a1.shape[0] != a1.shape[1]:
+        raise ValueError('Input a needs to be a square matrix.')
+
+    if n != b1.shape[0]:
+        # Last chance to catch 1x1 scalar a and 1D b arrays
+        if not (n == 1 and b1.size != 0):
+            raise ValueError('Input b has to have same number of rows as '
+                             'input a')
+
+    # accomodate empty arrays
+    if b1.size == 0:
+        return np.asfortranarray(b1.copy())
+
+    # regularize 1D b arrays to 2D and catch nD RHS arrays
+    if b1.ndim == 1:
+        if n == 1:
+            b1 = b1[None, :]
+        else:
+            b1 = b1[:, None]
+        b_is_1D = True
+    elif b1.ndim > 2:
+        b_is_ND = True
+
+    r_or_c = complex if np.iscomplexobj(a1) else float
+
+    if assume_a in ('gen', 'sym', 'her', 'pos'):
+        _structure = assume_a
+    else:
+        raise ValueError('{} is not a recognized matrix structure'
+                         ''.format(assume_a))
+
+    # Deprecate keyword "debug"
+    if debug is not None:
+        warnings.warn('Use of the "debug" keyword is deprecated '
+                      'and this keyword will be removed in the future '
+                      'versions of SciPy.', DeprecationWarning)
+
+    # Backwards compatibility - old keyword.
+    if sym_pos:
+        assume_a = 'pos'
+
+    if _structure == 'gen':
+        gesvx = get_lapack_funcs('gesvx', (a1, b1))
+        trans_conj = 'N'
+        if transposed:
+            trans_conj = 'T' if r_or_c is float else 'H'
+        (_, _, _, _, _, _, _,
+         x, rcond, _, _, info) = gesvx(a1, b1,
+                                       trans=trans_conj,
+                                       overwrite_a=overwrite_a,
+                                       overwrite_b=overwrite_b
+                                       )
+    elif _structure == 'sym':
+        sysvx, sysvx_lw = get_lapack_funcs(('sysvx', 'sysvx_lwork'), (a1, b1))
+        lwork = _compute_lwork(sysvx_lw, n, lower)
+        _, _, _, _, x, rcond, _, _, info = sysvx(a1, b1, lwork=lwork,
+                                                 lower=lower,
+                                                 overwrite_a=overwrite_a,
+                                                 overwrite_b=overwrite_b
+                                                 )
+    elif _structure == 'her':
+        hesvx, hesvx_lw = get_lapack_funcs(('hesvx', 'hesvx_lwork'), (a1, b1))
+        lwork = _compute_lwork(hesvx_lw, n, lower)
+        _, _, x, rcond, _, _, info = hesvx(a1, b1, lwork=lwork,
+                                           lower=lower,
+                                           overwrite_a=overwrite_a,
+                                           overwrite_b=overwrite_b
+                                           )
+    else:
+        posvx = get_lapack_funcs('posvx', (a1, b1))
+        _, _, _, _, _, x, rcond, _, _, info = posvx(a1, b1,
+                                                    lower=lower,
+                                                    overwrite_a=overwrite_a,
+                                                    overwrite_b=overwrite_b
+                                                    )
+
+    # Unlike ?xxSV, ?xxSVX writes the solution x to a separate array, and
+    # overwrites b with its scaled version which is thrown away. Thus, the
+    # solution does not admit the same shape with the original b. For
+    # backwards compatibility, we reshape it manually.
+    if b_is_1D:
+        x = x.ravel()
+    if b_is_ND:
+        x = x.reshape(*b1.shape, order='F')
+
+    if info < 0:
+        raise ValueError('LAPACK reported an illegal value in {}-th argument'
+                         '.'.format(-info))
+    elif info == 0:
+        return x
+    elif 0 < info <= n:
+        raise LinAlgError('Matrix is singular.')
+    elif info > n:
+        warnings.warn('scipy.linalg.solve\nIll-conditioned matrix detected.'
+                      ' Result is not guaranteed to be accurate.\nReciprocal'
+                      ' condition number: {}'.format(rcond), RuntimeWarning)
         return x
-    if info > 0:
-        raise LinAlgError("singular matrix")
-    raise ValueError('illegal value in %d-th argument of internal gesv|posv' %
-                     -info)
 
 
 def solve_triangular(a, b, trans=0, lower=False, unit_diagonal=False,
-                     overwrite_b=False, debug=False, check_finite=True):
+                     overwrite_b=False, debug=None, check_finite=True):
     """
     Solve the equation `a x = b` for `x`, assuming a is a triangular matrix.
 
@@ -154,6 +273,13 @@ def solve_triangular(a, b, trans=0, lower=False, unit_diagonal=False,
     .. versionadded:: 0.9.0
 
     """
+
+    # Deprecate keyword "debug"
+    if debug is not None:
+        warnings.warn('Use of the "debug" keyword is deprecated '
+                      'and this keyword will be removed in the future '
+                      'versions of SciPy.', DeprecationWarning)
+
     a1 = _asarray_validated(a, check_finite=check_finite)
     b1 = _asarray_validated(b, check_finite=check_finite)
     if len(a1.shape) != 2 or a1.shape[0] != a1.shape[1]:
@@ -178,7 +304,7 @@ def solve_triangular(a, b, trans=0, lower=False, unit_diagonal=False,
 
 
 def solve_banded(l_and_u, ab, b, overwrite_ab=False, overwrite_b=False,
-                 debug=False, check_finite=True):
+                 debug=None, check_finite=True):
     """
     Solve the equation a x = b for x, assuming a is banded matrix.
 
@@ -217,6 +343,13 @@ def solve_banded(l_and_u, ab, b, overwrite_ab=False, overwrite_b=False,
         shape of `b`.
 
     """
+
+    # Deprecate keyword "debug"
+    if debug is not None:
+        warnings.warn('Use of the "debug" keyword is deprecated '
+                      'and this keyword will be removed in the future '
+                      'versions of SciPy.', DeprecationWarning)
+
     a1 = _asarray_validated(ab, check_finite=check_finite, as_inexact=True)
     b1 = _asarray_validated(b, check_finite=check_finite, as_inexact=True)
     # Validate shapes.
@@ -251,8 +384,8 @@ def solve_banded(l_and_u, ab, b, overwrite_ab=False, overwrite_b=False,
         return x
     if info > 0:
         raise LinAlgError("singular matrix")
-    raise ValueError('illegal value in %d-th argument of internal gbsv/gtsv' %
-                     -info)
+    raise ValueError('illegal value in %d-th argument of internal '
+                     'gbsv/gtsv' % -info)
 
 
 def solveh_banded(ab, b, overwrite_ab=False, overwrite_b=False, lower=False,
@@ -330,8 +463,8 @@ def solveh_banded(ab, b, overwrite_ab=False, overwrite_b=False, lower=False,
     if info > 0:
         raise LinAlgError("%d-th leading minor not positive definite" % info)
     if info < 0:
-        raise ValueError('illegal value in %d-th argument of internal pbsv' %
-                         -info)
+        raise ValueError('illegal value in %d-th argument of internal '
+                         'pbsv' % -info)
     return x
 
 
@@ -385,14 +518,13 @@ def solve_toeplitz(c_or_cr, b, check_finite=True):
     vals = np.concatenate((r[-1:0:-1], c))
     if b is None:
         raise ValueError('illegal value, `b` is a required argument')
-    if vals.shape[0] != (2*b.shape[0] - 1):
-        raise ValueError('incompatible dimensions')
 
     b = _asarray_validated(b)
+    if vals.shape[0] != (2*b.shape[0] - 1):
+        raise ValueError('incompatible dimensions')
     if np.iscomplexobj(vals) or np.iscomplexobj(b):
         vals = np.asarray(vals, dtype=np.complex128, order='c')
         b = np.asarray(b, dtype=np.complex128)
-
     else:
         vals = np.asarray(vals, dtype=np.double, order='c')
         b = np.asarray(b, dtype=np.double)
@@ -660,14 +792,14 @@ def inv(a, overwrite_a=False, check_finite=True):
         raise ValueError('expected square matrix')
     overwrite_a = overwrite_a or _datacopied(a1, a)
     #XXX: I found no advantage or disadvantage of using finv.
-##     finv, = get_flinalg_funcs(('inv',),(a1,))
-##     if finv is not None:
-##         a_inv,info = finv(a1,overwrite_a=overwrite_a)
-##         if info==0:
-##             return a_inv
-##         if info>0: raise LinAlgError, "singular matrix"
-##         if info<0: raise ValueError,\
-##            'illegal value in %d-th argument of internal inv.getrf|getri'%(-info)
+#     finv, = get_flinalg_funcs(('inv',),(a1,))
+#     if finv is not None:
+#         a_inv,info = finv(a1,overwrite_a=overwrite_a)
+#         if info==0:
+#             return a_inv
+#         if info>0: raise LinAlgError, "singular matrix"
+#         if info<0: raise ValueError('illegal value in %d-th argument of '
+#                                     'internal inv.getrf|getri'%(-info))
     getrf, getri, getri_lwork = get_lapack_funcs(('getrf', 'getri',
                                                   'getri_lwork'),
                                                  (a1,))
@@ -691,7 +823,7 @@ def inv(a, overwrite_a=False, check_finite=True):
     return inv_a
 
 
-### Determinant
+# Determinant
 
 def det(a, overwrite_a=False, check_finite=True):
     """
@@ -750,7 +882,8 @@ def det(a, overwrite_a=False, check_finite=True):
                          'det.getrf' % -info)
     return a_det
 
-### Linear Least Squares
+# Linear Least Squares
+
 
 class LstsqLapackError(LinAlgError):
     pass
@@ -837,7 +970,8 @@ def lstsq(a, b, cond=None, overwrite_a=False, overwrite_b=False,
         raise ValueError('LAPACK driver "%s" is not found' % driver)
 
     lapack_func, lapack_lwork = get_lapack_funcs((driver,
-                                        '%s_lwork' % driver), (a1, b1))
+                                                 '%s_lwork' % driver),
+                                                 (a1, b1))
     real_data = True if (lapack_func.dtype.kind == 'f') else False
 
     if m < n:
@@ -872,22 +1006,22 @@ def lstsq(a, b, cond=None, overwrite_a=False, overwrite_b=False,
                     # size of the iwork array in query mode.  This bug was
                     # fixed in LAPACK 3.2.2, released July 21, 2010.
                     mesg = ("internal gelsd driver lwork query error, "
-                           "required iwork dimension not returned. "
-                           "This is likely the result of LAPACK bug "
-                           "0038, fixed in LAPACK 3.2.2 (released "
-                           "July 21, 2010). ")
-                    
+                            "required iwork dimension not returned. "
+                            "This is likely the result of LAPACK bug "
+                            "0038, fixed in LAPACK 3.2.2 (released "
+                            "July 21, 2010). ")
+
                     if lapack_driver is None:
                         # restart with gelss
                         lstsq.default_lapack_driver = 'gelss'
                         mesg += "Falling back to 'gelss' driver."
                         warnings.warn(mesg, RuntimeWarning)
                         return lstsq(a, b, cond, overwrite_a, overwrite_b,
-                                    check_finite, lapack_driver='gelss')
+                                     check_finite, lapack_driver='gelss')
 
                     # can't proceed, bail out
-                    mesg += ("Use a different lapack_driver when calling lstsq "
-                            "or upgrade LAPACK.")
+                    mesg += ("Use a different lapack_driver when calling lstsq"
+                             " or upgrade LAPACK.")
                     raise LstsqLapackError(mesg)
 
                 x, s, rank, info = lapack_func(a1, b1, lwork,
@@ -901,7 +1035,7 @@ def lstsq(a, b, cond=None, overwrite_a=False, overwrite_b=False,
             raise LinAlgError("SVD did not converge in Linear Least Squares")
         if info < 0:
             raise ValueError('illegal value in %d-th argument of internal %s'
-                                                    % (-info, lapack_driver))
+                             % (-info, lapack_driver))
         resids = np.asarray([], dtype=x.dtype)
         if m > n:
             x1 = x[:n]
@@ -912,9 +1046,9 @@ def lstsq(a, b, cond=None, overwrite_a=False, overwrite_b=False,
 
     elif driver == 'gelsy':
         lwork = _compute_lwork(lapack_lwork, m, n, nrhs, cond)
-        jptv = np.zeros((a1.shape[1],1), dtype=np.int32)
+        jptv = np.zeros((a1.shape[1], 1), dtype=np.int32)
         v, x, j, rank, info = lapack_func(a1, b1, jptv, cond,
-                                           lwork, False, False)
+                                          lwork, False, False)
         if info < 0:
             raise ValueError("illegal value in %d-th argument of internal "
                              "gelsy" % -info)
@@ -924,6 +1058,7 @@ def lstsq(a, b, cond=None, overwrite_a=False, overwrite_b=False,
         return x, np.array([], x.dtype), rank, None
 lstsq.default_lapack_driver = 'gelsd'
 
+
 def pinv(a, cond=None, rcond=None, return_rank=False, check_finite=True):
     """
     Compute the (Moore-Penrose) pseudo-inverse of a matrix.
@@ -1125,3 +1260,152 @@ def pinvh(a, cond=None, rcond=None, lower=True, return_rank=False,
         return B, len(psigma_diag)
     else:
         return B
+
+
+def matrix_balance(A, permute=True, scale=True, separate=False,
+                   overwrite_a=False):
+    """
+    Compute a diagonal similarity transformation for row/column balancing.
+
+    The balancing tries to equalize the row and column 1-norms by applying
+    a similarity transformation such that the magnitude variation of the
+    matrix entries is reflected to the scaling matrices.
+
+    Moreover, if enabled, the matrix is first permuted to isolate the upper
+    triangular parts of the matrix and, again if scaling is also enabled,
+    only the remaining subblocks are subjected to scaling.
+
+    The balanced matrix satisfies the following equality
+
+    .. math::
+
+                        B = T^{-1} A T
+
+    The scaling coefficients are approximated to the nearest power of 2
+    to avoid round-off errors.
+
+    Parameters
+    ----------
+    A : (n, n) array_like
+        Square data matrix for the balancing.
+    permute : bool, optional
+        The selector to define whether permutation of A is also performed
+        prior to scaling.
+    scale : bool, optional
+        The selector to turn on and off the scaling. If False, the matrix
+        will not be scaled.
+    separate : bool, optional
+        This switches from returning a full matrix of the transformation
+        to a tuple of two separate 1D permutation and scaling arrays.
+    overwrite_a : bool, optional
+        This is passed to xGEBAL directly. Essentially, overwrites the result
+        to the data. It might increase the space efficiency. See LAPACK manual
+        for details. This is False by default.
+
+    Returns
+    -------
+    B : (n, n) ndarray
+        Balanced matrix
+    T : (n, n) ndarray
+        A possibly permuted diagonal matrix whose nonzero entries are
+        integer powers of 2 to avoid numerical truncation errors.
+    scale, perm : (n,) ndarray
+        If ``separate`` keyword is set to True then instead of the array
+        ``T`` above, the scaling and the permutation vectors are given
+        separately as a tuple without allocating the full array ``T``.
+
+    .. versionadded:: 0.19.0
+
+    Notes
+    -----
+
+    This algorithm is particularly useful for eigenvalue and matrix
+    decompositions and in many cases it is already called by various
+    LAPACK routines.
+
+    The algorithm is based on the well-known technique of [1]_ and has
+    been modified to account for special cases. See [2]_ for details
+    which have been implemented since LAPACK v3.5.0. Before this version
+    there are corner cases where balancing can actually worsen the
+    conditioning. See [3]_ for such examples.
+
+    The code is a wrapper around LAPACK's xGEBAL routine family for matrix
+    balancing.
+
+    Examples
+    --------
+    >>> from scipy import linalg
+    >>> x = np.array([[1,2,0], [9,1,0.01], [1,2,10*np.pi]])
+
+    >>> y, permscale = linalg.matrix_balance(x)
+    >>> np.abs(x).sum(axis=0) / np.abs(x).sum(axis=1)
+    array([ 3.66666667,  0.4995005 ,  0.91312162])
+
+    >>> np.abs(y).sum(axis=0) / np.abs(y).sum(axis=1) # 1-norms approx. equal
+    array([ 1.10625   ,  0.90547703,  1.00011878])
+
+    >>> permscale  # only powers of 2 (0.5 == 2^(-1))
+    array([[  0.5,   0. ,   0. ],
+           [  0. ,   1. ,   0. ],
+           [  0. ,   0. ,  16. ]])
+
+    References
+    ----------
+    .. [1] : B.N. Parlett and C. Reinsch, "Balancing a Matrix for
+       Calculation of Eigenvalues and Eigenvectors", Numerische Mathematik,
+       Vol.13(4), 1969, DOI:10.1007/BF02165404
+
+    .. [2] : R. James, J. Langou, B.R. Lowery, "On matrix balancing and
+       eigenvector computation", 2014, Available online:
+       http://arxiv.org/abs/1401.5766
+
+    .. [3] :  D.S. Watkins. A case where balancing is harmful.
+       Electron. Trans. Numer. Anal, Vol.23, 2006.
+
+    """
+
+    A = np.atleast_2d(_asarray_validated(A, check_finite=True))
+
+    if not np.equal(*A.shape):
+        raise ValueError('The data matrix for balancing should be square.')
+
+    gebal = get_lapack_funcs(('gebal'), (A,))
+    B, lo, hi, ps, info = gebal(A, scale=scale, permute=permute,
+                                overwrite_a=overwrite_a)
+
+    if info < 0:
+        raise ValueError('xGEBAL exited with the internal error '
+                         '"illegal value in argument number {}.". See '
+                         'LAPACK documentation for the xGEBAL error codes.'
+                         ''.format(-info))
+
+    # Separate the permutations from the scalings and then convert to int
+    scaling = np.ones_like(ps, dtype=float)
+    scaling[lo:hi+1] = ps[lo:hi+1]
+
+    # gebal uses 1-indexing
+    ps = ps.astype(int, copy=False) - 1
+    n = A.shape[0]
+    perm = np.arange(n)
+
+    # LAPACK permutes with the ordering n --> hi, then 0--> lo
+    if hi < n:
+        for ind, x in enumerate(ps[hi+1:][::-1], 1):
+            if n-ind == x:
+                continue
+            perm[[x, n-ind]] = perm[[n-ind, x]]
+
+    if lo > 0:
+        for ind, x in enumerate(ps[:lo]):
+            if ind == x:
+                continue
+            perm[[x, ind]] = perm[[ind, x]]
+
+    if separate:
+        return B, (scaling, perm)
+
+    # get the inverse permutation
+    iperm = np.empty_like(perm)
+    iperm[perm] = np.arange(n)
+
+    return B, np.diag(scaling)[iperm, :]
