@@ -18,8 +18,29 @@ Package Content
    Output        -- Result from the fit.
    odr           -- Low-level function for ODR.
 
-   odr_error     -- Error exception.
-   odr_stop      -- Stop exception.
+   OdrWarning    -- Warning about potential problems when running ODR
+   OdrError      -- Error exception.
+   OdrStop       -- Stop exception.
+
+   odr_error     -- Same as OdrError (for backwards compatibility)
+   odr_stop      -- Same as OdrStop (for backwards compatibility)
+
+Prebuilt models:
+
+.. autosummary::
+   :toctree: generated/
+
+   polynomial
+
+.. data:: exponential
+
+.. data:: multilinear
+
+.. data:: unilinear
+
+.. data:: quadratic
+
+.. data:: polynomial
 
 Usage information
 =================
@@ -54,7 +75,7 @@ ODRPACK, in addition to the low-level `odr` function.
 
 Additional background information about ODRPACK can be found in the
 `ODRPACK User's Guide
-<http://docs.scipy.org/doc/external/odrpack_guide.pdf>`_, reading
+<https://docs.scipy.org/doc/external/odrpack_guide.pdf>`_, reading
 which is recommended.
 
 Basic usage
