@@ -47,6 +47,7 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
   PyObject *result;
   PyArrayObject *result_array = NULL;
   PyArrayObject *pyXplusD;
+  void *beta_dst;
 
   arg01 = PyTuple_New(2);
 
@@ -55,14 +56,14 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
       npy_intp dim2[2];
       dim2[0] = *m;
       dim2[1] = *n;
-      pyXplusD = (PyArrayObject *) PyArray_SimpleNew(2, dim2, PyArray_DOUBLE);
+      pyXplusD = (PyArrayObject *) PyArray_SimpleNew(2, dim2, NPY_DOUBLE);
       memcpy(pyXplusD->data, (void *)xplusd, (*m) * (*n) * sizeof(double));
     }
   else
     {
       npy_intp dim1[1];
       dim1[0] = *n;
-      pyXplusD = (PyArrayObject *) PyArray_SimpleNew(1, dim1, PyArray_DOUBLE);
+      pyXplusD = (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_DOUBLE);
       memcpy(pyXplusD->data, (void *)xplusd, (*n) * sizeof(double));
     }
 
@@ -83,8 +84,10 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
   Py_DECREF(arg01);
   *istop = 0;
 
-  memcpy(((PyArrayObject *) (odr_global.pyBeta))->data, (void *)beta,
-         (*np) * sizeof(double));
+  beta_dst = ((PyArrayObject *) (odr_global.pyBeta))->data;
+  if (beta != beta_dst) {
+      memcpy(beta_dst, (void *)beta, (*np) * sizeof(double));
+  }
 
   if ((*ideval % 10) >= 1)
     {
@@ -98,8 +101,6 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
 
       if ((result = PyEval_CallObject(odr_global.fcn, arglist)) == NULL)
         {
-          PyObject *tmpobj, *str1;
-
           if (PyErr_ExceptionMatches(odr_stop))
             {
               /* stop, don't fail */
@@ -108,29 +109,12 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
               Py_DECREF(arglist);
               return;
             }
-
-          PyErr_Print();
-          tmpobj = PyObject_GetAttrString(odr_global.fcn, "func_name");
-          if (tmpobj == NULL)
-            goto fail;
-
-          str1 =
-            PyString_FromString
-            ("Error occurred while calling the Python function named ");
-          if (str1 == NULL)
-            {
-              Py_DECREF(tmpobj);
-              goto fail;
-            }
-          PyString_ConcatAndDel(&str1, tmpobj);
-          PyErr_SetString(odr_error, PyString_AsString(str1));
-          Py_DECREF(str1);
           goto fail;
         }
 
       if ((result_array =
            (PyArrayObject *) PyArray_ContiguousFromObject(result,
-                                                          PyArray_DOUBLE, 0,
+                                                          NPY_DOUBLE, 0,
                                                           2)) == NULL)
         {
           PYERR2(odr_error,
@@ -153,8 +137,6 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
 
       if ((result = PyEval_CallObject(odr_global.fjacb, arglist)) == NULL)
         {
-          PyObject *tmpobj, *str1;
-
           if (PyErr_ExceptionMatches(odr_stop))
             {
               /* stop, don't fail */
@@ -163,29 +145,12 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
               Py_DECREF(arglist);
               return;
             }
-
-          PyErr_Print();
-          tmpobj = PyObject_GetAttrString(odr_global.fjacb, "func_name");
-          if (tmpobj == NULL)
-            goto fail;
-
-          str1 =
-            PyString_FromString
-            ("Error occurred while calling the Python function named ");
-          if (str1 == NULL)
-            {
-              Py_DECREF(tmpobj);
-              goto fail;
-            }
-          PyString_ConcatAndDel(&str1, tmpobj);
-          PyErr_SetString(odr_error, PyString_AsString(str1));
-          Py_DECREF(str1);
           goto fail;
         }
 
       if ((result_array =
            (PyArrayObject *) PyArray_ContiguousFromObject(result,
-                                                          PyArray_DOUBLE, 0,
+                                                          NPY_DOUBLE, 0,
                                                           2)) == NULL)
         {
           PYERR2(odr_error,
@@ -231,8 +196,6 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
 
       if ((result = PyEval_CallObject(odr_global.fjacd, arglist)) == NULL)
         {
-          PyObject *tmpobj, *str1;
-
           if (PyErr_ExceptionMatches(odr_stop))
             {
               /* stop, don't fail */
@@ -241,29 +204,12 @@ void fcn_callback(int *n, int *m, int *np, int *nq, int *ldn, int *ldm,
               Py_DECREF(arglist);
               return;
             }
-
-          PyErr_Print();
-          tmpobj = PyObject_GetAttrString(odr_global.fjacd, "func_name");
-          if (tmpobj == NULL)
-            goto fail;
-
-          str1 =
-            PyString_FromString
-            ("Error occurred while calling the Python function named ");
-          if (str1 == NULL)
-            {
-              Py_DECREF(tmpobj);
-              goto fail;
-            }
-          PyString_ConcatAndDel(&str1, tmpobj);
-          PyErr_SetString(odr_error, PyString_AsString(str1));
-          Py_DECREF(str1);
           goto fail;
         }
 
       if ((result_array =
            (PyArrayObject *) PyArray_ContiguousFromObject(result,
-                                                          PyArray_DOUBLE, 0,
+                                                          NPY_DOUBLE, 0,
                                                           2)) == NULL)
         {
           PYERR2(odr_error,
@@ -407,10 +353,10 @@ PyObject *gen_output(int n, int m, int np, int nq, int ldwe, int ld2we,
   wrk7--;
 
   dim1[0] = beta->dimensions[0];
-  sd_beta = (PyArrayObject *) PyArray_SimpleNew(1, dim1, PyArray_DOUBLE);
+  sd_beta = (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_DOUBLE);
   dim2[0] = beta->dimensions[0];
   dim2[1] = beta->dimensions[0];
-  cov_beta = (PyArrayObject *) PyArray_SimpleNew(2, dim2, PyArray_DOUBLE);
+  cov_beta = (PyArrayObject *) PyArray_SimpleNew(2, dim2, NPY_DOUBLE);
 
   memcpy(sd_beta->data, (void *)((double *)(work->data) + sd),
          np * sizeof(double));
@@ -453,32 +399,32 @@ PyObject *gen_output(int n, int m, int np, int nq, int ldwe, int ld2we,
         {
           dim1[0] = n;
           deltaA =
-            (PyArrayObject *) PyArray_SimpleNew(1, dim1, PyArray_DOUBLE);
+            (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_DOUBLE);
           xplusA =
-            (PyArrayObject *) PyArray_SimpleNew(1, dim1, PyArray_DOUBLE);
+            (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_DOUBLE);
         }
       else
         {
           dim2[0] = m;
           dim2[1] = n;
           deltaA =
-            (PyArrayObject *) PyArray_SimpleNew(2, dim2, PyArray_DOUBLE);
+            (PyArrayObject *) PyArray_SimpleNew(2, dim2, NPY_DOUBLE);
           xplusA =
-            (PyArrayObject *) PyArray_SimpleNew(2, dim2, PyArray_DOUBLE);
+            (PyArrayObject *) PyArray_SimpleNew(2, dim2, NPY_DOUBLE);
         }
 
       if (nq == 1)
         {
           dim1[0] = n;
-          epsA = (PyArrayObject *) PyArray_SimpleNew(1, dim1, PyArray_DOUBLE);
-          fnA = (PyArrayObject *) PyArray_SimpleNew(1, dim1, PyArray_DOUBLE);
+          epsA = (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_DOUBLE);
+          fnA = (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_DOUBLE);
         }
       else
         {
           dim2[0] = nq;
           dim2[1] = n;
-          epsA = (PyArrayObject *) PyArray_SimpleNew(2, dim2, PyArray_DOUBLE);
-          fnA = (PyArrayObject *) PyArray_SimpleNew(2, dim2, PyArray_DOUBLE);
+          epsA = (PyArrayObject *) PyArray_SimpleNew(2, dim2, NPY_DOUBLE);
+          fnA = (PyArrayObject *) PyArray_SimpleNew(2, dim2, NPY_DOUBLE);
         }
 
       memcpy(deltaA->data, (void *)((double *)(work->data) + delta),
@@ -671,7 +617,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
   if (!implicit)
     {
       if ((y =
-           (PyArrayObject *) PyArray_CopyFromObject(py, PyArray_DOUBLE, 1,
+           (PyArrayObject *) PyArray_CopyFromObject(py, NPY_DOUBLE, 1,
                                                     2)) == NULL)
         {
           PYERR(PyExc_ValueError,
@@ -679,7 +625,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
         }
       n = y->dimensions[y->nd - 1];     /* pick the last dimension */
       if ((x =
-           (PyArrayObject *) PyArray_CopyFromObject(px, PyArray_DOUBLE, 1,
+           (PyArrayObject *) PyArray_CopyFromObject(px, NPY_DOUBLE, 1,
                                                     2)) == NULL)
         {
           PYERR(PyExc_ValueError,
@@ -708,10 +654,10 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
       dim1[0] = 1;
 
       /* initialize y to a dummy array; never referenced */
-      y = (PyArrayObject *) PyArray_SimpleNew(1, dim1, PyArray_DOUBLE);
+      y = (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_DOUBLE);
 
       if ((x =
-           (PyArrayObject *) PyArray_CopyFromObject(px, PyArray_DOUBLE, 1,
+           (PyArrayObject *) PyArray_CopyFromObject(px, NPY_DOUBLE, 1,
                                                     2)) == NULL)
         {
           PYERR(PyExc_ValueError,
@@ -732,7 +678,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
     }                           /* x, y */
 
   if ((beta =
-       (PyArrayObject *) PyArray_CopyFromObject(initbeta, PyArray_DOUBLE, 1,
+       (PyArrayObject *) PyArray_CopyFromObject(initbeta, NPY_DOUBLE, 1,
                                                 1)) == NULL)
     {
       PYERR(PyExc_ValueError,
@@ -744,7 +690,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
     {
       ldwe = ld2we = 1;
       dim1[0] = n;
-      we = (PyArrayObject *) PyArray_SimpleNew(1, dim1, PyArray_DOUBLE);
+      we = (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_DOUBLE);
       ((double *)(we->data))[0] = -1.0;
     }
   else if (PyNumber_Check(pwe) && !PyArray_Check(pwe))
@@ -762,7 +708,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
       dim3[0] = nq;
       dim3[1] = 1;
       dim3[2] = 1;
-      we = (PyArrayObject *) PyArray_SimpleNew(3, dim3, PyArray_DOUBLE);
+      we = (PyArrayObject *) PyArray_SimpleNew(3, dim3, NPY_DOUBLE);
       if (implicit)
         {
           ((double *)(we->data))[0] = val;
@@ -778,7 +724,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
       /* we needs to be turned into an array */
 
       if ((we =
-           (PyArrayObject *) PyArray_CopyFromObject(pwe, PyArray_DOUBLE, 1,
+           (PyArrayObject *) PyArray_CopyFromObject(pwe, NPY_DOUBLE, 1,
                                                     3)) == NULL)
         {
           PYERR(PyExc_ValueError, "could not convert we to a suitable array");
@@ -841,7 +787,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
       ldwd = ld2wd = 1;
 
       dim1[0] = m;
-      wd = (PyArrayObject *) PyArray_SimpleNew(1, dim1, PyArray_DOUBLE);
+      wd = (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_DOUBLE);
       ((double *)(wd->data))[0] = -1.0;
     }
   else if (PyNumber_Check(pwd) && !PyArray_Check(pwd))
@@ -859,7 +805,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
       dim3[0] = 1;
       dim3[1] = 1;
       dim3[2] = m;
-      wd = (PyArrayObject *) PyArray_SimpleNew(3, dim3, PyArray_DOUBLE);
+      wd = (PyArrayObject *) PyArray_SimpleNew(3, dim3, NPY_DOUBLE);
       ((double *)(wd->data))[0] = -val;
       ldwd = ld2wd = 1;
     }
@@ -868,7 +814,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
       /* wd needs to be turned into an array */
 
       if ((wd =
-           (PyArrayObject *) PyArray_CopyFromObject(pwd, PyArray_DOUBLE, 1,
+           (PyArrayObject *) PyArray_CopyFromObject(pwd, NPY_DOUBLE, 1,
                                                     3)) == NULL)
         {
           PYERR(PyExc_ValueError, "could not convert wd to a suitable array");
@@ -931,7 +877,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
   if (pifixb == NULL)
     {
       dim1[0] = np;
-      ifixb = (PyArrayObject *) PyArray_SimpleNew(1, dim1, PyArray_INT);
+      ifixb = (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_INT);
       *(int *)(ifixb->data) = -1;      /* set first element negative */
     }
   else
@@ -939,7 +885,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
       /* pifixb is a sequence as checked before */
 
       if ((ifixb =
-           (PyArrayObject *) PyArray_CopyFromObject(pifixb, PyArray_INT, 1,
+           (PyArrayObject *) PyArray_CopyFromObject(pifixb, NPY_INT, 1,
                                                     1)) == NULL)
         {
           PYERR(PyExc_ValueError,
@@ -957,7 +903,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
     {
       dim2[0] = m;
       dim2[1] = 1;
-      ifixx = (PyArrayObject *) PyArray_SimpleNew(2, dim2, PyArray_INT);
+      ifixx = (PyArrayObject *) PyArray_SimpleNew(2, dim2, NPY_INT);
       *(int *)(ifixx->data) = -1;      /* set first element negative */
       ldifx = 1;
     }
@@ -966,7 +912,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
       /* pifixx is a sequence as checked before */
 
       if ((ifixx =
-           (PyArrayObject *) PyArray_CopyFromObject(pifixx, PyArray_INT, 1,
+           (PyArrayObject *) PyArray_CopyFromObject(pifixx, NPY_INT, 1,
                                                     2)) == NULL)
         {
           PYERR(PyExc_ValueError,
@@ -1010,13 +956,13 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
   if (pstpb == NULL)
     {
       dim1[0] = np;
-      stpb = (PyArrayObject *) PyArray_SimpleNew(1, dim1, PyArray_DOUBLE);
+      stpb = (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_DOUBLE);
       *(double *)(stpb->data) = 0.0;
     }
   else                          /* pstpb is a sequence */
     {
       if ((stpb =
-           (PyArrayObject *) PyArray_CopyFromObject(pstpb, PyArray_DOUBLE, 1,
+           (PyArrayObject *) PyArray_CopyFromObject(pstpb, NPY_DOUBLE, 1,
                                                     1)) == NULL
           || stpb->dimensions[0] != np)
         {
@@ -1029,14 +975,14 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
     {
       dim2[0] = 1;
       dim2[1] = m;
-      stpd = (PyArrayObject *) PyArray_SimpleNew(2, dim2, PyArray_DOUBLE);
+      stpd = (PyArrayObject *) PyArray_SimpleNew(2, dim2, NPY_DOUBLE);
       *(double *)(stpd->data) = 0.0;
       ldstpd = 1;
     }
   else
     {
       if ((stpd =
-           (PyArrayObject *) PyArray_CopyFromObject(pstpd, PyArray_DOUBLE, 1,
+           (PyArrayObject *) PyArray_CopyFromObject(pstpd, NPY_DOUBLE, 1,
                                                     2)) == NULL)
         {
           PYERR(PyExc_ValueError,
@@ -1061,13 +1007,13 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
   if (psclb == NULL)
     {
       dim1[0] = np;
-      sclb = (PyArrayObject *) PyArray_SimpleNew(1, dim1, PyArray_DOUBLE);
+      sclb = (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_DOUBLE);
       *(double *)(sclb->data) = 0.0;
     }
   else                          /* psclb is a sequence */
     {
       if ((sclb =
-           (PyArrayObject *) PyArray_CopyFromObject(psclb, PyArray_DOUBLE, 1,
+           (PyArrayObject *) PyArray_CopyFromObject(psclb, NPY_DOUBLE, 1,
                                                     1)) == NULL
           || sclb->dimensions[0] != np)
         {
@@ -1080,14 +1026,14 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
     {
       dim2[0] = 1;
       dim2[1] = n;
-      scld = (PyArrayObject *) PyArray_SimpleNew(2, dim2, PyArray_DOUBLE);
+      scld = (PyArrayObject *) PyArray_SimpleNew(2, dim2, NPY_DOUBLE);
       *(double *)(scld->data) = 0.0;
       ldscld = 1;
     }
   else
     {
       if ((scld =
-           (PyArrayObject *) PyArray_CopyFromObject(pscld, PyArray_DOUBLE, 1,
+           (PyArrayObject *) PyArray_CopyFromObject(pscld, NPY_DOUBLE, 1,
                                                     2)) == NULL)
         {
           PYERR(PyExc_ValueError,
@@ -1158,7 +1104,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
   if (pwork != NULL)
     {
       if ((work =
-           (PyArrayObject *) PyArray_CopyFromObject(pwork, PyArray_DOUBLE, 1,
+           (PyArrayObject *) PyArray_CopyFromObject(pwork, NPY_DOUBLE, 1,
                                                     1)) == NULL)
         {
           PYERR(PyExc_ValueError,
@@ -1174,13 +1120,13 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
     {
       /* initialize our own work array */
       dim1[0] = lwork;
-      work = (PyArrayObject *) PyArray_SimpleNew(1, dim1, PyArray_DOUBLE);
+      work = (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_DOUBLE);
     }                           /* work */
 
   if (piwork != NULL)
     {
       if ((iwork =
-           (PyArrayObject *) PyArray_CopyFromObject(piwork, PyArray_INT, 1,
+           (PyArrayObject *) PyArray_CopyFromObject(piwork, NPY_INT, 1,
                                                     1)) == NULL)
         {
           PYERR(PyExc_ValueError,
@@ -1196,7 +1142,7 @@ PyObject *odr(PyObject * self, PyObject * args, PyObject * kwds)
     {
       /* initialize our own iwork array */
       dim1[0] = liwork;
-      iwork = (PyArrayObject *) PyArray_SimpleNew(1, dim1, PyArray_INT);
+      iwork = (PyArrayObject *) PyArray_SimpleNew(1, dim1, NPY_INT);
     }                           /* iwork */
 
   /* check if what JOB requests can be done with what the user has 
@@ -1307,52 +1253,26 @@ fail:
   return NULL;
 }
 
-static void check_args(int n, int m, int np, int nq,
-                       PyArrayObject * beta,
-                       PyArrayObject * y, int ldy,
-                       PyArrayObject * x, int ldx,
-                       PyArrayObject * we, int ldwe, int ld2we,
-                       PyArrayObject * wd, int ldwd, int ld2wd,
-                       PyArrayObject * ifixb, PyArrayObject * ifixx,
-                       int ldifx, int job, int ndigit, double taufac,
-                       double sstol, double partol, int maxit,
-                       PyArrayObject * stpb, PyArrayObject * stpd,
-                       int ldstpd, PyArrayObject * sclb,
-                       PyArrayObject * scld, int ldscld,
-                       PyArrayObject * work, int lwork,
-                       PyArrayObject * iwork, int liwork, int info)
+
+PyObject *set_exceptions(PyObject * self, PyObject * args, PyObject * kwds)
 {
-  PyObject *printdict;
+    PyObject *exc_error, *exc_stop;
 
-  printdict =
-    Py_BuildValue
-    ("{s:i,s:i,s:i,s:i,s:O,s:O,s:i,s:O,s:i,s:O,s:i,s:i,s:O,s:i,s:i,s:O,s:O,s:i,s:i,s:i,s:d,s:d,s:d,s:i,s:O,s:O,s:i,s:O,s:O,s:i,s:O,s:i,s:O,s:i,s:i}",
-     "n", n, "m", m, "np", np, "nq", nq, "beta", (PyObject *) beta, "y",
-     (PyObject *) y, "ldy", ldy, "x", (PyObject *) x, "ldx", ldx, "we",
-     (PyObject *) we, "ldwe", ldwe, "ld2we", ld2we, "wd", (PyObject *) wd,
-     "ldwd", ldwd, "ld2wd", ld2wd, "ifixb", (PyObject *) ifixb, "ifixx",
-     (PyObject *) ifixx, "ldifx", ldifx, "job", job, "ndigit", ndigit,
-     "taufac", taufac, "sstol", sstol, "partol", partol, "maxit", maxit,
-     "stpb", (PyObject *) stpb, "stpd", (PyObject *) stpd, "ldstpd", ldstpd,
-     "sclb", (PyObject *) sclb, "scld", (PyObject *) scld, "ldscld", ldscld,
-     "work", (PyObject *) work, "lwork", lwork, "iwork", (PyObject *) iwork,
-     "liwork", liwork, "info", info);
-  if (printdict == NULL)
-    {
-      PyErr_Print();
-      return;
-    }
+    if (!PyArg_ParseTuple(args, "OO", &exc_error, &exc_stop))
+	return NULL;
 
-  PyObject_Print(printdict, stdout, Py_PRINT_RAW);
-  printf("\n");
-  Py_XDECREF(printdict);
+    Py_INCREF(exc_stop);
+    Py_INCREF(exc_error);
+    odr_stop = exc_stop;
+    odr_error = exc_error;
+
+    Py_INCREF(Py_None);
+    return Py_None;
 }
 
-static char odr__doc__[] =
-  "odr(fcn, beta0, y, x,\nwe=None, wd=None, fjacb=None, fjacd=None,\nextra_args=None, ifixx=None, ifixb=None, job=0, iprint=0,\nerrfile=None, rptfile=None, ndigit=0,\ntaufac=0.0, sstol=-1.0, partol=-1.0,\nmaxit=-1, stpb=None, stpd=None,\nsclb=None, scld=None, work=None, iwork=None,\nfull_output=0)";
-
 static PyMethodDef methods[] = {
-  {"odr", (PyCFunction) odr, METH_VARARGS | METH_KEYWORDS, odr__doc__},
+  {"_set_exceptions", (PyCFunction) set_exceptions, METH_VARARGS, NULL},
+  {"odr", (PyCFunction) odr, METH_VARARGS | METH_KEYWORDS, NULL},
   {NULL, NULL},
 };
 
@@ -1372,30 +1292,15 @@ static struct PyModuleDef moduledef = {
 PyObject *PyInit___odrpack(void)
 {
     PyObject *m, *s, *d;
-
-    m = PyModule_Create(&moduledef);
     import_array();
-
-    d = PyModule_GetDict(m);
-    odr_error = PyErr_NewException("odr.odrpack.odr_error", NULL, NULL);
-    odr_stop = PyErr_NewException("odr.odrpack.odr_stop", NULL, NULL);
-    PyDict_SetItemString(d, "odr_error", odr_error);
-    PyDict_SetItemString(d, "odr_stop", odr_stop);
-
+    m = PyModule_Create(&moduledef);
     return m;
 }
 #else
 PyMODINIT_FUNC init__odrpack(void)
 {
-  PyObject *m, *d;
-
-  import_array();
-
-  m = Py_InitModule("__odrpack", methods);
-  d = PyModule_GetDict(m);
-  odr_error = PyErr_NewException("odr.odrpack.odr_error", NULL, NULL);
-  odr_stop = PyErr_NewException("odr.odrpack.odr_stop", NULL, NULL);
-  PyDict_SetItemString(d, "odr_error", odr_error);
-  PyDict_SetItemString(d, "odr_stop", odr_stop);
+    PyObject *m, *d;
+    import_array();
+    m = Py_InitModule("__odrpack", methods);
 }
 #endif
