@@ -1,80 +1,6 @@
 """
 Python wrappers for Orthogonal Distance Regression (ODRPACK).
 
-Classes
-=======
-
-Data -- stores the data and weights to fit against
-
-RealData -- stores data with standard deviations and covariance matrices
-
-Model -- stores the model and its related information
-
-Output -- stores all of the output from an ODR run
-
-ODR -- collects all data and runs the fitting routine
-
-
-Exceptions
-==========
-
-odr_error -- error sometimes raised inside odr() and can be raised in the
-    fitting functions to tell ODRPACK to halt the procedure
-
-odr_stop -- error to raise in fitting functions to tell ODRPACK that the data or
-    parameters given are invalid
-
-Use
-===
-
-Basic use:
-
-1) Define the function you want to fit against.
-::
-
-  def f(B, x):
-      ''' Linear function y = m*x + b '''
-      return B[0]*x + B[1]
-
-      # B is a vector of the parameters.
-      # x is an array of the current x values.
-      # x is same format as the x passed to Data or RealData.
-
-      # Return an array in the same format as y passed to Data or RealData.
-
-2) Create a Model.
-::
-
-  linear = Model(f)
-
-3) Create a Data or RealData instance.
-::
-
-  mydata = Data(x, y, wd=1./power(sx,2), we=1./power(sy,2))
-
-or
-
-::
-
-  mydata = RealData(x, y, sx=sx, sy=sy)
-
-4) Instantiate ODR with your data, model and initial parameter estimate.
-::
-
-  myodr = ODR(mydata, linear, beta0=[1., 2.])
-
-5) Run the fit.
-::
-
-  myoutput = myodr.run()
-
-6) Examine output.
-::
-
-  myoutput.pprint()
-
-Read the docstrings and the accompanying tests for more advanced usage.
-
 Notes
 =====
 
@@ -110,12 +36,51 @@ robert.kern@gmail.com
 
 """
 
+from __future__ import division, print_function, absolute_import
+
 import numpy
+from warnings import warn
 from scipy.odr import __odrpack
 
+__all__ = ['odr', 'OdrWarning', 'OdrError', 'OdrStop',
+           'Data', 'RealData', 'Model', 'Output', 'ODR',
+           'odr_error', 'odr_stop']
+
 odr = __odrpack.odr
-odr_error = __odrpack.odr_error
-odr_stop = __odrpack.odr_stop
+
+
+class OdrWarning(UserWarning):
+    """
+    Warning indicating that the data passed into
+    ODR will cause problems when passed into 'odr'
+    that the user should be aware of.
+    """
+    pass
+
+
+class OdrError(Exception):
+    """
+    Exception indicating an error in fitting.
+
+    This is raised by `scipy.odr` if an error occurs during fitting.
+    """
+    pass
+
+
+class OdrStop(Exception):
+    """
+    Exception stopping fitting.
+
+    You can raise this exception in your objective function to tell
+    `scipy.odr` to stop fitting.
+    """
+    pass
+
+# Backwards compatibility
+odr_error = OdrError
+odr_stop = OdrStop
+
+__odrpack._set_exceptions(OdrError, OdrStop)
 
 
 def _conv(obj, dtype=None):
@@ -136,7 +101,7 @@ def _conv(obj, dtype=None):
             return obj
 
 
-def report_error(info):
+def _report_error(info):
     """ Interprets the return code of the odr routine.
 
     Parameters
@@ -159,10 +124,10 @@ def report_error(info):
     if info >= 5:
         # questionable results or fatal error
 
-        I = (info/10000 % 10,
-             info/1000 % 10,
-             info/100 % 10,
-             info/10 % 10,
+        I = (info//10000 % 10,
+             info//1000 % 10,
+             info//100 % 10,
+             info//10 % 10,
              info % 10)
         problems = []
 
@@ -170,7 +135,7 @@ def report_error(info):
             if I[1] != 0:
                 problems.append('Derivatives possibly not correct')
             if I[2] != 0:
-                problems.append('Error occured in callback')
+                problems.append('Error occurred in callback')
             if I[3] != 0:
                 problems.append('Problem is not full rank at solution')
             problems.append(stopreason)
@@ -204,7 +169,7 @@ def report_error(info):
         elif I[0] == 4:
             problems.append('Error in derivatives')
         elif I[0] == 5:
-            problems.append('Error occured in callback')
+            problems.append('Error occurred in callback')
         elif I[0] == 6:
             problems.append('Numerical error detected')
 
@@ -215,141 +180,178 @@ def report_error(info):
 
 
 class Data(object):
-    """ The Data class stores the data to fit.
+    """
+    The data to fit.
 
-    Each argument is attached to the member of the instance of the same name.
-    The structures of x and y are described in the Model class docstring. If
-    y is an integer, then the Data instance can only be used to fit with
-    implicit models where the dimensionality of the response is equal to the
-    specified value of y. The structures of wd and we are described below. meta
-    is an freeform dictionary for application-specific use.
-
-    we weights the effect a deviation in the response variable has on the fit.
-    wd weights the effect a deviation in the input variable has on the fit. To
-    handle multidimensional inputs and responses easily, the structure of these
-    arguments has the n'th dimensional axis first. These arguments heavily use
-    the structured arguments feature of ODRPACK to conveniently and flexibly
-    support all options. See the ODRPACK User's Guide for a full explanation of
-    how these weights are used in the algorithm. Basically, a higher value of
-    the weight for a particular data point makes a deviation at that point more
-    detrimental to the fit.
-
-      we -- if we is a scalar, then that value is used for all data points (and
+    Parameters
+    ----------
+    x : array_like
+        Observed data for the independent variable of the regression
+    y : array_like, optional
+        If array-like, observed data for the dependent variable of the
+        regression. A scalar input implies that the model to be used on
+        the data is implicit.
+    we : array_like, optional
+        If `we` is a scalar, then that value is used for all data points (and
         all dimensions of the response variable).
-
-        If we is a rank-1 array of length q (the dimensionality of the response
-        variable), then this vector is the diagonal of the covariant weighting
-        matrix for all data points.
-
-        If we is a rank-1 array of length n (the number of data points), then
+        If `we` is a rank-1 array of length q (the dimensionality of the
+        response variable), then this vector is the diagonal of the covariant
+        weighting matrix for all data points.
+        If `we` is a rank-1 array of length n (the number of data points), then
         the i'th element is the weight for the i'th response variable
         observation (single-dimensional only).
-
-        If we is a rank-2 array of shape (q, q), then this is the full covariant
-        weighting matrix broadcast to each observation.
-
-        If we is a rank-2 array of shape (q, n), then we[:,i] is the diagonal of
-        the covariant weighting matrix for the i'th observation.
-
-        If we is a rank-3 array of shape (q, q, n), then we[:,:,i] is the full
-        specification of the covariant weighting matrix for each observation.
-
+        If `we` is a rank-2 array of shape (q, q), then this is the full
+        covariant weighting matrix broadcast to each observation.
+        If `we` is a rank-2 array of shape (q, n), then `we[:,i]` is the
+        diagonal of the covariant weighting matrix for the i'th observation.
+        If `we` is a rank-3 array of shape (q, q, n), then `we[:,:,i]` is the
+        full specification of the covariant weighting matrix for each
+        observation.
         If the fit is implicit, then only a positive scalar value is used.
-
-      wd -- if wd is a scalar, then that value is used for all data points
-        (and all dimensions of the input variable). If wd = 0, then the
+    wd : array_like, optional
+        If `wd` is a scalar, then that value is used for all data points
+        (and all dimensions of the input variable). If `wd` = 0, then the
         covariant weighting matrix for each observation is set to the identity
         matrix (so each dimension of each observation has the same weight).
-
-        If wd is a rank-1 array of length m (the dimensionality of the input
+        If `wd` is a rank-1 array of length m (the dimensionality of the input
         variable), then this vector is the diagonal of the covariant weighting
         matrix for all data points.
-
-        If wd is a rank-1 array of length n (the number of data points), then
+        If `wd` is a rank-1 array of length n (the number of data points), then
         the i'th element is the weight for the i'th input variable observation
         (single-dimensional only).
+        If `wd` is a rank-2 array of shape (m, m), then this is the full
+        covariant weighting matrix broadcast to each observation.
+        If `wd` is a rank-2 array of shape (m, n), then `wd[:,i]` is the
+        diagonal of the covariant weighting matrix for the i'th observation.
+        If `wd` is a rank-3 array of shape (m, m, n), then `wd[:,:,i]` is the
+        full specification of the covariant weighting matrix for each
+        observation.
+    fix : array_like of ints, optional
+        The `fix` argument is the same as ifixx in the class ODR. It is an
+        array of integers with the same shape as data.x that determines which
+        input observations are treated as fixed. One can use a sequence of
+        length m (the dimensionality of the input observations) to fix some
+        dimensions for all observations. A value of 0 fixes the observation,
+        a value > 0 makes it free.
+    meta : dict, optional
+        Free-form dictionary for metadata.
 
-        If wd is a rank-2 array of shape (m, m), then this is the full covariant
-        weighting matrix broadcast to each observation.
+    Notes
+    -----
+    Each argument is attached to the member of the instance of the same name.
+    The structures of `x` and `y` are described in the Model class docstring.
+    If `y` is an integer, then the Data instance can only be used to fit with
+    implicit models where the dimensionality of the response is equal to the
+    specified value of `y`.
 
-        If wd is a rank-2 array of shape (m, n), then wd[:,i] is the diagonal of
-        the covariant weighting matrix for the i'th observation.
+    The `we` argument weights the effect a deviation in the response variable
+    has on the fit.  The `wd` argument weights the effect a deviation in the
+    input variable has on the fit. To handle multidimensional inputs and
+    responses easily, the structure of these arguments has the n'th
+    dimensional axis first. These arguments heavily use the structured
+    arguments feature of ODRPACK to conveniently and flexibly support all
+    options. See the ODRPACK User's Guide for a full explanation of how these
+    weights are used in the algorithm. Basically, a higher value of the weight
+    for a particular data point makes a deviation at that point more
+    detrimental to the fit.
 
-        If wd is a rank-3 array of shape (m, m, n), then wd[:,:,i] is the full
-        specification of the covariant weighting matrix for each observation.
-
-      fix -- fix is the same as ifixx in the class ODR. It is an array of integers
-        with the same shape as data.x that determines which input observations
-        are treated as fixed. One can use a sequence of length m (the
-        dimensionality of the input observations) to fix some dimensions for all
-        observations. A value of 0 fixes the observation, a value > 0 makes it
-        free.
-
-      meta -- optional, freeform dictionary for metadata
     """
 
     def __init__(self, x, y=None, we=None, wd=None, fix=None, meta={}):
         self.x = _conv(x)
+
+        if not isinstance(self.x, numpy.ndarray):
+            raise ValueError(("Expected an 'ndarray' of data for 'x', "
+                              "but instead got data of type '{name}'").format(
+                    name=type(self.x).__name__))
+
         self.y = _conv(y)
         self.we = _conv(we)
         self.wd = _conv(wd)
         self.fix = _conv(fix)
         self.meta = meta
 
-
     def set_meta(self, **kwds):
         """ Update the metadata dictionary with the keywords and data provided
         by keywords.
 
-        Example
-        -------
-        data.set_meta(lab="Ph 7; Lab 26", title="Ag110 + Ag108 Decay")
+        Examples
+        --------
+        ::
+
+            data.set_meta(lab="Ph 7; Lab 26", title="Ag110 + Ag108 Decay")
         """
 
         self.meta.update(kwds)
 
-
     def __getattr__(self, attr):
-        """ Dispatch aatribute access to the metadata dictionary.
+        """ Dispatch attribute access to the metadata dictionary.
         """
-
-        if attr in self.meta.keys():
+        if attr in self.meta:
             return self.meta[attr]
         else:
             raise AttributeError("'%s' not in metadata" % attr)
 
 
 class RealData(Data):
-    """ The RealData class stores the weightings as actual standard deviations
-    and/or covariances.
+    """
+    The data, with weightings as actual standard deviations and/or
+    covariances.
 
-    The weights needed for ODRPACK are generated on-the-fly with __getattr__
-    trickery.
+    Parameters
+    ----------
+    x : array_like
+        Observed data for the independent variable of the regression
+    y : array_like, optional
+        If array-like, observed data for the dependent variable of the
+        regression. A scalar input implies that the model to be used on
+        the data is implicit.
+    sx : array_like, optional
+        Standard deviations of `x`.
+        `sx` are standard deviations of `x` and are converted to weights by
+        dividing 1.0 by their squares.
+    sy : array_like, optional
+        Standard deviations of `y`.
+        `sy` are standard deviations of `y` and are converted to weights by
+        dividing 1.0 by their squares.
+    covx : array_like, optional
+        Covariance of `x`
+        `covx` is an array of covariance matrices of `x` and are converted to
+        weights by performing a matrix inversion on each observation's
+        covariance matrix.
+    covy : array_like, optional
+        Covariance of `y`
+        `covy` is an array of covariance matrices and are converted to
+        weights by performing a matrix inversion on each observation's
+        covariance matrix.
+    fix : array_like, optional
+        The argument and member fix is the same as Data.fix and ODR.ifixx:
+        It is an array of integers with the same shape as `x` that
+        determines which input observations are treated as fixed. One can
+        use a sequence of length m (the dimensionality of the input
+        observations) to fix some dimensions for all observations. A value
+        of 0 fixes the observation, a value > 0 makes it free.
+    meta : dict, optional
+        Free-form dictionary for metadata.
 
-    sx and sy are standard deviations of x and y and are converted to weights by
-    dividing 1.0 by their squares.
+    Notes
+    -----
+    The weights `wd` and `we` are computed from provided values as follows:
 
-      E.g.  wd = 1./numpy.power(sx, 2)
+    `sx` and `sy` are converted to weights by dividing 1.0 by their squares.
+    For example, ``wd = 1./numpy.power(`sx`, 2)``.
 
-    covx and covy are arrays of covariance matrices and are converted to weights
-    by performing a matrix inversion on each observation's covariance matrix.
+    `covx` and `covy` are arrays of covariance matrices and are converted to
+    weights by performing a matrix inversion on each observation's covariance
+    matrix.  For example, ``we[i] = numpy.linalg.inv(covy[i])``.
 
-      E.g.  we[i] = numpy.linalg.inv(covy[i])  # i in range(len(covy))
-                                               #   if covy.shape == (n,q,q)
+    These arguments follow the same structured argument conventions as wd and
+    we only restricted by their natures: `sx` and `sy` can't be rank-3, but
+    `covx` and `covy` can be.
 
-    These arguments follow the same structured argument conventions as wd and we
-    only restricted by their natures: sx and sy can't be rank-3, but covx and
-    covy can be.
+    Only set *either* `sx` or `covx` (not both). Setting both will raise an
+    exception.  Same with `sy` and `covy`.
 
-    Only set *either* sx or covx (not both). Setting both will raise an
-    exception.  Same with sy and covy.
-
-    The argument and member fix is the same as Data.fix and ODR.ifixx:
-      It is an array of integers with the same shape as data.x that determines
-      which input observations are treated as fixed. One can use a sequence of
-      length m (the dimensionality of the input observations) to fix some
-      dimensions for all observations. A value of 0 fixes the observation,
-      a value > 0 makes it free.
     """
 
     def __init__(self, x, y=None, sx=None, sy=None, covx=None, covy=None,
@@ -371,6 +373,12 @@ class RealData(Data):
             self._ga_flags['we'] = 'covy'
 
         self.x = _conv(x)
+
+        if not isinstance(self.x, numpy.ndarray):
+            raise ValueError(("Expected an 'ndarray' of data for 'x', "
+                              "but instead got data of type '{name}'").format(
+                    name=type(self.x).__name__))
+
         self.y = _conv(y)
         self.sx = _conv(sx)
         self.sy = _conv(sy)
@@ -379,7 +387,6 @@ class RealData(Data):
         self.fix = _conv(fix)
         self.meta = meta
 
-
     def _sd2wt(self, sd):
         """ Convert standard deviation to weights.
         """
@@ -402,16 +409,14 @@ class RealData(Data):
 
             return weights
 
-
     def __getattr__(self, attr):
-        lookup_tbl = {('wd', 'sx'):  (self._sd2wt, self.sx),
+        lookup_tbl = {('wd', 'sx'): (self._sd2wt, self.sx),
                       ('wd', 'covx'): (self._cov2wt, self.covx),
-                      ('we', 'sy'):  (self._sd2wt, self.sy),
+                      ('we', 'sy'): (self._sd2wt, self.sy),
                       ('we', 'covy'): (self._cov2wt, self.covy)}
 
-
         if attr not in ('wd', 'we'):
-            if attr in self.meta.keys():
+            if attr in self.meta:
                 return self.meta[attr]
             else:
                 raise AttributeError("'%s' not in metadata" % attr)
@@ -419,7 +424,7 @@ class RealData(Data):
             func, arg = lookup_tbl[(attr, self._ga_flags[attr])]
 
             if arg is not None:
-                return apply(func, (arg,))
+                return func(*(arg,))
             else:
                 return None
 
@@ -433,63 +438,70 @@ class Model(object):
     can provide a function that will provide reasonable starting values
     for the fit parameters possibly given the set of data.
 
-    The initialization method stores these into members of the same name.
-
-      fcn -- fit function
+    Parameters
+    ----------
+    fcn : function
           fcn(beta, x) --> y
+    fjacb : function
+          Jacobian of fcn wrt the fit parameters beta.
 
-      fjacb -- Jacobian of fcn wrt the fit parameters beta
           fjacb(beta, x) --> @f_i(x,B)/@B_j
+    fjacd : function
+          Jacobian of fcn wrt the (possibly multidimensional) input
+          variable.
 
-      fjacd -- Jacobian of fcn wrt the (possibly multidimensional) input
-          variable
           fjacd(beta, x) --> @f_i(x,B)/@x_j
+    extra_args : tuple, optional
+          If specified, `extra_args` should be a tuple of extra
+          arguments to pass to `fcn`, `fjacb`, and `fjacd`. Each will be called
+          by `apply(fcn, (beta, x) + extra_args)`
+    estimate : array_like of rank-1
+          Provides estimates of the fit parameters from the data
 
-      extra_args -- if specified, extra_args should be a tuple of extra
-          arguments to pass to fcn, fjacb, and fjacd. Each will be called
-          like the following: apply(fcn, (beta, x) + extra_args)
-
-      estimate -- provide estimates of the fit parameters from the data:
           estimate(data) --> estbeta
-
-      implicit -- boolean variable which, if TRUE, specifies that the model
-          is implicit; i.e fcn(beta, x) ~= 0 and there is no y data to fit
+    implicit : boolean
+          If TRUE, specifies that the model
+          is implicit; i.e `fcn(beta, x)` ~= 0 and there is no y data to fit
           against
-
-      meta -- optional
+    meta : dict, optional
           freeform dictionary of metadata for the model
 
-    Note that the fcn, fjacb, and fjacd operate on NumPy arrays and return
-    a NumPy array. The `estimate` object takes an instance of the Data class.
+    Notes
+    -----
+    Note that the `fcn`, `fjacb`, and `fjacd` operate on NumPy arrays and
+    return a NumPy array. The `estimate` object takes an instance of the
+    Data class.
 
-    Here are the rules for the shapes of the argument and return arrays:
+    Here are the rules for the shapes of the argument and return
+    arrays of the callback functions:
 
-      x -- if the input data is single-dimensional, then x is rank-1
-        array; i.e. x = array([1, 2, 3, ...]); x.shape = (n,)
-        If the input data is multi-dimensional, then x is a rank-2 array;
-        i.e., x = array([[1, 2, ...], [2, 4, ...]]); x.shape = (m, n) In
-        all cases, it has the same shape as the input data array passed to
-        odr(). m is the dimensionality of the input data, n is the number
+    `x`
+        if the input data is single-dimensional, then `x` is rank-1
+        array; i.e. ``x = array([1, 2, 3, ...]); x.shape = (n,)``
+        If the input data is multi-dimensional, then `x` is a rank-2 array;
+        i.e., ``x = array([[1, 2, ...], [2, 4, ...]]); x.shape = (m, n)``.
+        In all cases, it has the same shape as the input data array passed to
+        `odr`. `m` is the dimensionality of the input data, `n` is the number
         of observations.
-
-      y -- if the response variable is single-dimensional, then y is a
-        rank-1 array, i.e., y = array([2, 4, ...]); y.shape = (n,)
-        If the response variable is multi-dimensional, then y is a rank-2
-        array, i.e.,  y = array([[2, 4, ...], [3, 6, ...]]); y.shape =
-        (q, n) where q is the dimensionality of the response variable.
-
-      beta -- rank-1 array of length p where p is the number of parameters;
-        i.e. beta = array([B_1, B_2, ..., B_p])
-
-      fjacb -- if the response variable is multi-dimensional, then the
-        return array's shape is (q, p, n) such that fjacb(x,beta)[l,k,i] =
-        @f_l(X,B)/@B_k evaluated at the i'th data point.  If q == 1, then
-        the return array is only rank-2 and with shape (p, n).
-
-      fjacd -- as with fjacb, only the return array's shape is (q, m, n)
-        such that fjacd(x,beta)[l,j,i] = @f_l(X,B)/@X_j at the i'th data
-        point.  If q == 1, then the return array's shape is (m, n). If
-        m == 1, the shape is (q, n). If m == q == 1, the shape is (n,).
+    `y`
+        if the response variable is single-dimensional, then `y` is a
+        rank-1 array, i.e., ``y = array([2, 4, ...]); y.shape = (n,)``.
+        If the response variable is multi-dimensional, then `y` is a rank-2
+        array, i.e., ``y = array([[2, 4, ...], [3, 6, ...]]); y.shape =
+        (q, n)`` where `q` is the dimensionality of the response variable.
+    `beta`
+        rank-1 array of length `p` where `p` is the number of parameters;
+        i.e. ``beta = array([B_1, B_2, ..., B_p])``
+    `fjacb`
+        if the response variable is multi-dimensional, then the
+        return array's shape is `(q, p, n)` such that ``fjacb(x,beta)[l,k,i] =
+        d f_l(X,B)/d B_k`` evaluated at the i'th data point.  If `q == 1`, then
+        the return array is only rank-2 and with shape `(p, n)`.
+    `fjacd`
+        as with fjacb, only the return array's shape is `(q, m, n)`
+        such that ``fjacd(x,beta)[l,j,i] = d f_l(X,B)/d X_j`` at the i'th data
+        point.  If `q == 1`, then the return array's shape is `(m, n)`. If
+        `m == 1`, the shape is (q, n). If `m == q == 1`, the shape is `(n,)`.
 
     """
 
@@ -508,24 +520,22 @@ class Model(object):
         self.implicit = implicit
         self.meta = meta
 
-
     def set_meta(self, **kwds):
         """ Update the metadata dictionary with the keywords and data provided
         here.
 
-        Example
-        -------
+        Examples
+        --------
         set_meta(name="Exponential", equation="y = a exp(b x) + c")
         """
 
         self.meta.update(kwds)
 
-
     def __getattr__(self, attr):
         """ Dispatch attribute access to the metadata.
         """
 
-        if attr in self.meta.keys():
+        if attr in self.meta:
             return self.meta[attr]
         else:
             raise AttributeError("'%s' not in metadata" % attr)
@@ -535,9 +545,6 @@ class Output(object):
     """
     The Output class stores the output of an ODR run.
 
-    Takes one argument for initialization, the return value from the
-    function `odr`.
-
     Attributes
     ----------
     beta : ndarray
@@ -577,8 +584,9 @@ class Output(object):
 
     Notes
     -----
-    The attributes listed as "optional" above are only present if `odr` was run
-    with ``full_output=1``.
+    Takes one argument for initialization, the return value from the
+    function `odr`. The attributes listed as "optional" above are only
+    present if `odr` was run with ``full_output=1``.
 
     """
 
@@ -590,22 +598,21 @@ class Output(object):
         if len(output) == 4:
             # full output
             self.__dict__.update(output[3])
-            self.stopreason = report_error(self.info)
-
+            self.stopreason = _report_error(self.info)
 
     def pprint(self):
         """ Pretty-print important results.
         """
 
-        print 'Beta:', self.beta
-        print 'Beta Std Error:', self.sd_beta
-        print 'Beta Covariance:', self.cov_beta
+        print('Beta:', self.beta)
+        print('Beta Std Error:', self.sd_beta)
+        print('Beta Covariance:', self.cov_beta)
         if hasattr(self, 'info'):
-            print 'Residual Variance:',self.res_var
-            print 'Inverse Condition #:', self.inv_condnum
-            print 'Reason(s) for Halting:'
+            print('Residual Variance:',self.res_var)
+            print('Inverse Condition #:', self.inv_condnum)
+            print('Reason(s) for Halting:')
             for r in self.stopreason:
-                print '  %s' % r
+                print('  %s' % r)
 
 
 class ODR(object):
@@ -618,90 +625,100 @@ class ODR(object):
 
     Parameters
     ----------
-    data:
+    data : Data class instance
         instance of the Data class
-    model:
+    model : Model class instance
         instance of the Model class
-    beta0:
+
+    Other Parameters
+    ----------------
+    beta0 : array_like of rank-1
         a rank-1 sequence of initial parameter values. Optional if
         model provides an "estimate" function to estimate these values.
-    delta0: optional
+    delta0 : array_like of floats of rank-1, optional
         a (double-precision) float array to hold the initial values of
         the errors in the input variables. Must be same shape as data.x
-    ifixb: optional
+    ifixb : array_like of ints of rank-1, optional
         sequence of integers with the same length as beta0 that determines
         which parameters are held fixed. A value of 0 fixes the parameter,
         a value > 0 makes the parameter free.
-    ifixx: optional
+    ifixx : array_like of ints with same shape as data.x, optional
         an array of integers with the same shape as data.x that determines
         which input observations are treated as fixed. One can use a sequence
         of length m (the dimensionality of the input observations) to fix some
         dimensions for all observations. A value of 0 fixes the observation,
         a value > 0 makes it free.
-    job: optional
+    job : int, optional
         an integer telling ODRPACK what tasks to perform. See p. 31 of the
         ODRPACK User's Guide if you absolutely must set the value here. Use the
         method set_job post-initialization for a more readable interface.
-    iprint: optional
+    iprint : int, optional
         an integer telling ODRPACK what to print. See pp. 33-34 of the
         ODRPACK User's Guide if you absolutely must set the value here. Use the
         method set_iprint post-initialization for a more readable interface.
-    errfile: optional
+    errfile : str, optional
         string with the filename to print ODRPACK errors to. *Do Not Open
         This File Yourself!*
-    rptfile: optional
+    rptfile : str, optional
         string with the filename to print ODRPACK summaries to. *Do Not
         Open This File Yourself!*
-    ndigit: optional
+    ndigit : int, optional
         integer specifying the number of reliable digits in the computation
         of the function.
-    taufac: optional
+    taufac : float, optional
         float specifying the initial trust region. The default value is 1.
         The initial trust region is equal to taufac times the length of the
         first computed Gauss-Newton step. taufac must be less than 1.
-    sstol: optional
+    sstol : float, optional
         float specifying the tolerance for convergence based on the relative
         change in the sum-of-squares. The default value is eps**(1/2) where eps
         is the smallest value such that 1 + eps > 1 for double precision
         computation on the machine. sstol must be less than 1.
-    partol: optional
+    partol : float, optional
         float specifying the tolerance for convergence based on the relative
         change in the estimated parameters. The default value is eps**(2/3) for
-        explicit models and eps**(1/3) for implicit models. partol must be less
+        explicit models and ``eps**(1/3)`` for implicit models. partol must be less
         than 1.
-    maxit: optional
+    maxit : int, optional
         integer specifying the maximum number of iterations to perform. For
         first runs, maxit is the total number of iterations performed and
         defaults to 50.  For restarts, maxit is the number of additional
         iterations to perform and defaults to 10.
-    stpb: optional
-        sequence (len(stpb) == len(beta0)) of relative step sizes to compute
+    stpb : array_like, optional
+        sequence (``len(stpb) == len(beta0)``) of relative step sizes to compute
         finite difference derivatives wrt the parameters.
-    stpd: optional
-        array (stpd.shape == data.x.shape or stpd.shape == (m,)) of relative
+    stpd : optional
+        array (``stpd.shape == data.x.shape`` or ``stpd.shape == (m,)``) of relative
         step sizes to compute finite difference derivatives wrt the input
         variable errors. If stpd is a rank-1 array with length m (the
         dimensionality of the input variable), then the values are broadcast to
         all observations.
-    sclb: optional
-        sequence (len(stpb) == len(beta0)) of scaling factors for the
+    sclb : array_like, optional
+        sequence (``len(stpb) == len(beta0)``) of scaling factors for the
         parameters.  The purpose of these scaling factors are to scale all of
         the parameters to around unity. Normally appropriate scaling factors
         are computed if this argument is not specified. Specify them yourself
         if the automatic procedure goes awry.
-    scld: optional
+    scld : array_like, optional
         array (scld.shape == data.x.shape or scld.shape == (m,)) of scaling
         factors for the *errors* in the input variables. Again, these factors
         are automatically computed if you do not provide them. If scld.shape ==
         (m,), then the scaling factors are broadcast to all observations.
-    work: optional
+    work : ndarray, optional
         array to hold the double-valued working data for ODRPACK. When
         restarting, takes the value of self.output.work.
-    iwork: optional
+    iwork : ndarray, optional
         array to hold the integer-valued working data for ODRPACK. When
         restarting, takes the value of self.output.iwork.
-    output:
-        an instance if the Output class containing all of the returned
+
+    Attributes
+    ----------
+    data : Data
+        The data for this fit
+    model : Model
+        The model used in fit
+    output : Output
+        An instance if the Output class containing all of the returned
         data from an invocation of ODR.run() or ODR.restart()
 
     """
@@ -760,16 +777,16 @@ class ODR(object):
         if isinstance(self.data.y, numpy.ndarray):
             y_s = list(self.data.y.shape)
             if self.model.implicit:
-                raise odr_error("an implicit model cannot use response data")
+                raise OdrError("an implicit model cannot use response data")
         else:
             # implicit model with q == self.data.y
             y_s = [self.data.y, x_s[-1]]
             if not self.model.implicit:
-                raise odr_error("an explicit model needs response data")
+                raise OdrError("an explicit model needs response data")
             self.set_job(fit_type=1)
 
         if x_s[-1] != y_s[-1]:
-            raise odr_error("number of observations do not match")
+            raise OdrError("number of observations do not match")
 
         n = x_s[-1]
 
@@ -812,26 +829,31 @@ class ODR(object):
         res = self.model.fcn(*arglist)
 
         if res.shape not in fcn_perms:
-            print res.shape
-            print fcn_perms
-            raise odr_error("fcn does not output %s-shaped array" % y_s)
+            print(res.shape)
+            print(fcn_perms)
+            raise OdrError("fcn does not output %s-shaped array" % y_s)
 
         if self.model.fjacd is not None:
             res = self.model.fjacd(*arglist)
             if res.shape not in fjacd_perms:
-                raise odr_error(
-                    "fjacd does not output %s-shaped array" % (q, m, n))
+                raise OdrError(
+                    "fjacd does not output %s-shaped array" % repr((q, m, n)))
         if self.model.fjacb is not None:
             res = self.model.fjacb(*arglist)
             if res.shape not in fjacb_perms:
-                raise odr_error(
-                    "fjacb does not output %s-shaped array" % (q, p, n))
+                raise OdrError(
+                    "fjacb does not output %s-shaped array" % repr((q, p, n)))
 
         # check shape of delta0
 
         if self.delta0 is not None and self.delta0.shape != self.data.x.shape:
-            raise odr_error(
-                "delta0 is not a %s-shaped array" % self.data.x.shape)
+            raise OdrError(
+                "delta0 is not a %s-shaped array" % repr(self.data.x.shape))
+
+        if self.data.x.size == 0:
+            warn(("Empty data detected for ODR instance. "
+                  "Do not expect any fitting to occur"),
+                 OdrWarning)
 
     def _gen_work(self):
         """ Generate a suitable work array if one does not already exist.
@@ -878,7 +900,6 @@ class ODR(object):
         else:
             self.work = numpy.zeros((lwork,), float)
 
-
     def set_job(self, fit_type=None, deriv=None, var_calc=None,
         del_init=None, restart=None):
         """
@@ -938,10 +959,10 @@ class ODR(object):
         if self.job is None:
             job_l = [0, 0, 0, 0, 0]
         else:
-            job_l = [self.job / 10000 % 10,
-                     self.job / 1000 % 10,
-                     self.job / 100 % 10,
-                     self.job / 10 % 10,
+            job_l = [self.job // 10000 % 10,
+                     self.job // 1000 % 10,
+                     self.job // 100 % 10,
+                     self.job // 10 % 10,
                      self.job % 10]
 
         if fit_type in (0, 1, 2):
@@ -958,7 +979,6 @@ class ODR(object):
         self.job = (job_l[0]*10000 + job_l[1]*1000 +
                     job_l[2]*100 + job_l[3]*10 + job_l[4])
 
-
     def set_iprint(self, init=None, so_init=None,
         iter=None, so_iter=None, iter_step=None, final=None, so_final=None):
         """ Set the iprint parameter for the printing of computation reports.
@@ -988,26 +1008,26 @@ class ODR(object):
         if self.iprint is None:
             self.iprint = 0
 
-        ip = [self.iprint / 1000 % 10,
-              self.iprint / 100 % 10,
-              self.iprint / 10 % 10,
+        ip = [self.iprint // 1000 % 10,
+              self.iprint // 100 % 10,
+              self.iprint // 10 % 10,
               self.iprint % 10]
 
         # make a list to convert iprint digits to/from argument inputs
         #                   rptfile, stdout
-        ip2arg = [[0, 0], # none,  none
-                  [1, 0], # short, none
-                  [2, 0], # long,  none
-                  [1, 1], # short, short
-                  [2, 1], # long,  short
-                  [1, 2], # short, long
-                  [2, 2]] # long,  long
+        ip2arg = [[0, 0],  # none,  none
+                  [1, 0],  # short, none
+                  [2, 0],  # long,  none
+                  [1, 1],  # short, short
+                  [2, 1],  # long,  short
+                  [1, 2],  # short, long
+                  [2, 2]]  # long,  long
 
         if (self.rptfile is None and
             (so_init is not None or
              so_iter is not None or
              so_final is not None)):
-            raise odr_error(
+            raise OdrError(
                 "no rptfile specified, cannot output to stdout twice")
 
         iprint_l = ip2arg[ip[0]] + ip2arg[ip[1]] + ip2arg[ip[3]]
@@ -1035,7 +1055,6 @@ class ODR(object):
 
         self.iprint = ip[0]*1000 + ip[1]*100 + ip[2]*10 + ip[3]
 
-
     def run(self):
         """ Run the fitting routine with all of the information given.
 
@@ -1051,7 +1070,7 @@ class ODR(object):
                  'ndigit', 'taufac', 'sstol', 'partol', 'maxit', 'stpb',
                  'stpd', 'sclb', 'scld', 'work', 'iwork']
 
-        if self.delta0 is not None and self.job % 1000 / 10 == 1:
+        if self.delta0 is not None and self.job % 1000 // 10 == 1:
             # delta0 provided and fit is not a restart
             self._gen_work()
 
@@ -1077,11 +1096,10 @@ class ODR(object):
             if obj is not None:
                 kwds[attr] = obj
 
-        self.output = Output(apply(odr, args, kwds))
+        self.output = Output(odr(*args, **kwds))
 
         return self.output
 
-
     def restart(self, iter=None):
         """ Restarts the run with iter more iterations.
 
@@ -1097,7 +1115,7 @@ class ODR(object):
         """
 
         if self.output is None:
-            raise odr_error("cannot restart: run() has not been called before")
+            raise OdrError("cannot restart: run() has not been called before")
 
         self.set_job(restart=1)
         self.work = self.output.work
@@ -1106,5 +1124,3 @@ class ODR(object):
         self.maxit = iter
 
         return self.run()
-
-#### EOF #######################################################################
