@@ -1,11 +1,12 @@
 
-/*! @file cgsitf.c
+/*! @file cgsitrf.c
  * \brief Computes an ILU factorization of a general sparse matrix
  *
  * <pre>
- * -- SuperLU routine (version 4.0) --
+ * -- SuperLU routine (version 4.1) --
  * Lawrence Berkeley National Laboratory.
  * June 30, 2009
+ *
  * </pre>
  */
 
@@ -100,6 +101,17 @@ int num_drop_L;
  *	    storage scheme, i.e., U has types: Stype = SLU_NC,
  *	    Dtype = SLU_C, Mtype = SLU_TRU.
  *
+ * Glu      (input/output) GlobalLU_t *
+ *          If options->Fact == SamePattern_SameRowPerm, it is an input;
+ *              The matrix A will be factorized assuming that a 
+ *              factorization of a matrix with the same sparsity pattern
+ *              and similar numerical values was performed prior to this one.
+ *              Therefore, this factorization will reuse both row and column
+ *		scaling factors R and C, both row and column permutation
+ *		vectors perm_r and perm_c, and the L & U data structures
+ *		set up from the previous factorization.
+ *          Otherwise, it is an output.
+ *
  * stat     (output) SuperLUStat_t*
  *	    Record the statistics on runtime and floating-point operation count.
  *	    See slu_util.h for the definition of 'SuperLUStat_t'.
@@ -164,7 +176,9 @@ int num_drop_L;
 void
 cgsitrf(superlu_options_t *options, SuperMatrix *A, int relax, int panel_size,
 	int *etree, void *work, int lwork, int *perm_c, int *perm_r,
-	SuperMatrix *L, SuperMatrix *U, SuperLUStat_t *stat, int *info)
+	SuperMatrix *L, SuperMatrix *U, 
+    	GlobalLU_t *Glu, /* persistent to facilitate multiple factorizations */
+	SuperLUStat_t *stat, int *info)
 {
     /* Local working arrays */
     NCPformat *Astore;
@@ -192,8 +206,8 @@ cgsitrf(superlu_options_t *options, SuperMatrix *A, int relax, int panel_size,
     int       nzlumax;
     float    *amax; 
     complex    drop_sum;
-    static GlobalLU_t Glu; /* persistent to facilitate multiple factors. */
-    int       *iwork2;	   /* used by the second dropping rule */
+    float alpha, omega;  /* used in MILU, mimicing DRIC */
+    float    *swork2;	   /* used by the second dropping rule */
 
     /* Local scalars */
     fact_t    fact = options->Fact;
@@ -224,6 +238,7 @@ cgsitrf(superlu_options_t *options, SuperMatrix *A, int relax, int panel_size,
     int       nnzLj, nnzUj;
     double    tol_L = drop_tol, tol_U = drop_tol;
     complex zero = {0.0, 0.0};
+    float one = 1.0;
 
     /* Executable */	   
     iinfo    = 0;
@@ -238,14 +253,14 @@ cgsitrf(superlu_options_t *options, SuperMatrix *A, int relax, int panel_size,
 
     /* Allocate storage common to the factor routines */
     *info = cLUMemInit(fact, work, lwork, m, n, Astore->nnz, panel_size,
-		       gamma, L, U, &Glu, &iwork, &cwork);
+		       gamma, L, U, Glu, &iwork, &cwork);
     if ( *info ) return;
 
-    xsup    = Glu.xsup;
-    supno   = Glu.supno;
-    xlsub   = Glu.xlsub;
-    xlusup  = Glu.xlusup;
-    xusub   = Glu.xusub;
+    xsup    = Glu->xsup;
+    supno   = Glu->supno;
+    xlsub   = Glu->xlsub;
+    xlusup  = Glu->xlusup;
+    xusub   = Glu->xusub;
 
     SetIWork(m, n, panel_size, iwork, &segrep, &parent, &xplore,
 	     &repfnz, &panel_lsub, &marker_relax, &marker);
@@ -267,14 +282,15 @@ cgsitrf(superlu_options_t *options, SuperMatrix *A, int relax, int panel_size,
     for (k = 0; k < n; k++) iswap[k] = perm_c[k];
     amax = (float *) floatMalloc(panel_size);
     if (drop_rule & DROP_SECONDARY)
-	iwork2 = (int *)intMalloc(n);
+	swork2 = (float *)floatMalloc(n);
     else
-	iwork2 = NULL;
+	swork2 = NULL;
 
     nnzAj = 0;
     nnzLj = 0;
     nnzUj = 0;
-    last_drop = SUPERLU_MAX(min_mn - 2 * sp_ienv(3), (int)(min_mn * 0.95));
+    last_drop = SUPERLU_MAX(min_mn - 2 * sp_ienv(7), (int)(min_mn * 0.95));
+    alpha = pow((double)n, -1.0 / options->ILU_MILU_Dim);
 
     /* Identify relaxed snodes */
     relax_end = (int *) intMalloc(n);
@@ -335,7 +351,7 @@ cgsitrf(superlu_options_t *options, SuperMatrix *A, int relax, int panel_size,
 		/* Drop small rows */
                 stempv = (float *) tempv;
 		i = ilu_cdrop_row(options, first, last, tol_L, quota, &nnzLj,
-				  &fill_tol, &Glu, stempv, iwork2, 0);
+				  &fill_tol, Glu, stempv, swork2, 0);
 		/* Reset the parameters */
 		if (drop_rule & DROP_DYNAMIC) {
 		    if (gamma * nnzAj * (1.0 - 0.5 * (last + 1.0) / m)
@@ -355,7 +371,7 @@ cgsitrf(superlu_options_t *options, SuperMatrix *A, int relax, int panel_size,
 	     * -------------------------------------- */
 	    /* Determine the union of the row structure of the snode */
 	    if ( (*info = ilu_csnode_dfs(jcol, kcol, asub, xa_begin, xa_end,
-					 marker, &Glu)) != 0 )
+					 marker, Glu)) != 0 )
 		return;
 
 	    nextu    = xusub[jcol];
@@ -363,9 +379,9 @@ cgsitrf(superlu_options_t *options, SuperMatrix *A, int relax, int panel_size,
 	    jsupno   = supno[jcol];
 	    fsupc    = xsup[jsupno];
 	    new_next = nextlu + (xlsub[fsupc+1]-xlsub[fsupc])*(kcol-jcol+1);
-	    nzlumax = Glu.nzlumax;
+	    nzlumax = Glu->nzlumax;
 	    while ( new_next > nzlumax ) {
-		if ((*info = cLUMemXpand(jcol, nextlu, LUSUP, &nzlumax, &Glu)))
+		if ((*info = cLUMemXpand(jcol, nextlu, LUSUP, &nzlumax, Glu)))
 		    return;
 	    }
 
@@ -375,7 +391,7 @@ cgsitrf(superlu_options_t *options, SuperMatrix *A, int relax, int panel_size,
 		amax[0] = 0.0;
 		/* Scatter into SPA dense[*] */
 		for (k = xa_begin[icol]; k < xa_end[icol]; k++) {
-                    register float tmp = slu_c_abs1 (&a[k]);
+                    register float tmp = c_abs1 (&a[k]);
 		    if (tmp > amax[0]) amax[0] = tmp;
 		    dense[asub[k]] = a[k];
 		}
@@ -389,7 +405,7 @@ cgsitrf(superlu_options_t *options, SuperMatrix *A, int relax, int panel_size,
 		}
 
 		/* Numeric update within the snode */
-		csnode_bmod(icol, jsupno, fsupc, dense, tempv, &Glu, stat);
+		csnode_bmod(icol, jsupno, fsupc, dense, tempv, Glu, stat);
 
 		if (usepr) pivrow = iperm_r[icol];
 		fill_tol = pow(fill_ini, 1.0 - (double)icol / (double)min_mn);
@@ -397,7 +413,7 @@ cgsitrf(superlu_options_t *options, SuperMatrix *A, int relax, int panel_size,
 					  perm_r, iperm_c[icol], swap, iswap,
 					  marker_relax, &pivrow,
                                           amax[0] * fill_tol, milu, zero,
-                                          &Glu, stat)) ) {
+                                          Glu, stat)) ) {
 		    iinfo++;
 		    marker[pivrow] = kcol;
 		}
@@ -423,11 +439,11 @@ cgsitrf(superlu_options_t *options, SuperMatrix *A, int relax, int panel_size,
 	    /* symbolic factor on a panel of columns */
 	    ilu_cpanel_dfs(m, panel_size, jcol, A, perm_r, &nseg1,
                           dense, amax, panel_lsub, segrep, repfnz,
-                          marker, parent, xplore, &Glu);
+                          marker, parent, xplore, Glu);
 
 	    /* numeric sup-panel updates in topological order */
 	    cpanel_bmod(m, panel_size, jcol, nseg1, dense,
-			tempv, segrep, repfnz, &Glu, stat);
+			tempv, segrep, repfnz, Glu, stat);
 
 	    /* Sparse LU within the panel, and below panel diagonal */
 	    for (jj = jcol; jj < jcol + panel_size; jj++) {
@@ -440,33 +456,33 @@ cgsitrf(superlu_options_t *options, SuperMatrix *A, int relax, int panel_size,
 
 		if ((*info = ilu_ccolumn_dfs(m, jj, perm_r, &nseg,
 					     &panel_lsub[k], segrep, &repfnz[k],
-					     marker, parent, xplore, &Glu)))
+					     marker, parent, xplore, Glu)))
 		    return;
 
 		/* Numeric updates */
 		if ((*info = ccolumn_bmod(jj, (nseg - nseg1), &dense[k],
 					  tempv, &segrep[nseg1], &repfnz[k],
-					  jcol, &Glu, stat)) != 0) return;
+					  jcol, Glu, stat)) != 0) return;
 
 		/* Make a fill-in position if the column is entirely zero */
 		if (xlsub[jj + 1] == xlsub[jj]) {
 		    register int i, row;
 		    int nextl;
-		    int nzlmax = Glu.nzlmax;
-		    int *lsub = Glu.lsub;
+		    int nzlmax = Glu->nzlmax;
+		    int *lsub = Glu->lsub;
 		    int *marker2 = marker + 2 * m;
 
 		    /* Allocate memory */
 		    nextl = xlsub[jj] + 1;
 		    if (nextl >= nzlmax) {
-			int error = cLUMemXpand(jj, nextl, LSUB, &nzlmax, &Glu);
+			int error = cLUMemXpand(jj, nextl, LSUB, &nzlmax, Glu);
 			if (error) { *info = error; return; }
-			lsub = Glu.lsub;
+			lsub = Glu->lsub;
 		    }
 		    xlsub[jj + 1]++;
 		    assert(xlusup[jj]==xlusup[jj+1]);
 		    xlusup[jj + 1]++;
-		    Glu.lusup[xlusup[jj]] = zero;
+		    ((complex *) Glu->lusup)[xlusup[jj]] = zero;
 
 		    /* Choose a row index (pivrow) for fill-in */
 		    for (i = jj; i < n; i++)
@@ -495,8 +511,8 @@ cgsitrf(superlu_options_t *options, SuperMatrix *A, int relax, int panel_size,
 		if ((*info = ilu_ccopy_to_ucol(jj, nseg, segrep, &repfnz[k],
 					       perm_r, &dense[k], drop_rule,
 					       milu, amax[jj - jcol] * tol_U,
-					       quota, &drop_sum, &nnzUj, &Glu,
-					       iwork2)) != 0)
+					       quota, &drop_sum, &nnzUj, Glu,
+					       swork2)) != 0)
 		    return;
 
 		/* Reset the dropping threshold if required */
@@ -507,14 +523,18 @@ cgsitrf(superlu_options_t *options, SuperMatrix *A, int relax, int panel_size,
 			tol_U = SUPERLU_MAX(drop_tol, tol_U * 0.5);
 		}
 
-                cs_mult(&drop_sum, &drop_sum, MILU_ALPHA);
+		if (drop_sum.r != 0.0 && drop_sum.i != 0.0)
+		{
+                    omega = SUPERLU_MIN(2.0*(1.0-alpha)/c_abs1(&drop_sum), 1.0);
+                    cs_mult(&drop_sum, &drop_sum, omega);
+		}
 		if (usepr) pivrow = iperm_r[jj];
 		fill_tol = pow(fill_ini, 1.0 - (double)jj / (double)min_mn);
 		if ( (*info = ilu_cpivotL(jj, diag_pivot_thresh, &usepr, perm_r,
 					  iperm_c[jj], swap, iswap,
 					  marker_relax, &pivrow,
 					  amax[jj - jcol] * fill_tol, milu,
-					  drop_sum, &Glu, stat)) ) {
+					  drop_sum, Glu, stat)) ) {
 		    iinfo++;
 		    marker[m + pivrow] = jj;
 		    marker[2 * m + pivrow] = jj;
@@ -550,7 +570,7 @@ cgsitrf(superlu_options_t *options, SuperMatrix *A, int relax, int panel_size,
 		    /* Drop small rows */
                     stempv = (float *) tempv;
 		    i = ilu_cdrop_row(options, first, last, tol_L, quota,
-				      &nnzLj, &fill_tol, &Glu, stempv, iwork2,
+				      &nnzLj, &fill_tol, Glu, stempv, swork2,
 				      1);
 
 		    /* Reset the parameters */
@@ -586,10 +606,10 @@ cgsitrf(superlu_options_t *options, SuperMatrix *A, int relax, int panel_size,
 	    }
     }
 
-    ilu_countnz(min_mn, &nnzL, &nnzU, &Glu);
-    fixupL(min_mn, perm_r, &Glu);
+    ilu_countnz(min_mn, &nnzL, &nnzU, Glu);
+    fixupL(min_mn, perm_r, Glu);
 
-    cLUWorkFree(iwork, cwork, &Glu); /* Free work space and compress storage */
+    cLUWorkFree(iwork, cwork, Glu); /* Free work space and compress storage */
 
     if ( fact == SamePattern_SameRowPerm ) {
 	/* L and U structures may have changed due to possibly different
@@ -597,24 +617,27 @@ cgsitrf(superlu_options_t *options, SuperMatrix *A, int relax, int panel_size,
 	   There could also be memory expansions, so the array locations
 	   may have changed, */
 	((SCformat *)L->Store)->nnz = nnzL;
-	((SCformat *)L->Store)->nsuper = Glu.supno[n];
-	((SCformat *)L->Store)->nzval = Glu.lusup;
-	((SCformat *)L->Store)->nzval_colptr = Glu.xlusup;
-	((SCformat *)L->Store)->rowind = Glu.lsub;
-	((SCformat *)L->Store)->rowind_colptr = Glu.xlsub;
+	((SCformat *)L->Store)->nsuper = Glu->supno[n];
+	((SCformat *)L->Store)->nzval = (complex *) Glu->lusup;
+	((SCformat *)L->Store)->nzval_colptr = Glu->xlusup;
+	((SCformat *)L->Store)->rowind = Glu->lsub;
+	((SCformat *)L->Store)->rowind_colptr = Glu->xlsub;
 	((NCformat *)U->Store)->nnz = nnzU;
-	((NCformat *)U->Store)->nzval = Glu.ucol;
-	((NCformat *)U->Store)->rowind = Glu.usub;
-	((NCformat *)U->Store)->colptr = Glu.xusub;
+	((NCformat *)U->Store)->nzval = (complex *) Glu->ucol;
+	((NCformat *)U->Store)->rowind = Glu->usub;
+	((NCformat *)U->Store)->colptr = Glu->xusub;
     } else {
-	cCreate_SuperNode_Matrix(L, A->nrow, min_mn, nnzL, Glu.lusup,
-				 Glu.xlusup, Glu.lsub, Glu.xlsub, Glu.supno,
-				 Glu.xsup, SLU_SC, SLU_C, SLU_TRLU);
-	cCreate_CompCol_Matrix(U, min_mn, min_mn, nnzU, Glu.ucol,
-			       Glu.usub, Glu.xusub, SLU_NC, SLU_C, SLU_TRU);
+	cCreate_SuperNode_Matrix(L, A->nrow, min_mn, nnzL,
+              (complex *) Glu->lusup, Glu->xlusup,
+              Glu->lsub, Glu->xlsub, Glu->supno, Glu->xsup,
+	      SLU_SC, SLU_C, SLU_TRLU);
+	cCreate_CompCol_Matrix(U, min_mn, min_mn, nnzU,
+	      (complex *) Glu->ucol, Glu->usub, Glu->xusub,
+	      SLU_NC, SLU_C, SLU_TRU);
     }
 
     ops[FACT] += ops[TRSV] + ops[GEMV];
+    stat->expansions = --(Glu->num_expansions);
 
     if ( iperm_r_allocated ) SUPERLU_FREE (iperm_r);
     SUPERLU_FREE (iperm_c);
@@ -623,6 +646,6 @@ cgsitrf(superlu_options_t *options, SuperMatrix *A, int relax, int panel_size,
     SUPERLU_FREE (iswap);
     SUPERLU_FREE (relax_fsupc);
     SUPERLU_FREE (amax);
-    if ( iwork2 ) SUPERLU_FREE (iwork2);
+    if ( swork2 ) SUPERLU_FREE (swork2);
 
 }
