@@ -298,9 +298,9 @@ cprint_lu_col(char *msg, int jcol, int pivrow, int *xprune, GlobalLU_t *Glu)
     supno   = Glu->supno;
     lsub    = Glu->lsub;
     xlsub   = Glu->xlsub;
-    lusup   = Glu->lusup;
+    lusup   = (complex *) Glu->lusup;
     xlusup  = Glu->xlusup;
-    ucol    = Glu->ucol;
+    ucol    = (complex *) Glu->ucol;
     usub    = Glu->usub;
     xusub   = Glu->xusub;
     
@@ -408,8 +408,8 @@ void cinf_norm_error(int nrhs, SuperMatrix *X, complex *xtrue)
       err = xnorm = 0.0;
       for (i = 0; i < X->nrow; i++) {
         c_sub(&temp, &soln_work[i], &xtrue[i]);
-	err = SUPERLU_MAX(err, slu_c_abs(&temp));
-	xnorm = SUPERLU_MAX(xnorm, slu_c_abs(&soln_work[i]));
+	err = SUPERLU_MAX(err, c_abs(&temp));
+	xnorm = SUPERLU_MAX(xnorm, c_abs(&soln_work[i]));
       }
       err = err / xnorm;
       printf("||X - Xtrue||/||X|| = %e\n", err);
