@@ -92,7 +92,7 @@
  * perm_r   (input/output) int*, dimension (A->nrow)
  *          Row permutation vector which defines the permutation matrix Pr,
  *          perm_r[i] = j means row i of A is in position j in Pr*A.
- *          If options->Fact = SamePattern_SameRowPerm, the pivoting routine
+ *          If options->Fact == SamePattern_SameRowPerm, the pivoting routine
  *             will try to use the input perm_r, unless a certain threshold
  *             criterion is violated. In that case, perm_r is overwritten by
  *             a new permutation determined by partial pivoting or diagonal
@@ -109,6 +109,17 @@
  *          storage scheme, i.e., U has types: Stype = SLU_NC, 
  *          Dtype = SLU_D, Mtype = SLU_TRU.
  *
+ * Glu      (input/output) GlobalLU_t *
+ *          If options->Fact == SamePattern_SameRowPerm, it is an input;
+ *              The matrix A will be factorized assuming that a 
+ *              factorization of a matrix with the same sparsity pattern
+ *              and similar numerical values was performed prior to this one.
+ *              Therefore, this factorization will reuse both row and column
+ *		scaling factors R and C, both row and column permutation
+ *		vectors perm_r and perm_c, and the L & U data structures
+ *		set up from the previous factorization.
+ *          Otherwise, it is an output.
+ *
  * stat     (output) SuperLUStat_t*
  *          Record the statistics on runtime and floating-point operation count.
  *          See slu_util.h for the definition of 'SuperLUStat_t'.
@@ -179,6 +190,7 @@ void
 dgstrf (superlu_options_t *options, SuperMatrix *A,
         int relax, int panel_size, int *etree, void *work, int lwork,
         int *perm_c, int *perm_r, SuperMatrix *L, SuperMatrix *U,
+    	GlobalLU_t *Glu, /* persistent to facilitate multiple factorizations */
         SuperLUStat_t *stat, int *info)
 {
     /* Local working arrays */
@@ -201,7 +213,6 @@ dgstrf (superlu_options_t *options, SuperMatrix *A,
     int       *xlsub, *xlusup, *xusub;
     int       nzlumax;
     double fill_ratio = sp_ienv(6);  /* estimated fill ratio */
-    static    GlobalLU_t Glu; /* persistent to facilitate multiple factors. */
 
     /* Local scalars */
     fact_t    fact = options->Fact;
@@ -232,14 +243,14 @@ dgstrf (superlu_options_t *options, SuperMatrix *A,
 
     /* Allocate storage common to the factor routines */
     *info = dLUMemInit(fact, work, lwork, m, n, Astore->nnz,
-                       panel_size, fill_ratio, L, U, &Glu, &iwork, &dwork);
+                       panel_size, fill_ratio, L, U, Glu, &iwork, &dwork);
     if ( *info ) return;
     
-    xsup    = Glu.xsup;
-    supno   = Glu.supno;
-    xlsub   = Glu.xlsub;
-    xlusup  = Glu.xlusup;
-    xusub   = Glu.xusub;
+    xsup    = Glu->xsup;
+    supno   = Glu->supno;
+    xlsub   = Glu->xlsub;
+    xlusup  = Glu->xlusup;
+    xusub   = Glu->xusub;
     
     SetIWork(m, n, panel_size, iwork, &segrep, &parent, &xplore,
 	     &repfnz, &panel_lsub, &xprune, &marker);
@@ -285,7 +296,7 @@ dgstrf (superlu_options_t *options, SuperMatrix *A,
 	     * -------------------------------------- */
 	    /* Determine the union of the row structure of the snode */
 	    if ( (*info = dsnode_dfs(jcol, kcol, asub, xa_begin, xa_end,
-				    xprune, marker, &Glu)) != 0 )
+				    xprune, marker, Glu)) != 0 )
 		return;
 
             nextu    = xusub[jcol];
@@ -293,9 +304,9 @@ dgstrf (superlu_options_t *options, SuperMatrix *A,
 	    jsupno   = supno[jcol];
 	    fsupc    = xsup[jsupno];
 	    new_next = nextlu + (xlsub[fsupc+1]-xlsub[fsupc])*(kcol-jcol+1);
-	    nzlumax = Glu.nzlumax;
+	    nzlumax = Glu->nzlumax;
 	    while ( new_next > nzlumax ) {
-		if ( (*info = dLUMemXpand(jcol, nextlu, LUSUP, &nzlumax, &Glu)) )
+		if ( (*info = dLUMemXpand(jcol, nextlu, LUSUP, &nzlumax, Glu)) )
 		    return;
 	    }
     
@@ -307,14 +318,14 @@ dgstrf (superlu_options_t *options, SuperMatrix *A,
         	    dense[asub[k]] = a[k];
 
 	       	/* Numeric update within the snode */
-	        dsnode_bmod(icol, jsupno, fsupc, dense, tempv, &Glu, stat);
+	        dsnode_bmod(icol, jsupno, fsupc, dense, tempv, Glu, stat);
 
 		if ( (*info = dpivotL(icol, diag_pivot_thresh, &usepr, perm_r,
-				      iperm_r, iperm_c, &pivrow, &Glu, stat)) )
+				      iperm_r, iperm_c, &pivrow, Glu, stat)) )
 		    if ( iinfo == 0 ) iinfo = *info;
 		
 #ifdef DEBUG
-		dprint_lu_col("[1]: ", icol, pivrow, xprune, &Glu);
+		dprint_lu_col("[1]: ", icol, pivrow, xprune, Glu);
 #endif
 
 	    }
@@ -338,11 +349,11 @@ dgstrf (superlu_options_t *options, SuperMatrix *A,
 	    /* symbolic factor on a panel of columns */
 	    dpanel_dfs(m, panel_size, jcol, A, perm_r, &nseg1,
 		      dense, panel_lsub, segrep, repfnz, xprune,
-		      marker, parent, xplore, &Glu);
+		      marker, parent, xplore, Glu);
 	    
 	    /* numeric sup-panel updates in topological order */
 	    dpanel_bmod(m, panel_size, jcol, nseg1, dense,
-		        tempv, segrep, repfnz, &Glu, stat);
+		        tempv, segrep, repfnz, Glu, stat);
 	    
 	    /* Sparse LU within the panel, and below panel diagonal */
     	    for ( jj = jcol; jj < jcol + panel_size; jj++) {
@@ -352,31 +363,31 @@ dgstrf (superlu_options_t *options, SuperMatrix *A,
 
 	    	if ((*info = dcolumn_dfs(m, jj, perm_r, &nseg, &panel_lsub[k],
 					segrep, &repfnz[k], xprune, marker,
-					parent, xplore, &Glu)) != 0) return;
+					parent, xplore, Glu)) != 0) return;
 
 	      	/* Numeric updates */
 	    	if ((*info = dcolumn_bmod(jj, (nseg - nseg1), &dense[k],
 					 tempv, &segrep[nseg1], &repfnz[k],
-					 jcol, &Glu, stat)) != 0) return;
+					 jcol, Glu, stat)) != 0) return;
 		
 	        /* Copy the U-segments to ucol[*] */
 		if ((*info = dcopy_to_ucol(jj, nseg, segrep, &repfnz[k],
-					  perm_r, &dense[k], &Glu)) != 0)
+					  perm_r, &dense[k], Glu)) != 0)
 		    return;
 
 	    	if ( (*info = dpivotL(jj, diag_pivot_thresh, &usepr, perm_r,
-				      iperm_r, iperm_c, &pivrow, &Glu, stat)) )
+				      iperm_r, iperm_c, &pivrow, Glu, stat)) )
 		    if ( iinfo == 0 ) iinfo = *info;
 
 		/* Prune columns (0:jj-1) using column jj */
 	    	dpruneL(jj, perm_r, pivrow, nseg, segrep,
-                        &repfnz[k], xprune, &Glu);
+                        &repfnz[k], xprune, Glu);
 
 		/* Reset repfnz[] for this column */
 	    	resetrep_col (nseg, segrep, &repfnz[k]);
 		
 #ifdef DEBUG
-		dprint_lu_col("[2]: ", jj, pivrow, xprune, &Glu);
+		dprint_lu_col("[2]: ", jj, pivrow, xprune, Glu);
 #endif
 
 	    }
@@ -398,10 +409,10 @@ dgstrf (superlu_options_t *options, SuperMatrix *A,
 	    }
     }
 
-    countnz(min_mn, xprune, &nnzL, &nnzU, &Glu);
-    fixupL(min_mn, perm_r, &Glu);
+    countnz(min_mn, xprune, &nnzL, &nnzU, Glu);
+    fixupL(min_mn, perm_r, Glu);
 
-    dLUWorkFree(iwork, dwork, &Glu); /* Free work space and compress storage */
+    dLUWorkFree(iwork, dwork, Glu); /* Free work space and compress storage */
 
     if ( fact == SamePattern_SameRowPerm ) {
         /* L and U structures may have changed due to possibly different
@@ -409,25 +420,27 @@ dgstrf (superlu_options_t *options, SuperMatrix *A,
 	   There could also be memory expansions, so the array locations
            may have changed, */
         ((SCformat *)L->Store)->nnz = nnzL;
-	((SCformat *)L->Store)->nsuper = Glu.supno[n];
-	((SCformat *)L->Store)->nzval = Glu.lusup;
-	((SCformat *)L->Store)->nzval_colptr = Glu.xlusup;
-	((SCformat *)L->Store)->rowind = Glu.lsub;
-	((SCformat *)L->Store)->rowind_colptr = Glu.xlsub;
+	((SCformat *)L->Store)->nsuper = Glu->supno[n];
+	((SCformat *)L->Store)->nzval = (double *) Glu->lusup;
+	((SCformat *)L->Store)->nzval_colptr = Glu->xlusup;
+	((SCformat *)L->Store)->rowind = Glu->lsub;
+	((SCformat *)L->Store)->rowind_colptr = Glu->xlsub;
 	((NCformat *)U->Store)->nnz = nnzU;
-	((NCformat *)U->Store)->nzval = Glu.ucol;
-	((NCformat *)U->Store)->rowind = Glu.usub;
-	((NCformat *)U->Store)->colptr = Glu.xusub;
+	((NCformat *)U->Store)->nzval = (double *) Glu->ucol;
+	((NCformat *)U->Store)->rowind = Glu->usub;
+	((NCformat *)U->Store)->colptr = Glu->xusub;
     } else {
-        dCreate_SuperNode_Matrix(L, A->nrow, min_mn, nnzL, Glu.lusup, 
-	                         Glu.xlusup, Glu.lsub, Glu.xlsub, Glu.supno,
-			         Glu.xsup, SLU_SC, SLU_D, SLU_TRLU);
-    	dCreate_CompCol_Matrix(U, min_mn, min_mn, nnzU, Glu.ucol, 
-			       Glu.usub, Glu.xusub, SLU_NC, SLU_D, SLU_TRU);
+        dCreate_SuperNode_Matrix(L, A->nrow, min_mn, nnzL, 
+	      (double *) Glu->lusup, Glu->xlusup, 
+              Glu->lsub, Glu->xlsub, Glu->supno, Glu->xsup,
+              SLU_SC, SLU_D, SLU_TRLU);
+    	dCreate_CompCol_Matrix(U, min_mn, min_mn, nnzU, 
+	      (double *) Glu->ucol, Glu->usub, Glu->xusub,
+              SLU_NC, SLU_D, SLU_TRU);
     }
     
     ops[FACT] += ops[TRSV] + ops[GEMV];	
-    stat->expansions = --(Glu.num_expansions);
+    stat->expansions = --(Glu->num_expansions);
     
     if ( iperm_r_allocated ) SUPERLU_FREE (iperm_r);
     SUPERLU_FREE (iperm_c);
