@@ -128,7 +128,7 @@ dgstrs (trans_t trans, SuperMatrix *L, SuperMatrix *U,
 	*info = -6;
     if ( *info ) {
 	i = -(*info);
-	xerbla_("dgstrs", &i);
+	input_error("dgstrs", &i);
 	return;
     }
 
@@ -332,6 +332,6 @@ dprint_soln(int n, int nrhs, double *soln)
 {
     int i;
 
-    for (i = 0; i < n; i++) 
+    for (i = 0; i < n; i++)
   	printf("\t%d: %.4f\n", i, soln[i]);
 }
