@@ -143,7 +143,7 @@ typedef unsigned char Logical;
  *             prior to this one. Therefore, this factorization will reuse
  *             both row and column scaling factors R and C, both row and
  *             column permutation vectors perm_r and perm_c, and the
- *             data structure set up from the previous symbolic factorization.
+ *             L & U data structures set up from the previous factorization.
  *        = FACTORED: On entry, L, U, perm_r and perm_c contain the 
  *              factored form of A. If DiagScale is not NOEQUIL, the matrix
  *              A has been equilibrated with scaling factors R and C.
@@ -310,6 +310,28 @@ typedef struct {
 } mem_usage_t;
 
 
+typedef struct {
+    int     *xsup;    /* supernode and column mapping */
+    int     *supno;   
+    int     *lsub;    /* compressed L subscripts */
+    int	    *xlsub;
+    void    *lusup;   /* L supernodes */
+    int     *xlusup;
+    void    *ucol;    /* U columns */
+    int     *usub;
+    int	    *xusub;
+    int     nzlmax;   /* current max size of lsub */
+    int     nzumax;   /*    "    "    "      ucol */
+    int     nzlumax;  /*    "    "    "     lusup */
+    int     n;        /* number of columns in the matrix */
+    LU_space_t MemModel; /* 0 - system malloc'd; 1 - user provided */
+    int     num_expansions;
+    ExpHeader *expanders; /* Array of pointers to 4 types of memory */
+    LU_stack_t stack;     /* use user supplied memory */
+} GlobalLU_t;
+
+
+
 /***********************************************************************
  * Prototypes
  ***********************************************************************/
@@ -317,6 +339,8 @@ typedef struct {
 extern "C" {
 #endif
 
+extern int     input_error(char *, int *);
+
 extern void    Destroy_SuperMatrix_Store(SuperMatrix *);
 extern void    Destroy_CompCol_Matrix(SuperMatrix *);
 extern void    Destroy_CompRow_Matrix(SuperMatrix *);
@@ -348,7 +372,6 @@ extern int     spcoletree (int *, int *, int *, int, int, int *);
 extern int     *TreePostorder (int, int *);
 extern double  SuperLU_timer_ ();
 extern int     sp_ienv (int);
-extern int     lsame_ (char *, char *);
 extern int     xerbla_ (char *, int *);
 extern void    ifill (int *, int, int);
 extern void    snode_profile (int, int *);
