@@ -27,8 +27,8 @@ extern void    user_bcopy      (char *, char *, int);
 
 /* Macros to manipulate stack */
 #define StackFull(x)         ( x + Glu->stack.used >= Glu->stack.size )
-#define NotDoubleAlign(addr) ( (long int)addr & 7 )
-#define DoubleAlign(addr)    ( ((long int)addr + 7) & ~7L )
+#define NotDoubleAlign(addr) ( (intptr_t)addr & 7 )
+#define DoubleAlign(addr)    ( ((intptr_t)addr + 7) & ~7L )	
 #define TempSpace(m, w)      ( (2*w + 4 + NO_MARKER) * m * sizeof(int) + \
 			      (w + 1) * m * sizeof(float) )
 #define Reduce(alpha)        ((alpha + 1) / 2)  /* i.e. (alpha-1)/2 + 1 */
@@ -197,8 +197,7 @@ sLUMemInit(fact_t fact, void *work, int lwork, int m, int n, int annz,
     Glu->n    = n;
     Glu->num_expansions = 0;
 
-    if ( !Glu->expanders )	
-        Glu->expanders = (ExpHeader*)SUPERLU_MALLOC( NO_MEMTYPE *
+    Glu->expanders = (ExpHeader *) SUPERLU_MALLOC( NO_MEMTYPE *
                                                      sizeof(ExpHeader) );
     if ( !Glu->expanders ) ABORT("SUPERLU_MALLOC fails for expanders");
     
@@ -215,7 +214,7 @@ sLUMemInit(fact_t fact, void *work, int lwork, int m, int n, int annz,
 	}
 	
 #if ( PRNTlevel >= 1 )
-	printf("sLUMemInit() called: fill_ratio %ld, nzlmax %ld, nzumax %ld\n", 
+	printf("sLUMemInit() called: fill_ratio %.0f, nzlmax %ld, nzumax %ld\n", 
 	       fill_ratio, nzlmax, nzumax);
 	fflush(stdout);
 #endif	
@@ -306,9 +305,9 @@ sLUMemInit(fact_t fact, void *work, int lwork, int m, int n, int annz,
     Glu->supno   = supno;
     Glu->lsub    = lsub;
     Glu->xlsub   = xlsub;
-    Glu->lusup   = lusup;
+    Glu->lusup   = (void *) lusup;
     Glu->xlusup  = xlusup;
-    Glu->ucol    = ucol;
+    Glu->ucol    = (void *) ucol;
     Glu->usub    = usub;
     Glu->xusub   = xusub;
     Glu->nzlmax  = nzlmax;
@@ -332,7 +331,7 @@ sLUWorkInit(int m, int n, int panel_size, int **iworkptr,
 {
     int    isize, dsize, extra;
     float *old_ptr;
-    int    maxsuper = sp_ienv(3),
+    int    maxsuper = SUPERLU_MAX( sp_ienv(3), sp_ienv(7) ),
            rowblk   = sp_ienv(4);
 
     isize = ( (2 * panel_size + 3 + NO_MARKER ) * m + n ) * sizeof(int);
@@ -381,7 +380,7 @@ sSetRWork(int m, int panel_size, float *dworkptr,
 {
     float zero = 0.0;
 
-    int maxsuper = sp_ienv(3),
+    int maxsuper = SUPERLU_MAX( sp_ienv(3), sp_ienv(7) ),
         rowblk   = sp_ienv(4);
     *dense = dworkptr;
     *tempv = *dense + panel_size*m;
@@ -443,11 +442,11 @@ sLUMemXpand(int jcol,
 
     switch ( mem_type ) {
       case LUSUP:
-	Glu->lusup   = (float *) new_mem;
+	Glu->lusup   = (void *) new_mem;
 	Glu->nzlumax = *maxlen;
 	break;
       case UCOL:
-	Glu->ucol   = (float *) new_mem;
+	Glu->ucol   = (void *) new_mem;
 	Glu->nzumax = *maxlen;
 	break;
       case LSUB:
