@@ -300,6 +300,17 @@
  *         any element of A or B that makes X(j) an exact solution).
  *         If options->IterRefine = NOREFINE, berr = 1.0.
  *
+ * Glu      (input/output) GlobalLU_t *
+ *          If options->Fact == SamePattern_SameRowPerm, it is an input;
+ *              The matrix A will be factorized assuming that a 
+ *              factorization of a matrix with the same sparsity pattern
+ *              and similar numerical values was performed prior to this one.
+ *              Therefore, this factorization will reuse both row and column
+ *		scaling factors R and C, both row and column permutation
+ *		vectors perm_r and perm_c, and the L & U data structures
+ *		set up from the previous factorization.
+ *          Otherwise, it is an output.
+ *
  * mem_usage (output) mem_usage_t*
  *         Record the memory usage statistics, consisting of following fields:
  *         - for_lu (float)
@@ -338,7 +349,7 @@ zgssvx(superlu_options_t *options, SuperMatrix *A, int *perm_c, int *perm_r,
        SuperMatrix *L, SuperMatrix *U, void *work, int lwork,
        SuperMatrix *B, SuperMatrix *X, double *recip_pivot_growth, 
        double *rcond, double *ferr, double *berr, 
-       mem_usage_t *mem_usage, SuperLUStat_t *stat, int *info )
+       GlobalLU_t *Glu, mem_usage_t *mem_usage, SuperLUStat_t *stat, int *info )
 {
 
 
@@ -377,9 +388,9 @@ zgssvx(superlu_options_t *options, SuperMatrix *A, int *perm_c, int *perm_r,
 	rowequ = FALSE;
 	colequ = FALSE;
     } else {
-	rowequ = lsame_(equed, "R") || lsame_(equed, "B");
-	colequ = lsame_(equed, "C") || lsame_(equed, "B");
-	smlnum = dlamch_("Safe minimum");
+	rowequ = strncmp(equed, "R", 1)==0 || strncmp(equed, "B", 1)==0;
+	colequ = strncmp(equed, "C", 1)==0 || strncmp(equed, "B", 1)==0;
+	smlnum = dmach("Safe minimum");   /* lamch_("Safe minimum"); */
 	bignum = 1. / smlnum;
     }
 
@@ -400,8 +411,8 @@ printf("dgssvx: Fact=%4d, Trans=%4d, equed=%c\n",
 	      (A->Stype != SLU_NC && A->Stype != SLU_NR) ||
 	      A->Dtype != SLU_Z || A->Mtype != SLU_GE )
 	*info = -2;
-    else if (options->Fact == FACTORED &&
-	     !(rowequ || colequ || lsame_(equed, "N")))
+    else if ( options->Fact == FACTORED &&
+	     !(rowequ || colequ || strncmp(equed, "N", 1)==0) )
 	*info = -6;
     else {
 	if (rowequ) {
@@ -449,7 +460,7 @@ printf("dgssvx: Fact=%4d, Trans=%4d, equed=%c\n",
     }
     if (*info != 0) {
 	i = -(*info);
-	xerbla_("zgssvx", &i);
+	input_error("zgssvx", &i);
 	return;
     }
     
@@ -487,8 +498,8 @@ printf("dgssvx: Fact=%4d, Trans=%4d, equed=%c\n",
 	if ( info1 == 0 ) {
 	    /* Equilibrate matrix A. */
 	    zlaqgs(AA, R, C, rowcnd, colcnd, amax, equed);
-	    rowequ = lsame_(equed, "R") || lsame_(equed, "B");
-	    colequ = lsame_(equed, "C") || lsame_(equed, "B");
+	    rowequ = strncmp(equed, "R", 1)==0 || strncmp(equed, "B", 1)==0;
+	    colequ = strncmp(equed, "C", 1)==0 || strncmp(equed, "B", 1)==0;
 	}
 	utime[EQUIL] = SuperLU_timer_() - t0;
     }
@@ -521,7 +532,7 @@ printf("dgssvx: Fact=%4d, Trans=%4d, equed=%c\n",
 	/* Compute the LU factorization of A*Pc. */
 	t0 = SuperLU_timer_();
 	zgstrf(options, &AC, relax, panel_size, etree,
-                work, lwork, perm_c, perm_r, L, U, stat, info);
+                work, lwork, perm_c, perm_r, L, U, Glu, stat, info);
 	utime[FACT] = SuperLU_timer_() - t0;
 	
 	if ( lwork == -1 ) {
@@ -530,16 +541,18 @@ printf("dgssvx: Fact=%4d, Trans=%4d, equed=%c\n",
 	}
     }
 
-    if ( options->PivotGrowth ) {
-        if ( *info > 0 ) {
-	    if ( *info <= A->ncol ) {
-	        /* Compute the reciprocal pivot growth factor of the leading
-	           rank-deficient *info columns of A. */
-	        *recip_pivot_growth = zPivotGrowth(*info, AA, perm_c, L, U);
-	    }
-	    return;
+    if ( *info > 0 ) {
+        if ( *info <= A->ncol ) {
+	    /* Compute the reciprocal pivot growth factor of the leading
+	       rank-deficient (*info) columns of A. */
+	    *recip_pivot_growth = zPivotGrowth(*info, AA, perm_c, L, U);
         }
+	return;
+    }
 
+    /* *info == 0 at this point. */
+
+    if ( options->PivotGrowth ) {
         /* Compute the reciprocal pivot growth factor *recip_pivot_growth. */
         *recip_pivot_growth = zPivotGrowth(A->ncol, AA, perm_c, L, U);
     }
@@ -553,7 +566,7 @@ printf("dgssvx: Fact=%4d, Trans=%4d, equed=%c\n",
 	    *(unsigned char *)norm = 'I';
         }
         anorm = zlangs(norm, AA);
-        zgscon(norm, L, U, anorm, rcond, stat, info);
+        zgscon(norm, L, U, anorm, rcond, stat, &info1);
         utime[RCOND] = SuperLU_timer_() - t0;
     }
     
@@ -577,7 +590,7 @@ printf("dgssvx: Fact=%4d, Trans=%4d, equed=%c\n",
 	        Xmat[i + j*ldx] = Bmat[i + j*ldb];
     
         t0 = SuperLU_timer_();
-        zgstrs (trant, L, U, perm_c, perm_r, X, stat, info);
+        zgstrs (trant, L, U, perm_c, perm_r, X, stat, &info1);
         utime[SOLVE] = SuperLU_timer_() - t0;
     
         /* Use iterative refinement to improve the computed solution and compute
@@ -585,7 +598,7 @@ printf("dgssvx: Fact=%4d, Trans=%4d, equed=%c\n",
         t0 = SuperLU_timer_();
         if ( options->IterRefine != NOREFINE ) {
             zgsrfs(trant, AA, L, U, perm_c, perm_r, equed, R, C, B,
-                   X, ferr, berr, stat, info);
+                   X, ferr, berr, stat, &info1);
         } else {
             for (j = 0; j < nrhs; ++j) ferr[j] = berr[j] = 1.0;
         }
@@ -607,7 +620,8 @@ printf("dgssvx: Fact=%4d, Trans=%4d, equed=%c\n",
 
     if ( options->ConditionNumber ) {
         /* Set INFO = A->ncol+1 if the matrix is singular to working precision. */
-        if ( *rcond < dlamch_("E") ) *info = A->ncol + 1;
+        /*if ( *rcond < dlamch_("E") ) *info = A->ncol + 1;*/
+        if ( *rcond < dmach("E") ) *info = A->ncol + 1;
     }
 
     if ( nofact ) {
