@@ -12,4 +12,6 @@ npy_cdouble faddeeva_erfi_complex(npy_cdouble);
 npy_cdouble faddeeva_log_ndtr(npy_cdouble);
 npy_cdouble faddeeva_ndtr(npy_cdouble);
 npy_cdouble faddeeva_w(npy_cdouble);
+#include "_wright.h"
+npy_cdouble wrightomega(npy_cdouble);
 #endif
