@@ -7,20 +7,24 @@ from __future__ import division, print_function, absolute_import
 import warnings
 
 import numpy as np
+import math
 from scipy._lib.six import xrange
-from numpy import (pi, asarray, floor, isscalar, iscomplex, real, imag, sqrt,
-                   where, mgrid, sin, place, issubdtype, extract,
-                   less, inexact, nan, zeros, atleast_1d, sinc)
-from ._ufuncs import (ellipkm1, mathieu_a, mathieu_b, iv, jv, gamma, psi, zeta,
-                      hankel1, hankel2, yv, kv, gammaln, ndtri, errprint, poch,
-                      binom)
+from numpy import (pi, asarray, floor, isscalar, iscomplex, real,
+                   imag, sqrt, where, mgrid, sin, place, issubdtype,
+                   extract, less, inexact, nan, zeros, sinc)
+from . import _ufuncs as ufuncs
+from ._ufuncs import (ellipkm1, mathieu_a, mathieu_b, iv, jv, gamma,
+                      psi, _zeta, hankel1, hankel2, yv, kv, _gammaln,
+                      ndtri, poch, binom, hyp0f1)
 from . import specfun
 from . import orthogonal
+from ._comb import _comb_int
+
 
 __all__ = ['agm', 'ai_zeros', 'assoc_laguerre', 'bei_zeros', 'beip_zeros',
            'ber_zeros', 'bernoulli', 'berp_zeros', 'bessel_diff_formula',
            'bi_zeros', 'clpmn', 'comb', 'digamma', 'diric', 'ellipk',
-           'erf_zeros', 'erfcinv', 'erfinv', 'errprint', 'euler', 'factorial',
+           'erf_zeros', 'erfcinv', 'erfinv', 'euler', 'factorial',
            'factorialk', 'factorial2', 'fresnel_zeros',
            'fresnelc_zeros', 'fresnels_zeros', 'gamma', 'gammaln', 'h1vp',
            'h2vp', 'hankel1', 'hankel2', 'hyp0f1', 'iv', 'ivp', 'jn_zeros',
@@ -32,14 +36,7 @@ __all__ = ['agm', 'ai_zeros', 'assoc_laguerre', 'bei_zeros', 'beip_zeros',
            'polygamma', 'pro_cv_seq', 'psi', 'riccati_jn', 'riccati_yn',
            'sinc', 'sph_in', 'sph_inkn',
            'sph_jn', 'sph_jnyn', 'sph_kn', 'sph_yn', 'y0_zeros', 'y1_zeros',
-           'y1p_zeros', 'yn_zeros', 'ynp_zeros', 'yv', 'yvp', 'zeta',
-           'SpecialFunctionWarning']
-
-
-class SpecialFunctionWarning(Warning):
-    """Warning that can be issued with ``errprint(True)``"""
-    pass
-warnings.simplefilter("always", category=SpecialFunctionWarning)
+           'y1p_zeros', 'yn_zeros', 'ynp_zeros', 'yv', 'yvp', 'zeta']
 
 
 def diric(x, n):
@@ -143,6 +140,47 @@ def diric(x, n):
     return y
 
 
+def gammaln(x):
+    """
+    Logarithm of the absolute value of the Gamma function for real inputs.
+
+    Parameters
+    ----------
+    x : array-like
+        Values on the real line at which to compute ``gammaln``
+
+    Returns
+    -------
+    gammaln : ndarray
+        Values of ``gammaln`` at x.
+
+    See Also
+    --------
+    gammasgn : sign of the gamma function
+    loggamma : principal branch of the logarithm of the gamma function
+
+    Notes
+    -----
+    When used in conjunction with `gammasgn`, this function is useful
+    for working in logspace on the real axis without having to deal with
+    complex numbers, via the relation ``exp(gammaln(x)) = gammasgn(x)*gamma(x)``.
+
+    Note that `gammaln` currently accepts complex-valued inputs, but it is not
+    the same function as for real-valued inputs, and the branch is not
+    well-defined --- using `gammaln` with complex is deprecated and will be
+    disallowed in future Scipy versions.
+
+    For complex-valued log-gamma, use `loggamma` instead of `gammaln`.
+
+    """
+    if np.iscomplexobj(x):
+        warnings.warn(("Use of gammaln for complex arguments is "
+                       "deprecated as of scipy 0.18.0. Use "
+                       "scipy.special.loggamma instead."),
+                      DeprecationWarning)
+    return _gammaln(x)
+
+
 def jnjnp_zeros(nt):
     """Compute zeros of integer-order Bessel functions Jn and Jn'.
 
@@ -433,11 +471,17 @@ def jvp(v, z, n=1):
     n : int, default 1
         Order of derivative
 
+    Notes
+    -----
+    The derivative is computed using the relation DLFM 10.6.7 [2]_.
+
     References
     ----------
     .. [1] Zhang, Shanjie and Jin, Jianming. "Computation of Special
            Functions", John Wiley and Sons, 1996, chapter 5.
            http://jin.ece.illinois.edu/specfunc.html
+    .. [2] NIST Digital Library of Mathematical Functions.
+           http://dlmf.nist.gov/10.6.E7
 
     """
     if not isinstance(n, int) or (n < 0):
@@ -460,11 +504,17 @@ def yvp(v, z, n=1):
     n : int, default 1
         Order of derivative
 
+    Notes
+    -----
+    The derivative is computed using the relation DLFM 10.6.7 [2]_.
+
     References
     ----------
     .. [1] Zhang, Shanjie and Jin, Jianming. "Computation of Special
            Functions", John Wiley and Sons, 1996, chapter 5.
            http://jin.ece.illinois.edu/specfunc.html
+    .. [2] NIST Digital Library of Mathematical Functions.
+           http://dlmf.nist.gov/10.6.E7
 
     """
     if not isinstance(n, int) or (n < 0):
@@ -495,11 +545,30 @@ def kvp(v, z, n=1):
     out : ndarray
         The results
 
+    Examples
+    --------
+    Calculate multiple values at order 5:
+
+    >>> from scipy.special import kvp
+    >>> kvp(5, (1, 2, 3+5j))
+    array([-1849.0354+0.j    ,   -25.7735+0.j    ,    -0.0307+0.0875j])
+
+    Calculate for a single value at multiple orders:
+
+    >>> kvp((4, 4.5, 5), 1)
+    array([ -184.0309,  -568.9585, -1849.0354])
+
+    Notes
+    -----
+    The derivative is computed using the relation DLFM 10.29.5 [2]_.
+
     References
     ----------
     .. [1] Zhang, Shanjie and Jin, Jianming. "Computation of Special
            Functions", John Wiley and Sons, 1996, chapter 6.
            http://jin.ece.illinois.edu/specfunc.html
+    .. [2] NIST Digital Library of Mathematical Functions.
+           http://dlmf.nist.gov/10.29.E5
 
     """
     if not isinstance(n, int) or (n < 0):
@@ -523,24 +592,17 @@ def ivp(v, z, n=1):
     n : int, default 1
         Order of derivative
 
+    Notes
+    -----
+    The derivative is computed using the relation DLFM 10.29.5 [2]_.
+
     References
     ----------
     .. [1] Zhang, Shanjie and Jin, Jianming. "Computation of Special
            Functions", John Wiley and Sons, 1996, chapter 6.
            http://jin.ece.illinois.edu/specfunc.html
-
-    Examples
-    --------
-    Calculate multiple values at order 5:
-
-    >>> from scipy.special import kvp
-    >>> kvp(5, (1, 2, 3+5j))
-    array([-1849.0354+0.j    ,   -25.7735+0.j    ,    -0.0307+0.0875j])
-
-    Calculate for a single value at multiple orders:
-
-    >>> kvp((4, 4.5, 5), 1)
-    array([ -184.0309,  -568.9585, -1849.0354])
+    .. [2] NIST Digital Library of Mathematical Functions.
+           http://dlmf.nist.gov/10.29.E5
 
     """
     if not isinstance(n, int) or (n < 0):
@@ -563,11 +625,17 @@ def h1vp(v, z, n=1):
     n : int, default 1
         Order of derivative
 
+    Notes
+    -----
+    The derivative is computed using the relation DLFM 10.6.7 [2]_.
+
     References
     ----------
     .. [1] Zhang, Shanjie and Jin, Jianming. "Computation of Special
            Functions", John Wiley and Sons, 1996, chapter 5.
            http://jin.ece.illinois.edu/specfunc.html
+    .. [2] NIST Digital Library of Mathematical Functions.
+           http://dlmf.nist.gov/10.6.E7
 
     """
     if not isinstance(n, int) or (n < 0):
@@ -590,11 +658,17 @@ def h2vp(v, z, n=1):
     n : int, default 1
         Order of derivative
 
+    Notes
+    -----
+    The derivative is computed using the relation DLFM 10.6.7 [2]_.
+
     References
     ----------
     .. [1] Zhang, Shanjie and Jin, Jianming. "Computation of Special
            Functions", John Wiley and Sons, 1996, chapter 5.
            http://jin.ece.illinois.edu/specfunc.html
+    .. [2] NIST Digital Library of Mathematical Functions.
+           http://dlmf.nist.gov/10.6.E7
 
     """
     if not isinstance(n, int) or (n < 0):
@@ -605,6 +679,9 @@ def h2vp(v, z, n=1):
         return _bessel_diff_formula(v, z, n, hankel2, -1)
 
 
+@np.deprecate(message="scipy.special.sph_jn is deprecated in scipy 0.18.0. "
+                      "Use scipy.special.spherical_jn instead. "
+                      "Note that the new function has a different signature.")
 def sph_jn(n, z):
     """Compute spherical Bessel function jn(z) and derivative.
 
@@ -625,6 +702,10 @@ def sph_jn(n, z):
     jnp : ndarray
         First derivative j0'(z), ..., jn'(z)
 
+    See also
+    --------
+    spherical_jn
+
     References
     ----------
     .. [1] Zhang, Shanjie and Jin, Jianming. "Computation of Special
@@ -647,6 +728,9 @@ def sph_jn(n, z):
     return jn[:(n+1)], jnp[:(n+1)]
 
 
+@np.deprecate(message="scipy.special.sph_yn is deprecated in scipy 0.18.0. "
+                      "Use scipy.special.spherical_yn instead. "
+                      "Note that the new function has a different signature.")
 def sph_yn(n, z):
     """Compute spherical Bessel function yn(z) and derivative.
 
@@ -667,6 +751,10 @@ def sph_yn(n, z):
     ynp : ndarray
         First derivative y0'(z), ..., yn'(z)
 
+    See also
+    --------
+    spherical_yn
+
     References
     ----------
     .. [1] Zhang, Shanjie and Jin, Jianming. "Computation of Special
@@ -689,6 +777,10 @@ def sph_yn(n, z):
     return yn[:(n+1)], ynp[:(n+1)]
 
 
+@np.deprecate(message="scipy.special.sph_jnyn is deprecated in scipy 0.18.0. "
+                      "Use scipy.special.spherical_jn and "
+                      "scipy.special.spherical_yn instead. "
+                      "Note that the new function has a different signature.")
 def sph_jnyn(n, z):
     """Compute spherical Bessel functions jn(z) and yn(z) and derivatives.
 
@@ -713,6 +805,11 @@ def sph_jnyn(n, z):
     ynp : ndarray
         First derivative y0'(z), ..., yn'(z)
 
+    See also
+    --------
+    spherical_jn
+    spherical_yn
+
     References
     ----------
     .. [1] Zhang, Shanjie and Jin, Jianming. "Computation of Special
@@ -736,6 +833,9 @@ def sph_jnyn(n, z):
     return jn[:(n+1)], jnp[:(n+1)], yn[:(n+1)], ynp[:(n+1)]
 
 
+@np.deprecate(message="scipy.special.sph_in is deprecated in scipy 0.18.0. "
+                      "Use scipy.special.spherical_in instead. "
+                      "Note that the new function has a different signature.")
 def sph_in(n, z):
     """Compute spherical Bessel function in(z) and derivative.
 
@@ -756,6 +856,10 @@ def sph_in(n, z):
     inp : ndarray
         First derivative i0'(z), ..., in'(z)
 
+    See also
+    --------
+    spherical_in
+
     References
     ----------
     .. [1] Zhang, Shanjie and Jin, Jianming. "Computation of Special
@@ -778,6 +882,9 @@ def sph_in(n, z):
     return In[:(n+1)], Inp[:(n+1)]
 
 
+@np.deprecate(message="scipy.special.sph_kn is deprecated in scipy 0.18.0. "
+                      "Use scipy.special.spherical_kn instead. "
+                      "Note that the new function has a different signature.")
 def sph_kn(n, z):
     """Compute spherical Bessel function kn(z) and derivative.
 
@@ -798,6 +905,10 @@ def sph_kn(n, z):
     knp : ndarray
         First derivative k0'(z), ..., kn'(z)
 
+    See also
+    --------
+    spherical_kn
+
     References
     ----------
     .. [1] Zhang, Shanjie and Jin, Jianming. "Computation of Special
@@ -820,6 +931,10 @@ def sph_kn(n, z):
     return kn[:(n+1)], knp[:(n+1)]
 
 
+@np.deprecate(message="scipy.special.sph_inkn is deprecated in scipy 0.18.0. "
+                      "Use scipy.special.spherical_in and "
+                      "scipy.special.spherical_kn instead. "
+                      "Note that the new function has a different signature.")
 def sph_inkn(n, z):
     """Compute spherical Bessel functions in(z), kn(z), and derivatives.
 
@@ -844,6 +959,11 @@ def sph_inkn(n, z):
     knp : ndarray
         First derivative k0'(z), ..., kn'(z)
 
+    See also
+    --------
+    spherical_in
+    spherical_kn
+
     References
     ----------
     .. [1] Zhang, Shanjie and Jin, Jianming. "Computation of Special
@@ -868,10 +988,14 @@ def sph_inkn(n, z):
 
 
 def riccati_jn(n, x):
-    """Compute Ricatti-Bessel function of the first kind and derivative.
+    r"""Compute Ricatti-Bessel function of the first kind and its derivative.
 
-    This function computes the value and first derivative of the function for
-    all orders up to and including n.
+    The Ricatti-Bessel function of the first kind is defined as :math:`x
+    j_n(x)`, where :math:`j_n` is the spherical Bessel function of the first
+    kind of order :math:`n`.
+
+    This function computes the value and first derivative of the
+    Ricatti-Bessel function for all orders up to and including `n`.
 
     Parameters
     ----------
@@ -887,11 +1011,21 @@ def riccati_jn(n, x):
     jnp : ndarray
         First derivative j0'(x), ..., jn'(x)
 
+    Notes
+    -----
+    The computation is carried out via backward recurrence, using the
+    relation DLMF 10.51.1 [2]_.
+
+    Wrapper for a Fortran routine created by Shanjie Zhang and Jianming
+    Jin [1]_.
+
     References
     ----------
     .. [1] Zhang, Shanjie and Jin, Jianming. "Computation of Special
            Functions", John Wiley and Sons, 1996.
            http://jin.ece.illinois.edu/specfunc.html
+    .. [2] NIST Digital Library of Mathematical Functions.
+           http://dlmf.nist.gov/10.51.E1
 
     """
     if not (isscalar(n) and isscalar(x)):
@@ -907,10 +1041,14 @@ def riccati_jn(n, x):
 
 
 def riccati_yn(n, x):
-    """Compute Ricatti-Bessel function of the second kind and derivative.
+    """Compute Ricatti-Bessel function of the second kind and its derivative.
+
+    The Ricatti-Bessel function of the second kind is defined as :math:`x
+    y_n(x)`, where :math:`y_n` is the spherical Bessel function of the second
+    kind of order :math:`n`.
 
     This function computes the value and first derivative of the function for
-    all orders up to and including n.
+    all orders up to and including `n`.
 
     Parameters
     ----------
@@ -926,11 +1064,21 @@ def riccati_yn(n, x):
     ynp : ndarray
         First derivative y0'(x), ..., yn'(x)
 
+    Notes
+    -----
+    The computation is carried out via ascending recurrence, using the
+    relation DLMF 10.51.1 [2]_.
+
+    Wrapper for a Fortran routine created by Shanjie Zhang and Jianming
+    Jin [1]_.
+
     References
     ----------
     .. [1] Zhang, Shanjie and Jin, Jianming. "Computation of Special
            Functions", John Wiley and Sons, 1996.
            http://jin.ece.illinois.edu/specfunc.html
+    .. [2] NIST Digital Library of Mathematical Functions.
+           http://dlmf.nist.gov/10.51.E1
 
     """
     if not (isscalar(n) and isscalar(x)):
@@ -1017,47 +1165,11 @@ def fresnel_zeros(nt):
     return specfun.fcszo(2, nt), specfun.fcszo(1, nt)
 
 
-def hyp0f1(v, z):
-    r"""Confluent hypergeometric limit function 0F1.
-
-    Parameters
-    ----------
-    v, z : array_like
-        Input values.
-
-    Returns
-    -------
-    hyp0f1 : ndarray
-        The confluent hypergeometric limit function.
-
-    Notes
-    -----
-    This function is defined as:
-
-    .. math:: _0F_1(v, z) = \sum_{k=0}^{\inf}\frac{z^k}{(v)_k k!}.
-
-    It's also the limit as q -> infinity of ``1F1(q;v;z/q)``, and satisfies
-    the differential equation :math:`f''(z) + vf'(z) = f(z)`.
-    """
-    v = atleast_1d(v)
-    z = atleast_1d(z)
-    v, z = np.broadcast_arrays(v, z)
-    arg = 2 * sqrt(abs(z))
-    old_err = np.seterr(all='ignore')  # for z=0, a<1 and num=inf, next lines
-    num = where(z.real >= 0, iv(v - 1, arg), jv(v - 1, arg))
-    den = abs(z)**((v - 1.0) / 2)
-    num *= gamma(v)
-    np.seterr(**old_err)
-    num[z == 0] = 1
-    den[z == 0] = 1
-    return num / den
-
-
 def assoc_laguerre(x, n, k=0.0):
-    """Compute nth-order generalized (associated) Laguerre polynomial.
+    """Compute the generalized (associated) Laguerre polynomial of degree n and order k.
 
-    The polynomial :math:`L^(alpha)_n(x)` is orthogonal over ``[0, inf)``,
-    with weighting function ``exp(-x) * x**alpha`` with ``alpha > -1``.
+    The polynomial :math:`L^{(k)}_n(x)` is orthogonal over ``[0, inf)``,
+    with weighting function ``exp(-x) * x**k`` with ``k > -1``.
 
     Notes
     -----
@@ -1219,7 +1331,7 @@ def mathieu_odd_coef(m, q):
 
 
 def lpmn(m, n, z):
-    """Associated Legendre function of the first kind, Pmn(z).
+    """Sequence of associated Legendre functions of the first kind.
 
     Computes the associated Legendre function of the first kind of order m and
     degree n, ``Pmn(z)`` = :math:`P_n^m(z)`, and its derivative, ``Pmn'(z)``.
@@ -1277,15 +1389,14 @@ def lpmn(m, n, z):
     if (m < 0):
         mp = -m
         mf, nf = mgrid[0:mp+1, 0:n+1]
-        sv = errprint(0)
-        if abs(z) < 1:
-            # Ferrer function; DLMF 14.9.3
-            fixarr = where(mf > nf, 0.0,
-                           (-1)**mf * gamma(nf-mf+1) / gamma(nf+mf+1))
-        else:
-            # Match to clpmn; DLMF 14.9.13
-            fixarr = where(mf > nf, 0.0, gamma(nf-mf+1) / gamma(nf+mf+1))
-        sv = errprint(sv)
+        with ufuncs.errstate(all='ignore'):
+            if abs(z) < 1:
+                # Ferrer function; DLMF 14.9.3
+                fixarr = where(mf > nf, 0.0,
+                               (-1)**mf * gamma(nf-mf+1) / gamma(nf+mf+1))
+            else:
+                # Match to clpmn; DLMF 14.9.13
+                fixarr = where(mf > nf, 0.0, gamma(nf-mf+1) / gamma(nf+mf+1))
     else:
         mp = m
     p, pd = specfun.lpmn(mp, n, z)
@@ -1296,7 +1407,7 @@ def lpmn(m, n, z):
 
 
 def clpmn(m, n, z, type=3):
-    """Associated Legendre function of the first kind, Pmn(z).
+    """Associated Legendre function of the first kind for complex arguments.
 
     Computes the associated Legendre function of the first kind of order m and
     degree n, ``Pmn(z)`` = :math:`P_n^m(z)`, and its derivative, ``Pmn'(z)``.
@@ -1361,13 +1472,12 @@ def clpmn(m, n, z, type=3):
     if (m < 0):
         mp = -m
         mf, nf = mgrid[0:mp+1, 0:n+1]
-        sv = errprint(0)
-        if type == 2:
-            fixarr = where(mf > nf, 0.0,
-                           (-1)**mf * gamma(nf-mf+1) / gamma(nf+mf+1))
-        else:
-            fixarr = where(mf > nf, 0.0, gamma(nf-mf+1) / gamma(nf+mf+1))
-        sv = errprint(sv)
+        with ufuncs.errstate(all='ignore'):
+            if type == 2:
+                fixarr = where(mf > nf, 0.0,
+                               (-1)**mf * gamma(nf-mf+1) / gamma(nf+mf+1))
+            else:
+                fixarr = where(mf > nf, 0.0, gamma(nf-mf+1) / gamma(nf+mf+1))
     else:
         mp = m
     p, pd = specfun.clpmn(mp, n, real(z), imag(z), type)
@@ -1378,7 +1488,7 @@ def clpmn(m, n, z, type=3):
 
 
 def lqmn(m, n, z):
-    """Associated Legendre function of the second kind, Qmn(z).
+    """Sequence of associated Legendre functions of the second kind.
 
     Computes the associated Legendre function of the second kind of order m and
     degree n, ``Qmn(z)`` = :math:`Q_n^m(z)`, and its derivative, ``Qmn'(z)``.
@@ -1471,7 +1581,7 @@ def euler(n):
 
 
 def lpn(n, z):
-    """Legendre functions of the first kind, Pn(z).
+    """Legendre function of the first kind.
 
     Compute sequence of Legendre functions of the first kind (polynomials),
     Pn(z) and derivatives for all degrees from 0 to n (inclusive).
@@ -1501,7 +1611,7 @@ def lpn(n, z):
 
 
 def lqn(n, z):
-    """Legendre functions of the second kind, Qn(z).
+    """Legendre function of the second kind.
 
     Compute sequence of Legendre functions of the second kind, Qn(z) and
     derivatives for all degrees from 0 to n (inclusive).
@@ -1605,7 +1715,14 @@ def bi_zeros(nt):
 
 
 def lmbda(v, x):
-    """Jahnke-Emden Lambda function, Lambdav(x).
+    r"""Jahnke-Emden Lambda function, Lambdav(x).
+
+    This function is defined as [2]_,
+
+    .. math:: \Lambda_v(x) = \Gamma(v+1) \frac{J_v(x)}{(x/2)^v},
+
+    where :math:`\Gamma` is the gamma function and :math:`J_v` is the
+    Bessel function of the first kind.
 
     Parameters
     ----------
@@ -1626,7 +1743,8 @@ def lmbda(v, x):
     .. [1] Zhang, Shanjie and Jin, Jianming. "Computation of Special
            Functions", John Wiley and Sons, 1996.
            http://jin.ece.illinois.edu/specfunc.html
-
+    .. [2] Jahnke, E. and Emde, F. "Tables of Functions with Formulae and
+           Curves" (4th ed.), Dover, 1945
     """
     if not (isscalar(v) and isscalar(x)):
         raise ValueError("arguments must be scalars.")
@@ -1961,7 +2079,8 @@ def ellipk(m):
 
     Notes
     -----
-    For more precision around point m = 1, use `ellipkm1`.
+    For more precision around point m = 1, use `ellipkm1`, which this
+    function calls.
 
     See Also
     --------
@@ -2016,6 +2135,10 @@ def comb(N, k, exact=False, repetition=False):
     val : int, ndarray
         The total number of combinations.
 
+    See Also
+    --------
+    binom : Binomial coefficient ufunc
+
     Notes
     -----
     - Array arguments accepted only for exact=False case.
@@ -2037,14 +2160,7 @@ def comb(N, k, exact=False, repetition=False):
     if repetition:
         return comb(N + k - 1, k, exact)
     if exact:
-        N = int(N)
-        k = int(k)
-        if (k > N) or (N < 0) or (k < 0):
-            return 0
-        val = 1
-        for j in xrange(min(k, N-k)):
-            val = (val*(N-j))//(j+1)
-        return val
+        return _comb_int(N, k)
     else:
         k, N = asarray(k), asarray(N)
         cond = (k <= N) & (N >= 0) & (k >= 0)
@@ -2110,29 +2226,60 @@ def perm(N, k, exact=False):
         return vals
 
 
+# http://stackoverflow.com/a/16327037/125507
+def _range_prod(lo, hi):
+    """
+    Product of a range of numbers.
+
+    Returns the product of
+    lo * (lo+1) * (lo+2) * ... * (hi-2) * (hi-1) * hi
+    = hi! / (lo-1)!
+
+    Breaks into smaller products first for speed:
+    _range_prod(2, 9) = ((2*3)*(4*5))*((6*7)*(8*9))
+    """
+    if lo + 1 < hi:
+        mid = (hi + lo) // 2
+        return _range_prod(lo, mid) * _range_prod(mid + 1, hi)
+    if lo == hi:
+        return lo
+    return lo * hi
+
+
 def factorial(n, exact=False):
-    """The factorial function, n! = special.gamma(n+1).
+    """
+    The factorial of a number or array of numbers.
 
-    If exact is 0, then floating point precision is used, otherwise
-    exact long integer is computed.
+    The factorial of non-negative integer `n` is the product of all
+    positive integers less than or equal to `n`::
 
-    - Array argument accepted only for exact=False case.
-    - If n<0, the return value is 0.
+        n! = n * (n - 1) * (n - 2) * ... * 1
 
     Parameters
     ----------
     n : int or array_like of ints
-        Calculate ``n!``.  Arrays are only supported with `exact` set
-        to False.  If ``n < 0``, the return value is 0.
+        Input values.  If ``n < 0``, the return value is 0.
     exact : bool, optional
-        The result can be approximated rapidly using the gamma-formula
-        above.  If `exact` is set to True, calculate the
-        answer exactly using integer arithmetic. Default is False.
+        If True, calculate the answer exactly using long integer arithmetic.
+        If False, result is approximated in floating point rapidly using the
+        `gamma` function.
+        Default is False.
 
     Returns
     -------
-    nf : float or int
-        Factorial of `n`, as an integer or a float depending on `exact`.
+    nf : float or int or ndarray
+        Factorial of `n`, as integer or float depending on `exact`.
+
+    Notes
+    -----
+    For arrays with ``exact=True``, the factorial is computed only once, for
+    the largest input, with each other result computed in the process.
+    The output dtype is increased to ``int64`` or ``object`` if necessary.
+
+    With ``exact=False`` the factorial is approximated using the gamma
+    function:
+
+    .. math:: n! = \\Gamma(n+1)
 
     Examples
     --------
@@ -2140,20 +2287,47 @@ def factorial(n, exact=False):
     >>> arr = np.array([3, 4, 5])
     >>> factorial(arr, exact=False)
     array([   6.,   24.,  120.])
+    >>> factorial(arr, exact=True)
+    array([  6,  24, 120])
     >>> factorial(5, exact=True)
     120L
 
     """
     if exact:
-        if n < 0:
-            return 0
-        val = 1
-        for k in xrange(1, n+1):
-            val *= k
-        return val
+        if np.ndim(n) == 0:
+            return 0 if n < 0 else math.factorial(n)
+        else:
+            n = asarray(n)
+            un = np.unique(n).astype(object)
+
+            # Convert to object array of long ints if np.int can't handle size
+            if un[-1] > 20:
+                dt = object
+            elif un[-1] > 12:
+                dt = np.int64
+            else:
+                dt = np.int
+
+            out = np.empty_like(n, dtype=dt)
+
+            # Handle invalid/trivial values
+            un = un[un > 1]
+            out[n < 2] = 1
+            out[n < 0] = 0
+
+            # Calculate products of each range of numbers
+            if un.size:
+                val = math.factorial(un[0])
+                out[n == un[0]] = val
+                for i in xrange(len(un) - 1):
+                    prev = un[i] + 1
+                    current = un[i + 1]
+                    val *= _range_prod(prev, current)
+                    out[n == current] = val
+            return out
     else:
         n = asarray(n)
-        vals = gamma(n+1)
+        vals = gamma(n + 1)
         return where(n >= 0, vals, 0)
 
 
@@ -2267,3 +2441,24 @@ def factorialk(n, k, exact=True):
         return val
     else:
         raise NotImplementedError
+
+
+def zeta(x, q=None, out=None):
+    r"""
+    Riemann zeta function.
+
+    The two-argument version is the Hurwitz zeta function:
+
+    .. math:: \zeta(x, q) = \sum_{k=0}^{\infty} \frac{1}{(k + q)^x},
+
+    Riemann zeta function corresponds to ``q = 1``.
+
+    See also
+    --------
+    zetac
+
+    """
+    if q is None:
+        q = 1
+    return _zeta(x, q, out)
+
