@@ -1,6 +1,6 @@
-/*							chdtr.c
+/*                                                     chdtr.c
  *
- *	Chi-square distribution
+ *     Chi-square distribution
  *
  *
  *
@@ -32,7 +32,7 @@
  * The incomplete Gamma integral is used, according to the
  * formula
  *
- *	y = chdtr( v, x ) = igam( v/2.0, x/2.0 ).
+ *     y = chdtr( v, x ) = igam( v/2.0, x/2.0 ).
  *
  *
  * The arguments must both be positive.
@@ -139,52 +139,51 @@
  *
  */
 
-/*								chdtr() */
+/*                                                             chdtr() */
 
 
 /*
-Cephes Math Library Release 2.0:  April, 1987
-Copyright 1984, 1987 by Stephen L. Moshier
-Direct inquiries to 30 Frost Street, Cambridge, MA 02140
-*/
+ * Cephes Math Library Release 2.0:  April, 1987
+ * Copyright 1984, 1987 by Stephen L. Moshier
+ * Direct inquiries to 30 Frost Street, Cambridge, MA 02140
+ */
 
 #include "mconf.h"
 
-double chdtrc(df,x)
+double chdtrc(df, x)
 double df, x;
 {
 
-if (x < 0.0) return 1.0;   /* modified by T. Oliphant */
-return( igamc( df/2.0, x/2.0 ) );
+    if (x < 0.0)
+	return 1.0;		/* modified by T. Oliphant */
+    return (igamc(df / 2.0, x / 2.0));
 }
 
 
 
-double chdtr(df,x)
+double chdtr(df, x)
 double df, x;
 {
 
-if( (x < 0.0))  /* || (df < 1.0) ) */
-	{
-	mtherr( "chdtr", DOMAIN );
-	return(NPY_NAN);
-	}
-return( igam( df/2.0, x/2.0 ) );
+    if ((x < 0.0)) {		/* || (df < 1.0) ) */
+	mtherr("chdtr", DOMAIN);
+	return (NPY_NAN);
+    }
+    return (igam(df / 2.0, x / 2.0));
 }
 
 
 
-double chdtri( df, y )
+double chdtri(df, y)
 double df, y;
 {
-double x;
+    double x;
 
-if( (y < 0.0) || (y > 1.0)) /* || (df < 1.0) ) */
-	{
-	mtherr( "chdtri", DOMAIN );
-	return(NPY_NAN);
-	}
+    if ((y < 0.0) || (y > 1.0)) {	/* || (df < 1.0) ) */
+	mtherr("chdtri", DOMAIN);
+	return (NPY_NAN);
+    }
 
-x = igami( 0.5 * df, y );
-return( 2.0 * x );
+    x = igami(0.5 * df, y);
+    return (2.0 * x);
 }
