@@ -48,14 +48,26 @@
  * Copyright 1985 by Stephen L. Moshier
  * Direct inquiries to 30 Frost Street, Cambridge, MA 02140 */
 
+/* Sources
+ * [1] NIST, "The Digital Library of Mathematical Functions", dlmf.nist.gov
+ */
+
+/* Scipy changes:
+ * - 09-10-2016: improved asymptotic expansion for large n
+ */
+
 #include "mconf.h"
+#include "polevl.h"
+#include "expn.h"
+
 #define EUL 0.57721566490153286060
 #define BIG  1.44115188075855872E+17
 extern double MACHEP, MAXLOG;
 
-double expn(n, x)
-int n;
-double x;
+double expn_large_n(int, double);
+
+
+double expn(int n, double x)
 {
     double ans, r, t, yk, xk;
     double pk, pkm1, pkm2, qk, qkm1, qkm2;
@@ -67,58 +79,53 @@ double x;
 	goto domerr;
 
     if (x < 0) {
-      domerr:mtherr("expn", DOMAIN);
+      domerr: mtherr("expn", DOMAIN);
 	return (NPY_INFINITY);
     }
 
-    if (x > MAXLOG)
+    if (x > MAXLOG) {
 	return (0.0);
+    }
 
     if (x == 0.0) {
 	if (n < 2) {
 	    mtherr("expn", SING);
 	    return (NPY_INFINITY);
 	}
-	else
+	else {
 	    return (1.0 / (n - 1.0));
+	}
     }
 
-    if (n == 0)
+    if (n == 0) {
 	return (exp(-x) / x);
-
-    /*                                                     expn.c  */
-    /*             Expansion for large n           */
+    }
 
-    if (n > 5000) {
-	xk = x + n;
-	yk = 1.0 / (xk * xk);
-	t = n;
-	ans = yk * t * (6.0 * x * x - 8.0 * t * x + t * t);
-	ans = yk * (ans + t * (t - 2.0 * x));
-	ans = yk * (ans + t);
-	ans = (ans + 1.0) * exp(-x) / xk;
+    /* Asymptotic expansion for large n, DLMF 8.20(ii) */
+    if (n > 50) {
+	ans = expn_large_n(n, x);
 	goto done;
     }
 
-    if (x > 1.0)
+    if (x > 1.0) {
 	goto cfrac;
-
-    /*                                                     expn.c  */
-
-    /*             Power series expansion          */
+    }
 
+    /* Power series expansion, DLMF 8.19.8 */
     psi = -EUL - log(x);
-    for (i = 1; i < n; i++)
+    for (i = 1; i < n; i++) {
 	psi = psi + 1.0 / i;
+    }
 
     z = -x;
     xk = 0.0;
     yk = 1.0;
     pk = 1.0 - n;
-    if (n == 1)
+    if (n == 1) {
 	ans = 0.0;
-    else
+    } else {
 	ans = 1.0 / pk;
+    }
     do {
 	xk += 1.0;
 	yk *= z / xk;
@@ -130,16 +137,14 @@ double x;
 	    t = fabs(yk / ans);
 	else
 	    t = 1.0;
-    }
-    while (t > MACHEP);
+    } while (t > MACHEP);
     k = xk;
     t = n;
     r = n - 1;
     ans = (pow(z, r) * psi / Gamma(t)) - ans;
     goto done;
-
-    /*                                                     expn.c  */
-    /*             continued fraction              */
+
+    /* Continued fraction, DLMF 8.19.17 */
   cfrac:
     k = 1;
     pkm2 = 1.0;
@@ -153,8 +158,7 @@ double x;
 	if (k & 1) {
 	    yk = 1.0;
 	    xk = n + (k - 1) / 2;
-	}
-	else {
+	} else {
 	    yk = x;
 	    xk = k / 2;
 	}
@@ -164,9 +168,9 @@ double x;
 	    r = pk / qk;
 	    t = fabs((ans - r) / r);
 	    ans = r;
-	}
-	else
+	} else {
 	    t = 1.0;
+	}
 	pkm2 = pkm1;
 	pkm1 = pk;
 	qkm2 = qkm1;
@@ -177,11 +181,44 @@ double x;
 	    qkm2 /= big;
 	    qkm1 /= big;
 	}
-    }
-    while (t > MACHEP);
+    } while (t > MACHEP);
 
     ans *= exp(-x);
 
   done:
     return (ans);
 }
+
+
+/* Asymptotic expansion for large n, DLMF 8.20(ii) */
+double expn_large_n(int n, double x)
+{
+    int k;
+    double p = n;
+    double lambda = x/p;
+    double multiplier = 1/p/(lambda + 1)/(lambda + 1);
+    double fac = 1;
+    double res = 1; /* A[0] = 1 */
+    double expfac, term;
+
+    expfac = exp(-lambda*p)/(lambda + 1)/p;
+    if (expfac == 0) {
+	mtherr("expn", UNDERFLOW);
+	return 0;
+    }
+
+    /* Do the k = 1 term outside the loop since A[1] = 1 */
+    fac *= multiplier;
+    res += fac;
+
+    for (k = 2; k < nA; k++) {
+	fac *= multiplier;
+	term = fac*polevl(lambda, A[k], Adegs[k]);
+	res += term;
+	if (fabs(term) < MACHEP*fabs(res)) {
+	    break;
+	}
+    }
+
+    return expfac*res;
+}
