@@ -42,7 +42,6 @@
  * ACCURACY:
  *    Relative error (except absolute when |psi| < 1):
  * arithmetic   domain     # trials      peak         rms
- *    DEC       0,30         2500       1.7e-16     2.0e-17
  *    IEEE      0,30        30000       1.3e-15     1.4e-16
  *    IEEE      -30,0       40000       1.5e-15     2.2e-16
  *
@@ -58,7 +57,6 @@
 
 #include "mconf.h"
 
-#ifdef UNK
 static double A[] = {
     8.33333333333333333333E-2,
     -2.10927960927960927961E-2,
@@ -68,43 +66,6 @@ static double A[] = {
     -8.33333333333333333333E-3,
     8.33333333333333333333E-2
 };
-#endif
-
-#ifdef DEC
-static unsigned short A[] = {
-    0037252, 0125252, 0125252, 0125253,
-    0136654, 0145314, 0126312, 0146255,
-    0036370, 0037017, 0101740, 0174076,
-    0136210, 0104210, 0104210, 0104211,
-    0036202, 0004040, 0101010, 0020202,
-    0136410, 0104210, 0104210, 0104211,
-    0037252, 0125252, 0125252, 0125253
-};
-#endif
-
-#ifdef IBMPC
-static unsigned short A[] = {
-    0x5555, 0x5555, 0x5555, 0x3fb5,
-    0x5996, 0x9599, 0x9959, 0xbf95,
-    0x1f08, 0xf07c, 0x07c1, 0x3f7f,
-    0x1111, 0x1111, 0x1111, 0xbf71,
-    0x0410, 0x1041, 0x4104, 0x3f70,
-    0x1111, 0x1111, 0x1111, 0xbf81,
-    0x5555, 0x5555, 0x5555, 0x3fb5
-};
-#endif
-
-#ifdef MIEEE
-static unsigned short A[] = {
-    0x3fb5, 0x5555, 0x5555, 0x5555,
-    0xbf95, 0x9959, 0x9599, 0x5996,
-    0x3f7f, 0x07c1, 0xf07c, 0x1f08,
-    0xbf71, 0x1111, 0x1111, 0x1111,
-    0x3f70, 0x4104, 0x1041, 0x0410,
-    0xbf81, 0x1111, 0x1111, 0x1111,
-    0x3fb5, 0x5555, 0x5555, 0x5555
-};
-#endif
 
 double psi(x)
 double x;
